/*
// TI File $Revision: /main/2 $
// Checkin $Date: February 20, 2009   15:34:01 $
//###########################################################################
//
// FILE:    28035_RAM_lnk.cmd
//
// TITLE:   Linker Command File For 28035 examples that run out of RAM
//
//          This ONLY includes all SARAM blocks on the 28035 device.
//          This does not include flash or OTP.
//
//          Keep in mind that L0,L1,L2, and L3 are protected by the code
//          security module.
//
//          What this means is in most cases you will want to move to
//          another memory map file which has more memory defined.
//
//###########################################################################
// $TI Release: 2803x Internal Release 2 $
// $Release Date: November 11, 2008 $
//###########################################################################
*/

/* ======================================================
// For Code Composer Studio V2.2 and later
// ---------------------------------------
// In addition to this memory linker command file,
// add the header linker command file directly to the project.
// The header linker command file is required to link the
// peripheral structures to the proper locations within
// the memory map.
//
// The header linker files are found in <base>\DSP2803x_headers\cmd
//
// For BIOS applications add:      DSP2803x_Headers_BIOS.cmd
// For nonBIOS applications add:   DSP2803x_Headers_nonBIOS.cmd
========================================================= */

/* ======================================================
// For Code Composer Studio prior to V2.2
// --------------------------------------
// 1) Use one of the following -l statements to include the
// header linker command file in the project. The header linker
// file is required to link the peripheral structures to the proper
// locations within the memory map                                    */

/* Uncomment this line to include file only for non-BIOS applications */
/* -l DSP2803x_Headers_nonBIOS.cmd */

/* Uncomment this line to include file only for BIOS applications */
/* -l DSP2803x_Headers_BIOS.cmd */

/* 2) In your project add the path to <base>\DSP2803x_headers\cmd to the
   library search path under project->build options, linker tab,
   library search path (-i).
/*========================================================= */

/* Define the memory block start/length for the DSP2803x
   PAGE 0 will be used to organize program sections
   PAGE 1 will be used to organize data sections

   Notes:
         Memory blocks on F28035 are uniform (ie same
         physical memory) in both PAGE 0 and PAGE 1.
         That is the same memory region should not be
         defined for both PAGE 0 and PAGE 1.
         Doing so will result in corruption of program
         and/or data.

         L0 block is mirrored - that is it
         can be accessed in high memory or low memory.
         For simplicity only one instance is used in this
         linker file.

         Contiguous SARAM memory blocks can be combined
         if required to create a larger memory block.
*/

MEMORY
{
	/*
	 * [PAGE n : ] name [(attr)] : origin = constant, length = constant[, fill = constant]
	 * PAGE 0 : text
	 * PAGE 1 : data
	 * PAGE 2 : used by boot-rom
	 */
	/*
	 * On chip memory
	 */
/*
	M0_SRAM		(RWXI)	: origin = 0x00000000, length = 0x0400	?*  1K x 16 SARAM	00:0000 - 00:03ff *?
	M1_SRAM		(RWXI)	: origin = 0x00000400, length = 0x0400	?*  1K x 16 SARAM	00:0400 - 00:07ff *?

	L0_SRAM		(RWXI)	: origin = 0x00008000, length = 0x0800	?*  2K x 16 SRAM	00:8000 - 00:87ff *?
	L1_SRAM		(RWXI)	: origin = 0x00008800, length = 0x0400	?*  1K x 16 DP-SRAM	00:8800 - 00:8bff *?
	L2_SRAM		(RWXI)	: origin = 0x00008c00, length = 0x0400	?*  1K x 16 DP-SRAM	00:8c00 - 00:8fff *?
	L3_SRAM		(RWXI)	: origin = 0x00009000, length = 0x1000	?*  4K x 16 DP-SRAM	00:9000 - 00:9fff *?
*/

	PAGE 0 : ENTRY		(RWXI)	: origin = 0x0000, length = 0x0002
	PAGE 2 : _bootrom_			: origin = 0x0002, length = 0x004e
	PAGE 0 : SRAM00		(RWXI)	: origin = 0x0050, length = 0x03b0

	PAGE 1 : SRAM01		(RWXI)	: origin = 0x0400, length = 0x0400

	PAGE 1 : SRAM1		(RWXI)	: origin = 0x8000, length = 0x0c00
	PAGE 0 : SRAM0		(RWXI)	: origin = 0x8c00, length = 0x1400


/*	OTP			(RX)	: origin = 0x003d7800, length = 0x0400	?*  1K x 16 User OneTime */
	FLASH		(RX)	: origin = 0x003e8000, length = 0xfff8	/* 64K x 16 Flash Memory */
	PASSWORD	(R)		: origin = 0x003f7ff8, length = 0x0008	/* 128-bits Password */

/*	L0_SRAM		(RWXI)	: origin = 0x003f8000, length = 0x0800	?*  2K x 16 SRAM (mirror) */

/*	BOOT_ROM	(R)		: origin = 0x003fe000, length = 0x2000	?* BOOT ROM */

/*
 * TI sample
 * PAGE 0 :	?* for text *?
 *
 *	?* BEGIN is used for the "boot to SARAM" bootloader mode   *?
 *
 *	BEGIN      : origin = 0x000000, length = 0x000002
 *
 *	RAMM0      : origin = 0x000050, length = 0x0003B0
 *	RAML0L1    : origin = 0x008000, length = 0x000C00
 *
 *	RESET      : origin = 0x3FFFC0, length = 0x000002
 *	IQTABLES   : origin = 0x3FE000, length = 0x000B50     ?* IQ Math Tables in Boot ROM *?
 *	IQTABLES2  : origin = 0x3FEB50, length = 0x00008C     ?* IQ Math Tables in Boot ROM *?
 *	IQTABLES3  : origin = 0x3FEBDC, length = 0x0000AA	 ?* IQ Math Tables in Boot ROM *?
 *
 *	BOOTROM    : origin = 0x3FF27C, length = 0x000D44
 *
 *
 *PAGE 1 :	?* for data *?
 *
 *	BOOT_RSVD   : origin = 0x000002, length = 0x00004E     ?* Part of M0, BOOT rom will use this for stack *?
 *	RAMM1       : origin = 0x000480, length = 0x000380     ?* on-chip RAM block M1 *?
 *	RAML2       : origin = 0x008C00, length = 0x000400
 *	RAML3       : origin = 0x009000, length = 0x001000
 */

}

	_ramfuncs_loadstart = 0;
	_ramfuncs_loadsize = 0;
	_ramfuncs_runstart = 0;

SECTIONS
{
/*
 * <<< Initialized Sections Created by the Compiler >>>
 * +---------+---------------------------------------------------------------+-------------------------
 * | Name    | Contents                                                      | Restrictions
 * +---------+---------------------------------------------------------------+-------------------------
 * | .cinit  | Tables for explicitly initialized global and static variables | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .const  | Global and static const variables that are explicitly         | Low 64K
 * |         | initialized and contain string literals                       |
 * +---------+---------------------------------------------------------------+-------------------------
 * | .econst | Far constant variables                                        | Anywhere in data
 * +---------+---------------------------------------------------------------+-------------------------
 * | .pinit  | Table of constructors to be called at startup                 | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .switch | Jump tables for large switch statements                       | Program with -mt option
 * |         |                                                               | Data without -mt option
 * +---------+---------------------------------------------------------------+-------------------------
 * | .text   |Executable code and constants                                  | Program
 * +---------+---------------------------------------------------------------+-------------------------
 *
 * <<< Uninitialized Sections Created by the Compiler >>>
 * +----------+------------------------------------+-----------------------
 * | Name     | Contents                           | Restrictions
 * +----------+------------------------------------+-----------------------
 * | .bss     | Global and static variables        | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .ebss    | Far global/static variables        | Anywhere in data
 * +----------+------------------------------------+-----------------------
 * | .stack   | Stack                              | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .sysmem  | Memory for malloc functions (heap) | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .esysmem | Memory for far_malloc functions    | Anywhere in data
 * +----------+------------------------------------+-----------------------
 *
 * <<< Additional Sections by program >>>
 * +-----------+------------------------------------+-----------------------
 * | Name      | Contents                           | Restrictions
 * +-----------+------------------------------------+-----------------------
 * | codestart |
 * +-----------+------------------------------------+-----------------------
 * | ramfuncs  |
 * +-----------+------------------------------------+-----------------------
 * | .reset    |
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 *
 */

	codestart	: > ENTRY
	ramfuncs	: > SRAM00

	.text		: > SRAM0
	.cinit		: > SRAM0
	.pinit		: > SRAM1
	.switch		: > SRAM1

	.reset		: > SRAM0

	.bss		: > SRAM1
	.ebss		: > SRAM1
	.econst		: > SRAM0
	.sysmem		: > SRAM1
	.esysmem	: > SRAM1
	.stack		: > SRAM01

	csm_rsvd	: TYPE = DSECT
	passwords	: TYPE = DSECT

/*
 * TI sample
 *   ?* Setup for "boot to SARAM" mode:
 *      The codestart section (found in DSP28_CodeStartBranch.asm)
 *      re-directs execution to the start of user code.  *?
 *   codestart        : > BEGIN,     PAGE = 0
 *   ramfuncs         : > RAMM0      PAGE = 0
 *   .text            : > RAML0L1,   PAGE = 0
 *   .cinit           : > RAMM0,     PAGE = 0
 *   .pinit           : > RAMM0,     PAGE = 0
 *   .switch          : > RAMM0,     PAGE = 0
 *   .reset           : > RESET,     PAGE = 0, TYPE = DSECT ?* not used, *?
 *
 *   .stack           : > RAMM1,     PAGE = 1
 *   .ebss            : > RAML2,     PAGE = 1
 *   .econst          : > RAML2,     PAGE = 1
 *   .esysmem         : > RAML2,     PAGE = 1
 *
 *   IQmath           : > RAML0L1,   PAGE = 0
 *   IQmathTables     : > IQTABLES,  PAGE = 0, TYPE = NOLOAD
 *
 *  ?* Uncomment the section below if calling the IQNexp() or IQexp()
 *      functions from the IQMath.lib library in order to utilize the
 *      relevant IQ Math table in Boot ROM (This saves space and Boot ROM
 *      is 1 wait-state). If this section is not uncommented, IQmathTables2
 *      will be loaded into other memory (SARAM, Flash, etc.) and will take
 *      up space, but 0 wait-state is possible.
 *   *?
 *   ?*
 *   IQmathTables2    : > IQTABLES2, PAGE = 0, TYPE = NOLOAD
 *   {
 *
 *              IQmath.lib<IQNexpTable.obj> (IQmathTablesRam)
 *
 *   }
 *   *?
 *   ?* Uncomment the section below if calling the IQNasin() or IQasin()
 *      functions from the IQMath.lib library in order to utilize the
 *      relevant IQ Math table in Boot ROM (This saves space and Boot ROM
 *      is 1 wait-state). If this section is not uncommented, IQmathTables2
 *      will be loaded into other memory (SARAM, Flash, etc.) and will take
 *      up space, but 0 wait-state is possible.
 *   *?
 *   ?*
 *   IQmathTables3    : > IQTABLES3, PAGE = 0, TYPE = NOLOAD
 *   {
 *
 *              IQmath.lib<IQNasinTable.obj> (IQmathTablesRam)
 *
 *   }
 *   *?
 */

}

/*
//===========================================================================
// End of file.
//===========================================================================
*/
