/*** ringLib.c - Ring Buffer Manager ***/

/*** Copyright(C) 1989-2011 Kazuya Mimura ***/
/*** Copyrigth(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/16 mimu
              Clean-up & remodify
 * 1989/07/xx mimu
              First Version
 */

/*** NOTE ***/

/*** INCLUDE ***/
#include "ringLib.h"

/*** DEFINE ***/
/*** FUNCTION ***/
/*** WORK ***/

/*******************************************************************************
 *
 */
int ringInit( Ring * pRing )
{
	int retVal;

	retVal = -1;
	if( pRing->pDataTop ) {
		if( pRing->size >= 4 ) {
			pRing->pDataBotom = pRing->pDataTop + pRing->size;
			pRing->pGet = pRing->pDataTop;
			pRing->pSetNext = pRing->pDataTop + 1;

			retVal = 0;
		}
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int ringSet( Ring * pRing, unsigned char * pData, int size )
{
	int retVal;

	retVal = 0;

	for( ; size > 0; ) {
		if( pRing->pSetNext == pRing->pGet ) {
			return retVal;
		}
		*pRing->pSetNext = *pData;
		pRing->pSetNext ++;
		if( pRing->pSetNext == pRing->pDataBotom ) {
			pRing->pSetNext = pRing->pDataTop;
		} 
		pData ++;
		size --;
		retVal ++;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int ringGet( Ring * pRing, unsigned char * pData, int size )
{
	int retVal;
	unsigned char * pNext;

	retVal = 0;

	pNext = pRing->pGet + 1;
	if( pNext == pRing->pDataBotom ) {
		pNext = pRing->pDataTop;
	}

	for( ; size > 0; ) {
		if( pNext == pRing->pSetNext ) {
			return retVal;
		}

		*pData = *pNext;
		pData ++;
		size --;
		pRing->pGet = pNext;
		retVal ++;

		pNext ++;
		if( pNext == pRing->pDataBotom ) {
			pNext = pRing->pDataTop;
		}
	}

	return retVal;
}

/*** end of file : ringLib.c ***/
