
#include "config.h"
#include "sysLib.h"

//#include "DSP2803x_Device.h"
#include "tms320f28035/tms320f28035.h"

extern void InitSysCtrl(void);
extern void InitPieCtrl( void );

#define PIECTRL  (volatile unsigned short *)0x00000CE0 /* PIE, Control Register            */
#define PIEACK   (volatile unsigned short *)0x00000CE1 /* PIE, Acknowledge Register        */
#define PIEIER1  (volatile unsigned short *)0x00000CE2 /* PIE, INT1 Group Enable Register  */
#define PIEIFR1  (volatile unsigned short *)0x00000CE3 /* PIE, INT1 Group Flag Register    */
#define PIEIER2  (volatile unsigned short *)0x00000CE4 /* PIE, INT2 Group Enable Register  */
#define PIEIFR2  (volatile unsigned short *)0x00000CE5 /* PIE, INT2 Group Flag Register    */
#define PIEIER3  (volatile unsigned short *)0x00000CE6 /* PIE, INT3 Group Enable Register  */
#define PIEIFR3  (volatile unsigned short *)0x00000CE7 /* PIE, INT3 Group Flag Register    */
#define PIEIER4  (volatile unsigned short *)0x00000CE8 /* PIE, INT4 Group Enable Register  */
#define PIEIFR4  (volatile unsigned short *)0x00000CE9 /* PIE, INT4 Group Flag Register    */
#define PIEIER5  (volatile unsigned short *)0x00000CEA /* PIE, INT5 Group Enable Register  */
#define PIEIFR5  (volatile unsigned short *)0x00000CEB /* PIE, INT5 Group Flag Register    */
#define PIEIER6  (volatile unsigned short *)0x00000CEC /* PIE, INT6 Group Enable Register  */
#define PIEIFR6  (volatile unsigned short *)0x00000CED /* PIE, INT6 Group Flag Register    */
#define PIEIER7  (volatile unsigned short *)0x00000CEE /* PIE, INT7 Group Enable Register  */
#define PIEIFR7  (volatile unsigned short *)0x00000CEF /* PIE, INT7 Group Flag Register    */
#define PIEIER8  (volatile unsigned short *)0x00000CF0 /* PIE, INT8 Group Enable Register  */
#define PIEIFR8  (volatile unsigned short *)0x00000CF1 /* PIE, INT8 Group Flag Register    */
#define PIEIER9  (volatile unsigned short *)0x00000CF2 /* PIE, INT9 Group Enable Register  */
#define PIEIFR9  (volatile unsigned short *)0x00000CF3 /* PIE, INT9 Group Flag Register    */
#define PIEIER10 (volatile unsigned short *)0x00000CF4 /* PIE, INT10 Group Enable Register */
#define PIEIFR10 (volatile unsigned short *)0x00000CF5 /* PIE, INT10 Group Flag Register   */
#define PIEIER11 (volatile unsigned short *)0x00000CF6 /* PIE, INT11 Group Enable Register */
#define PIEIFR11 (volatile unsigned short *)0x00000CF7 /* PIE, INT11 Group Flag Register   */
#define PIEIER12 (volatile unsigned short *)0x00000CF8 /* PIE, INT12 Group Enable Register */
#define PIEIFR12 (volatile unsigned short *)0x00000CF9 /* PIE, INT12 Group Flag Register   */

#define	PIE_REG_END (volatile unsigned short *)0x00000CFA

static interrupt void intDummyFunc( void ){ for(;;){;} }
static void intFunc( IntID id );

#define	intFuncName( _id )	intFunc_##_id
#define	_IntFunc( _id ) interrupt void intFuncName(_id)( ) { intFunc(_id); }

#ifdef	SUPPORT_CPU_TIMER_0
_IntFunc(IntID_Timer0)
#endif
#ifdef	SUPPORT_CPU_TIMER_1
_IntFunc(IntID_Timer1)
#endif
#ifdef	SUPPORT_CPU_TIMER_2
_IntFunc(IntID_Timer2)
#endif

#ifdef	SUPPORT_SCI_A
_IntFunc(IntID_SciA_RX)
_IntFunc(IntID_SciA_TX)
#endif

static const struct int_cfg_t {
	unsigned short ierVal;
	volatile unsigned short * pPieieReg;
	unsigned short pieieVal;
	unsigned short pieackVal;

	unsigned long vect;
	void (*pIntFunc)( );

} intCfgTbl[IntID_Num] =
{
	/* ierVal, pPieieReg, pieieVal, pieackVal, vect,       pIntFunc */
#ifdef	SUPPORT_CPU_TIMER_0
	{  0x0001, PIEIER1,   0x40,     0x0001,    0x00000D4C, intFuncName(IntID_Timer0) },		/* cpu timer 0 (int 1.7) */
#endif
#ifdef	SUPPORT_CPU_TIMER_1
	{  0x0001, PIEIER1,   0x08,     0x0001,    0x00000d46, intFuncName(IntID_Timer1) },		/* cpu timer 1 (int 1.4) */
#endif
#ifdef	SUPPORT_CPU_TIMER_2
	{  0x0001, PIEIER1,   0x10,     0x0001,    0x00000d48, intFuncName(IntID_Timer2) },		/* cpu timer 2 (int 1.5) */
#endif

#ifdef	SUPPORT_SCI_A
	{  0x0100, PIEIER9,   0x01,     0x0100,    0x00000dc0, intFuncName(IntID_SciA_RX) },		/* sci A Rx (int 9.1) */
	{  0x0100, PIEIER9,   0x02,     0x0100,    0x00000dc2, intFuncName(IntID_SciA_TX) },		/* sci A Tx (int 9.2) */
#endif

};

static struct int_mng_t {
	void (*pDrvFunc)( void * arg );
	void * arg;
} intMngTbl[IntID_Num];

extern unsigned int ramfuncs_loadstart;
extern unsigned int ramfuncs_loadsize;
extern unsigned int ramfuncs_runstart;

/*******************************************************************************
 *
 */
int sysInit( void )
{
	/*
	 * Step 0. 'ramfunc' section initialize
	 */
	if( ramfuncs_runstart != ramfuncs_loadstart ) {
		memcpy( & ramfuncs_runstart,
		        & ramfuncs_loadstart,
		        (int)& ramfuncs_runstart );
	}

	/*
	 * Step 1. Initialize System Control:
	 */
	InitSysCtrl();		/* PLL, WatchDog, enable Peripheral Clocks
						 * This example function is found in the DSP2803x_SysCtrl.c file.
						 */

	/*
	 * Step 2. Initalize GPIO:
	 */
/*	 InitGpio();		?*
						 * This example function is found in the DSP2803x_Gpio.c file and
						 * illustrates how to set the GPIO to it's default state.
						 */

	/*
	 * Step 3. Clear all interrupts and initialize PIE vector table:
	 */
	DINT;				/* Disable CPU interrupts */

//	InitPieCtrl();		/* Initialize the PIE control registers to their default state.
//						 * The default state is all PIE interrupts disabled and flags
//						 * are cleared.
//						 * This function is found in the DSP2803x_PieCtrl.c file.
//						 */
	// Disable the PIE
//	PieCtrlRegs.PIECTRL.bit.ENPIE = 0;
	*PIECTRL &= ~1;

//	// Clear all PIEIER registers:
//	PieCtrlRegs.PIEIER1.all = 0;
//	PieCtrlRegs.PIEIER2.all = 0;
//	PieCtrlRegs.PIEIER3.all = 0;
//	PieCtrlRegs.PIEIER4.all = 0;
//	PieCtrlRegs.PIEIER5.all = 0;
//	PieCtrlRegs.PIEIER6.all = 0;
//	PieCtrlRegs.PIEIER7.all = 0;
//	PieCtrlRegs.PIEIER8.all = 0;
//	PieCtrlRegs.PIEIER9.all = 0;
//	PieCtrlRegs.PIEIER10.all = 0;
//	PieCtrlRegs.PIEIER11.all = 0;
//	PieCtrlRegs.PIEIER12.all = 0;
//
//	// Clear all PIEIFR registers:
//	PieCtrlRegs.PIEIFR1.all = 0;
//	PieCtrlRegs.PIEIFR2.all = 0;
//	PieCtrlRegs.PIEIFR3.all = 0;
//	PieCtrlRegs.PIEIFR4.all = 0;
//	PieCtrlRegs.PIEIFR5.all = 0;
//	PieCtrlRegs.PIEIFR6.all = 0;
//	PieCtrlRegs.PIEIFR7.all = 0;
//	PieCtrlRegs.PIEIFR8.all = 0;
//	PieCtrlRegs.PIEIFR9.all = 0;
//	PieCtrlRegs.PIEIFR10.all = 0;
//	PieCtrlRegs.PIEIFR11.all = 0;
//	PieCtrlRegs.PIEIFR12.all = 0;
	{
		volatile unsigned short * pReg;

		for( pReg=PIEIER1; pReg!=PIE_REG_END; pReg++ ) {
			*pReg = 0;
		}
	}

	IER = 0x0000;		/* Disable CPU interrupts */
	IFR = 0x0000;		/* and clear all CPU interrupt flags: */

//	InitPieVectTable();	/* Initialize the PIE vector table with pointers to the shell Interrupt
//						 * Service Routines (ISR).
//						 * This will populate the entire table, even if the interrupt
//						 * is not used in this example.  This is useful for debug purposes.
//						 * The shell ISR routines are found in DSP2803x_DefaultIsr.c.
//						 * This function is found in DSP2803x_PieVect.c.
//						 */
	{
		int i;
//		unsigned long * p = (unsigned long *)&PieVectTable;
		unsigned long * p = (unsigned long *)PIE_VECT_ADRS;

		/* Do not write over first 3 32-bit locations (these locations are
		 * initialized by Boot ROM with boot variables)
		 */
		p += 3;

		EALLOW;
		for( i=0; i<125; i++, p++ ) {
			*p = (unsigned long)intDummyFunc;
		}
		EDIS;

//		PieCtrlRegs.PIECTRL.bit.ENPIE = 1;
		*PIECTRL |= 1;
	}

// Enable global Interrupts and higher priority real-time debug events:
   EINT;   // Enable Global interrupt INTM
   ERTM;   // Enable Global realtime interrupt DBGM

   return 0;
}

/*******************************************************************************
 *
 */
//int sysIntLock(void);
//     move to sysLibasm.asm
// {
// }
//

/*******************************************************************************
 *
 */
void sysIntUnlock( int mask )
{
	if( ! mask ) {
		EINT;
	}
}

/*******************************************************************************
 *
 */
int sysIntConnect( IntID id, void (*pFunc)(void *), void * arg )
{
	int retVal = -1;
	int	intLockVal;

	if( id < IntID_Num ) {

		intLockVal = sysIntLock();

		EALLOW;
		*(unsigned long *)intCfgTbl[id].vect = (unsigned long)intCfgTbl[id].pIntFunc;
		EDIS;

		intMngTbl[id].pDrvFunc = pFunc;
		intMngTbl[id].arg      = arg;

		sysIntUnlock( intLockVal );

		retVal = 0;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int sysIntEnable( IntID id )
{
	int retVal = -1;
	int intLockVal;

	if( id <IntID_Num ) {
		intLockVal = sysIntLock();

		IER |= intCfgTbl[id].ierVal;
		* intCfgTbl[id].pPieieReg |= intCfgTbl[id].pieieVal;

		sysIntUnlock( intLockVal );

		retVal = 0;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int sysIntDisable( IntID id )
{
	int retVal = -1;
	int intLockVal;

	if( id <IntID_Num ) {
		intLockVal = sysIntLock();

//		IER &= ~intCfgTbl[id].ierVal;
		* intCfgTbl[id].pPieieReg &= ~intCfgTbl[id].pieieVal;

		sysIntUnlock( intLockVal );

		retVal = 0;
	}

	return retVal;
}

/*******************************************************************************
 *
 */
//static interrupt intDummyFunc( void )
//{
//	for( ; ; ) {
//		;
//	}
//}

/*******************************************************************************
 *
 */
static void intFunc( IntID id )
{
	intMngTbl[id].pDrvFunc( intMngTbl[id].arg );

	*PIEACK = intCfgTbl[id].pieackVal;
}

/*******************************************************************************
 *
 */
void sysDummyFunc()
{
}

/*******************************************************************************
 *
 */
void sysHalt( const char * msg )
{
	for( ; ; ) {
		;
	}
}
