/*** loader.c - ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

#include <string.h>

#include "config.h"
#include "sysLib.h"

#include "tms320f28035/timerDrv.h"
#include "tms320f28035/sciDrv.h"
#include "tms320f28035/flashDrv.h"

static int wait_sync(void);

//static void timer_isr( void * arg );
static int cmd_rx( int limit );
static void result_tx( const char * );

//static void send_ack(void);
static const char * ack = "a\x11\x0d\x0a";
#define	send_ack()	result_tx(ack)
//static void send_nak( void );
static const char * nak = "n\x11\x0d\x0a";
#define	send_nak()	result_tx(nak)
//static void send_busy( void );
static const char * busy = "b\x13\x0d\x0a";
#define	send_busy()	result_tx(busy);
//static void send_version( void );
static const char * version = "a\x11Ver.0.00\x0d\x0a";
#define	send_version()	result_tx(version);

static int prog_init( void );
static int prog_data( void );
static int prog_end( void );

static int a2hex( unsigned char c );

//static volatile unsigned int rxTimer;
static int rxDataLen;
static unsigned char rxData[128];

/*******************************************************************************
 *
 */
void loader( void )
{
	int bDone;

//	timerDrvInit( 60000000 );
//	timerConnect( 0, timer_isr, 0 );
//	timerConfig( 0, 1000 );
//	timerRun( 0 );

	sciDrvInit(  60000000/4 );

//	sciCtrl( 9600, 8, 'N', 1 );
	sciCtrl( 115200, 8, 'N', 1 );

	sciOpen();

	if( wait_sync() ) {
		return;
	}


	for( bDone=0; bDone==0; ) {
		if( cmd_rx(0) ) {
			switch( rxData[0] ) {
			case 's':
				send_ack();
				break;
			case 'v':
				send_version();
				break;
			case 'w':
				if( prog_init() ) {
					send_nak();
				} else {
					send_ack();
				}
				break;
			case 'S':
				if( prog_data() ) {
					send_nak();
				} else {
					send_ack();
				}
				break;
			case 'e':
				if( prog_end() ) {
					send_nak();
				} else {
					send_ack();
					bDone = 1;
				}
				break;
			default :
				send_nak();
				break;
			}
		}
	}

}

/*******************************************************************************
 *
 */
static int wait_sync( void )
{
	int retry;
	int bRetry;

	for( retry=0, bRetry=1; ( (bRetry) && (retry<15000) ); retry++ ) {
		if( cmd_rx(100) ) {
			if( rxData[0] == 's' ) {
				send_ack();
				bRetry = 0;
			}
		}
	}

	if( bRetry ) {
		return -1;
	}

	return 0;
}

/*******************************************************************************
 *
 */
//static void timer_isr( void * arg )
//{
//	if( rxTimer > 0 ) {
//		rxTimer --;
//	}
//}

/*******************************************************************************
 *
 */
static int cmd_rx( int limit )
{
	int rxLen;
	int bRetry;
	int count;

	rxDataLen = 0;
	count = 0;

	for( bRetry=1; bRetry; ) {
		rxLen = sciRx( &rxData[rxDataLen], 1, 0 );
		if( rxLen == 1 ) {
			switch(rxData[rxDataLen]) {
			case 0x0d:
				break;
			case 0x0a:
				rxData[rxDataLen] = '\0';
				bRetry = 0;
				break;
			default:
				rxDataLen++;
				if( rxDataLen >= 128 ) {
					rxDataLen = 0;
				}
				break;
			}
		} else {
			if( limit ) {
				count++;
				if( count >= limit ) {
					bRetry = 0;
				}
			}
		}
	}

	if( bRetry ) {
		return 0;
	}

	return 1;
}

/*******************************************************************************
 *
 */
static void result_tx( const char * msg )
{
	int len;

	len = strlen( msg );
	sciTx( (unsigned char *)msg, len, len );
}

/******************************************************************************/
/* FLASH                                                                      */
/******************************************************************************/
//static const int setTop = FLASH_PROGRAM_SEC_NO;
//static const int secNum = FLASH_PROGRAM_SEC_NUM;
static unsigned long topAdrs;

static unsigned long flash_size;
/*******************************************************************************
 *
 */
static int prog_init( void )
{
	int secNo;

	send_busy();

	topAdrs = 0;
	flash_size = 0;
	for( secNo=FLASH_PROGRAM_SEC_NO; secNo<(FLASH_PROGRAM_SEC_NO+FLASH_PROGRAM_SEC_NUM); secNo ++ ) {
		flash_size += flashSecSize( secNo );
	}

	flash_size <<= 1;	// to Byte address

	return flashErase( FLASH_PROGRAM_SEC_NO, FLASH_PROGRAM_SEC_NUM );
}

/*******************************************************************************
 *
 */
static int prog_data( void )
{
	unsigned char sum;
	unsigned char * p;
	int adrsLen;
	int recodeLen;
	unsigned long adrs;
	int hex;
	int dataLen;
	static unsigned short flashData[64];

	if( rxData[0] != 'S' ) {
		return -1;
	}

	p = &rxData[1];

	/*
	 * recode type
	 */
	switch( *p++ ) {
	case '0':
		return 0;
	case '1':	// address len = 4
		adrsLen = 4;
		break;
	case '2':	// address len = 6
		adrsLen = 6;
		break;
	case '3':	// address len = 8
		adrsLen = 8;
		break;
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		return 0;
	default:
		return -1;
	}

	sum = 0;

	/*
	 * recode length
	 */
	recodeLen = a2hex( *p++ );
	recodeLen <<= 4;

	recodeLen += a2hex( *p++ );

	sum += recodeLen;
	if( recodeLen > 0xff ) {
		return -1;
	}

	recodeLen <<= 1;

	/*
	 * address
	 */
	adrs = 0;
	for( ; adrsLen>0; adrsLen-- ) {
		hex = a2hex( *p++ );
		if( hex > 0x0f ) {
			return -1;
		}
		adrs <<= 4;
		adrs += (unsigned long)hex;
		recodeLen --;
		if( (recodeLen & 1) == 0 ) {
			sum += (unsigned char)(adrs & 0xff);
		}
	}

	if( adrs & 1 ) {
		return -1;
	}

	if( topAdrs == 0 ) {
		topAdrs = adrs;
	}

	if( adrs >= (topAdrs + flash_size) ) {
		return -1;
	}

	/*
	 * data
	 */
#if 1
	{
		int count4;
		dataLen = 0;
		recodeLen -=2;
		count4 = 0;

		for( ; recodeLen; recodeLen -- ){
			hex = a2hex( *p++ ) ;
			if( hex > 0x0f ) {
				return -1;
			}
			switch( count4 ) {
			case 0:
				flashData[dataLen] = (hex<<12) & 0xf000;
				count4 ++;
				break;
			case 1:
				flashData[dataLen] |= (hex<<8) & 0x0f00;
				sum += (unsigned char)(flashData[dataLen] >> 8);
				count4++;
				break;
			case 2:
				flashData[dataLen] |= (hex<<4) & 0x00f0;
				count4++;
				break;
			case 3:
				flashData[dataLen] |= hex & 0x000f;
				sum += (unsigned char)(flashData[dataLen]);
				count4 = 0;
				dataLen++;
				break;
			}
		}
	}
#else
	{
		unsigned short data;
		int datast;

		dataLen = 0;
		data = 0;
		recodeLen -= 2;
		if( recodeLen & 0x03 ) {
			return -1;
		}
		for( ; recodeLen; recodeLen -= 4) {
			for( datast=0; datast<4; ) {
				hex = a2hex( *p ++ );
				if( hex > 0x0f ) {
					return -1;
				}
				data <<= 4;
				data |= (unsigned short)hex;
				datast ++;
				if( (datast & 1) == 0 ) {
					sum += (unsigned char)(data & 0xff);
				}
			}
			flashData[dataLen] = data;
			dataLen ++;
		}
	}
#endif

	/*
	 * sum
	 */
	sum += (unsigned char)(a2hex(*p++)<<4);
	sum += (unsigned char)(a2hex(*p++));
	sum &= 0xff;
	if( sum != 0xff ) {
		return -1;
	}

	send_busy();
	flashWrite( FLASH_PROGRAM_SEC_NO, (adrs - topAdrs)/2, &flashData[0], dataLen );
	return 0;
}

/*******************************************************************************
 *
 */
static int prog_end( void )
{
	return 0;
}

/*******************************************************************************
 *
 */
static int a2hex( unsigned char c )
{
	if( ('0'<=c) && (c<='9') ) {
		return (unsigned int)(c-'0');
	} else if( ('A'<=c) && (c<='F') ) {
		return (unsigned int)( c-'A'+0x0a);
	} else if( ('a'<=c) && (c<='f') ) {
		return (unsigned int)( c-'a'+0x0a);
	}

	return 0x7fff;
}
/*** end of file : loader.c ***/
