L 1 "../src/bsp/tms320f28035/dspCtrl.c"
N/*** dspCtrl.c - TMS320F28035 DSP control (clock, PLL, Power, watchdog) ***/
N
N/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/19 mimu
N              First Version
N */
N
N/*** NOTE ***
N * XCLK         0x7010 1 XCLKOUT/XCLKIN Control
N * PLLSTS       0x7011 1 PLL Status
N * CLKCTL       0x7012 1 Clock Control
N * PLLLOCKPRD   0x7013 1 PLL Lock Period
N * INTOSC1TRIM  0x7014 1 Internal Oscillator 1 Trim
N * INTOSC2TRIM  0x7016 1 Internal Oscillator 2 Trim
N *
N * LOSPCP       0x701B 1 Low-Speed Peripheral Clock Pre-Scaler
N * PCLKCR0      0x701C 1 Peripheral Clock Control 0
N * PCLKCR1      0x701D 1 Peripheral Clock Control 1
N * LPMCR0       0x701E 1 Low Power Mode Control 0
N * PCLKCR3      0x7020 1 Peripheral Clock Control 3
N * PLLCR        0x7021 1 PLL Control
N *
N * SCSR         0x7022 1 System Control & Status
N * WDCNTR       0x7023 1 Watchdog Counter
N *
N * WDKEY        0x7025 1 Watchdog Reset Key
N *
N * WDCR         0x7029 1 Watchdog Control
N *
N * BORCFG       0x0985 1 BOR Configuration
N */
N
N/*** INCLUDE ***/
N#include "config.h"
L 1 "D:/project/fpalette/ccs4/fpalette/ide_led/src/bsp/config.h" 1
N/*** config.h ***/
N
N
N#ifndef	INC_config_H
N#define	INC_config_H
N
N/******************************************************************************/
N/* NvRam Configuration                                                        */
N/******************************************************************************/
N#define	FLASH_PROGRAM_SEC_NO		0
N#define	FLASH_PROGRAM_SEC_NUM		6
N
N#define	FLASH_BOOTLOADER_SEC_NO		(FLASH_PROGRAM_SEC_NO + FLASH_PROGRAM_SEC_NUM)
N#define	FLASH_BOOTLOADER_SEC_NUM	2
N
N/******************************************************************************/
N/* Driver Configuration                                                       */
N/******************************************************************************/
N
N/*** CPU TIMER ***/
N#define	SUPPORT_CPU_TIMER_0		/* !! ATTATION !! system need timer0 */
N//#define	SUPPORT_CPU_TIMER_1
N//#define	SUPPORT_CPU_TIMER_2
N
N/*** SCI ***/
N#define	SUPPORT_SCI_A
N
N/*** Flash Mem. ***/
N//#define	SUPPORT_FLASH
N
N/******************************************************************************/
N/* GPIO map                                                                   */
N/******************************************************************************/
N#define	GPIO_NO_CPU_LED	39
N
N//#define	GPIO_NO_SCIRXDA	7
N#define	GPIO_NO_SCIRXDA	28
N
N//#define	GPIO_NO_SCITXDA	12
N#define	GPIO_NO_SCITXDA	29
N
N
N/*-----------------------------------------------------------------------------
N      Specify the PLL control register (PLLCR) and divide select (DIVSEL) value.
N-----------------------------------------------------------------------------*/
N
N//#define DSP28_DIVSEL   0 // Enable /4 for SYSCLKOUT
N//#define DSP28_DIVSEL   1 // Disable /4 for SYSCKOUT
N#define DSP28_DIVSEL   2 // Enable /2 for SYSCLKOUT
N//#define DSP28_DIVSEL   3 // Enable /1 for SYSCLKOUT
N
N
N#define DSP28_PLLCR   12    // Uncomment for 60 MHz devices [60 MHz = (10MHz * 12)/2]
N//#define DSP28_PLLCR   11
N//#define DSP28_PLLCR   10
N//#define DSP28_PLLCR    9
N//#define DSP28_PLLCR    8      // Uncomment for 40 MHz devices [40 MHz = (10MHz * 8)/2]
N//#define DSP28_PLLCR    7
N//#define DSP28_PLLCR    6
N//#define DSP28_PLLCR    5
N//#define DSP28_PLLCR    4
N//#define DSP28_PLLCR    3
N//#define DSP28_PLLCR    2
N//#define DSP28_PLLCR    1
N//#define DSP28_PLLCR    0  // PLL is bypassed in this mode
N//----------------------------------------------------------------------------
N
N/*-----------------------------------------------------------------------------
N      Specify the clock rate of the CPU (SYSCLKOUT) in nS.
N
N      Take into account the input clock frequency and the PLL multiplier
N      selected in step 1.
N
N      Use one of the values provided, or define your own.
N      The trailing L is required tells the compiler to treat
N      the number as a 64-bit value.
N
N      Only one statement should be uncommented.
N
N      Example:   60 MHz devices:
N                 CLKIN is a 10 MHz crystal or internal 10 MHz oscillator
N
N                 In step 1 the user specified PLLCR = 0xC for a
N                 60 MHz CPU clock (SYSCLKOUT = 60 MHz).
N
N                 In this case, the CPU_RATE will be 16.667L
N                 Uncomment the line: #define CPU_RATE 16.667L
N
N-----------------------------------------------------------------------------*/
N
N#define CPU_RATE   16.667L   // for a 60MHz CPU clock speed (SYSCLKOUT)
N//#define CPU_RATE   20.000L   // for a 50MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   25.000L   // for a 40MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   33.333L   // for a 30MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   41.667L   // for a 24MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   50.000L   // for a 20MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   66.667L   // for a 15MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE  100.000L   // for a 10MHz CPU clock speed  (SYSCLKOUT)
N
N#if 0
S//----------------------------------------------------------------------------
S
S// The following pointer to a function call calibrates the ADC and internal oscillators
S#define Device_cal (void   (*)(void))0x3D7C80
S
S//---------------------------------------------------------------------------
S// Include Example Header Files:
S//
S
S#include "DSP2803x_GlobalPrototypes.h"         // Prototypes for global functions within the
S                                               // .c files.
S#include "DSP2803x_EPwm_defines.h"             // Macros used for PWM examples.
S#include "DSP2803x_I2C_defines.h"              // Macros used for I2C examples.
S#include "DSP2803x_Cla_Defines.h"              // Macros used for CLA examples.
S
S
S#define   PARTNO_28030PAG   0xAA
S#define   PARTNO_28030PN    0xAB
S
S#define   PARTNO_28031PAG   0xAE
S#define   PARTNO_28031PN    0xAF
S
S#define   PARTNO_28032PAG   0xB2
S#define   PARTNO_28032PN    0xB3
S
S#define   PARTNO_28033PAG   0xB6
S#define   PARTNO_28033PN    0xB7
S
S#define   PARTNO_28034PAG   0xBA
S#define   PARTNO_28034PN    0xBB
S
S#define   PARTNO_28035PAG   0xBE
S#define   PARTNO_28035PN    0xBF
S
S// Include files not used with DSP/BIOS
S#ifndef DSP28_BIOS
S#include "DSP2803x_DefaultISR.h"
S#endif
N#endif
N
Nextern void DSP28x_usDelay(unsigned long Count);
N// DO NOT MODIFY THIS LINE.
N#define DELAY_US(A)  DSP28x_usDelay(((((long double) A * 1000.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)
N
N#endif	/* INC_config_H */
N
N/*** end of file : config.h ***/
L 37 "../src/bsp/tms320f28035/dspCtrl.c" 2
N
N#include "tms320f28035.h"
L 1 "..\src\bsp\tms320f28035\tms320f28035.h" 1
N/*** tms320f28035.h - TI TMS320F28035 ***/
N
N/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/22 mimu
N              First Version
N */
N
N#ifndef	INC_tms320f28035_H
N#define	INC_tms320f28035_H
N
N
N/*** Core control ***/
Nextern cregister volatile unsigned int IFR;
Nextern cregister volatile unsigned int IER;
N
N#define  EINT   asm(" clrc INTM")
N#define  DINT   asm(" setc INTM")
N#define  ERTM   asm(" clrc DBGM")
N#define  DRTM   asm(" setc DBGM")
N#define  EALLOW asm(" EALLOW")
N#define  EDIS   asm(" EDIS")
N#define  ESTOP0 asm(" ESTOP0")
N
N
N/*** Memory Map ***/
N
N#define	PIE_VECT_ADRS	0x000d00
N
N
N#endif	/* INC_tms320f28035_H */
N
N/*** end of file : tms320f28035.h ***/
L 39 "../src/bsp/tms320f28035/dspCtrl.c" 2
N#include "dspCtrl.h"
L 1 "..\src\bsp\tms320f28035\dspCtrl.h" 1
N/*** clkCtrl.h - TMS320F28035 Clock and PLL control ***/
N
N/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/19 mimu
N              First Version
N */
N
N#include "config.h"
N
N#ifndef	PCLKCR0_INIT
N#define	PCLKCR0_INIT	0x0000
N#endif
N
N#ifndef	PCLKCR1_INIT
N#define	PCLKCR1_INIT	0x0000
N#endif
N
N#ifndef	PCLKCR3_INIT
N#define	PCLKCR3_INIT	0x2700
N#endif
N
Nint clk_init( void );
N
N#define	_CLK_ID_( _reg, _bit )	((_reg)<<8) | (_bit))
N#define	_CLK_ID_2_REG_NO(_id)	(((_id)>>8)&0xf)
N#define	_CLK_ID_2_BIT(_id)		((_id)&0xff)
N
N#define	CLKID_ECAN_A	_CLK_ID(0,14)	/* ECAN-A clock								*/
N#define	CLKID_SCI_A		_CLK_ID(0,10)	/* SCI-A clock								*/
N#define	CLKID_SPI_B		_CLK_ID(0, 9)	/* SPI-B clock								*/
N#define	CLKID_SPI_A		_CLK_ID(0, 8)	/* SPI-A clock								*/
N#define	CLKID_I2C		_CLK_ID(0, 4)	/* I2C clock								*/
N#define	CLKID_ADC		_CLK_ID(0, 3)	/* ADC clock								*/
N#define	CLKID_TBCLK		_CLK_ID(0, 2)	/* ePWM Module Time Base Clock (TBCLK) Sync	*/
N#define	CLKID_LIN_A		_CLK_ID(0, 1)	/* LIN-A clock								*/
N#define	CLKID_HRPWM		_CLK_ID(0, 0)	/* HRPWM clock								*/
N
N#define	CLKID_EQEP1		_CLK_ID(1,15)	/* eQEP1 clock								*/
N#define	CLKID_ECAP1		_CLK_ID(1,14)	/* eCAP1 clock								*/
N#define	CLKID_EPWM7		_CLK_ID(1, 6)	/* ePWM7 clock								*/
N#define	CLKID_EPWM6		_CLK_ID(1, 5)	/* ePWM6 clock								*/
N#define	CLKID_EPWM5		_CLK_ID(1, 4)	/* ePWM5 clock								*/
N#define	CLKID_EPWM4		_CLK_ID(1, 3)	/* ePWM4 clock								*/
N#define	CLKID_EPWM3		_CLK_ID(1, 2)	/* ePWM3 clock								*/
N#define	CLKID_EPWM2		_CLK_ID(1, 1)	/* ePWM2 clock								*/
N#define	CLKID_EPWM1		_CLK_ID(1, 0)	/* ePWM1 clock								*/
N
N#define	CLKID_CLA		_CLK_ID(3,14)	/* CLA module clock							*/
N#define	CLKID_GPIOIN	_CLK_ID(3,13)	/* GPIO Input Clock							*/
N#define	CLKID_CPUTIMER2	_CLK_ID(3,10)	/* CPU Timer 2 Clock						*/
N#define	CLKID_CPUTIMER1	_CLK_ID(3, 9)	/* CPU Timer 1 Clock						*/
N#define	CLKID_CPUTIMER0	_CLK_ID(3, 0)	/* CPU Timer 0 Clock						*/
N#define	CLKID_COMP3		_CLK_ID(3, 2)	/* Comparator3 clock						*/
N#define	CLKID_COMP2		_CLK_ID(3, 1)	/* Comparator2 clock						*/
N#define	CLKID_COMP1)	_CLK_ID(3, 0)	/* Comparator1 clock						*/
N
Nint clk_ctrl( int clkID, int bEnable );
N
N
N#define	CLK_LSPCLK_1		0	/* Low speed clock = SYSCLKOUT/1  */
N#define	CLK_LSPCLK_2		1	/* Low speed clock = SYSCLKOUT/2  */
N#define	CLK_LSPCLK_4		2	/* Low speed clock = SYSCLKOUT/4  */
N#define	CLK_LSPCLK_6		3	/* Low speed clock = SYSCLKOUT/6  */
N#define	CLK_LSPCLK_8		4	/* Low speed clock = SYSCLKOUT/8  */
N#define	CLK_LSPCLK_10		5	/* Low speed clock = SYSCLKOUT/10 */
N#define	CLK_LSPCLK_12		6	/* Low speed clock = SYSCLKOUT/12 */
N#define	CLK_LSPCLK_14		7	/* Low speed clock = SYSCLKOUT/14 */
N
Nint clk_lsp( int val );
N
N#define	Device_cal	(void(*)(void))0x3d7c80
N
N/*** end of file : clkCtrl.h ***/
L 40 "../src/bsp/tms320f28035/dspCtrl.c" 2
N
N/*** DEFINE ***/
N/*** XCLK ***/
N#define	XCLK		(volatile unsigned short *)0x7010	/* XCLKOUT/XCLKIN Control					*/
N#define	XCLK_XCLKINSEL	(1<<6)	/* XCLKIN Source Select */
N#define	XCLK_XCLKOUTDIV_4	(0)	/* XCLKOUT = SYSCLKOUT/4 */
N#define	XCLK_XCLKOUTDIV_2	(1)	/* XCLKOUT = SYSCLKOUT/2 */
N#define	XCLK_XCLKOUTDIV_1	(2)	/* XCLKOUT = SYSCLKOUT/1 */
N#define	XCLK_XCLKOUTDIV_Off	(3)	/* XCLKOUT = Off         */
N
N/*** PLLSTS ***/
N#define	PLLSTS		(volatile unsigned short *)0x7011	/* PLL Status								*/
N#define	PLLSTS_NORMRDYE	(1<<15)	/* NORMRDY Enable */
N
N#define	PLLSTS_DIVSEL_MASK	(3<<7)
N#define	PLLSTS_DIVSEL_4		(0<<7)	/* Select Divide By 4 for CLKIN */
N#define	PLLSTS_DIVSEL_2		(2<<7)	/* Select Divide By 2 for CLKIN */
N#define	PLLSTS_DIVSEL_1		(3<<7)	/* Select Divide By 1 for CLKIN. (This mode can be used only when PLL is off or bypassed.) */
N#define	PLLSTS_DIVSEL(_v)	(((_v)&0x03) <<7)
N#define	PLLSTS_MCLKOFF		(1<<6)	/* Missing clock-detect off */
N#define	PLLSTS_OSCOFF		(1<<5)	/* Oscillator Clock Off */
N#define	PLLSTS_MCLKCLR		(1<<4)	/* Missing Clock Clear */
N#define	PLLSTS_MCLKSTS		(1<<3)	/* Missing Clock Status */
N#define	PLLSTS_PLLOFF		(1<<2)	/* PLL Off */
N#define	PLLSTS_PLLLOCKS		(1<<0)	/* PLL Lock Status */
N
N/*** CLKCTL ***/
N#define	CLKCTL		(volatile unsigned short *)0x7012	/* Clock Control							*/
N#define	CLKCTL_NMIRESETSEL	(1<<15)	/* NMI Reset Select */
N#define	CLKCTL_XTALOSCOFF	(1<<14)	/* Crystal Oscillator Off */
N#define	CLKCTL_XCLKINOFF	(1<<13)	/* XCLKIN Off */
N#define	CLKCTL_WDHALTI	(1<<12)	/* Watchdog HALT Mode Ignore */
N#define	CLKCTL_INTOSC2HALTI	(1<<11)	/* Internal Oscillator 2 HALT Mode Ignore */
N#define	CLKCTL_INTOSC2OFF	(1<<10)	/* Internal Oscillator 2 Off */
N#define	CLKCTL_INTOSC1HALTI	(1<<9)	/* Internal Oscillator 1 HALT Mode Ignore */
N#define	CLKCTL_INTOSC1OFF	(1<<8)	/* Internal Oscillator 1 Off */
N
N#define	CLKCTL_TMR2CLKPRESCALE_1	(0<<5)	/* CPU Timer 2 Clock Pre-Scale Value */
N#define	CLKCTL_TMR2CLKPRESCALE_2	(1<<5)
N#define	CLKCTL_TMR2CLKPRESCALE_4	(2<<5)
N#define	CLKCTL_TMR2CLKPRESCALE_8	(3<<5)
N#define	CLKCTL_TMR2CLKPRESCALE_16	(4<<5)
N												/* CPU Timer 2 Clock Source Select */
N#define	CLKCTL_TMR2CLKSRCSEL_SYSCLKOUT	(0<<3)	/* SYSCLKOUT Selected (default on reset, pre-scaler is bypassed) */
N#define	CLKCTL_TMR2CLKSRCSEL_EXT_OSC	(1<<3)	/* External Oscillator Selected (at XOR output) */
N#define	CLKCTL_TMR2CLKSRCSEL_OSC1		(2<<3)	/* Internal Oscillator 1 Selected */
N#define	CLKCTL_TMR2CLKSRCSEL_OSC2		(3<<3)	/* Internal Oscillator 2 Selected. */
N#define	CLKCTL_WDCLKSRCSEL		(1<<2)	/* Watchdog Clock Source Select Bit */
N#define	CLKCTL_OSCCLKSRC2SEL	(1<<1)	/* Oscillator 2 Clock Source Select Bit */
N#define	CLKCTL_OSCCLKSRCSEL		(1<<0)	/* Oscillator Clock Source Select Bit */
N
N
N
N#define	PLLLOCKPRD	(volatile unsigned short *)0x7013	/* PLL Lock Period							*/
N
N/*** INTOSCxTRIM ***/
N#define	INTOSC1TRIM	(volatile unsigned short *)0x7014	/* Internal Oscillator 1 Trim				*/
N#define	INTOSC2TRIM	(volatile unsigned short *)0x7016	/* Internal Oscillator 2 Trim				*/
N
N#define	INTOSCxTRIM_FINETRIM(_v)	(((_v)&0x3f)<<9)
N#define	INTOSCxCOARSETRIM(_v)		((_v) & 0xff)
N
N
N/*
N * for clock
N */
N/*** LOSPCP ***/
N#define	LOSPCP		(volatile unsigned short *)0x701B	/* Low-Speed Peripheral Clock Pre-Scaler	*/
N#define	LOSPCP_LSPCLK_1		0	/* Low speed clock = SYSCLKOUT/1  */
N#define	LOSPCP_LSPCLK_2		1	/* Low speed clock = SYSCLKOUT/2  */
N#define	LOSPCP_LSPCLK_4		2	/* Low speed clock = SYSCLKOUT/4  (default) */
N#define	LOSPCP_LSPCLK_6		3	/* Low speed clock = SYSCLKOUT/6  */
N#define	LOSPCP_LSPCLK_8		4	/* Low speed clock = SYSCLKOUT/8  */
N#define	LOSPCP_LSPCLK_10	5	/* Low speed clock = SYSCLKOUT/10 */
N#define	LOSPCP_LSPCLK_12	6	/* Low speed clock = SYSCLKOUT/12 */
N#define	LOSPCP_LSPCLK_14	7	/* Low speed clock = SYSCLKOUT/14 */
N
N/** PCLKCR0 ***/
N#define	PCLKCR0		(volatile unsigned short *)0x701C	/* Peripheral Clock Control 0				*/
N#define	PCLKCR0_ECANAENCLK	(1<<14)	/* ECAN-A clock enable			*/
N#define	PCLKCR0_SCIAENCLK	(1<<10)	/* SCI-A clock enable			*/
N#define	PCLKCR0_SPIBENCLK	(1<<9)	/* SPIBENCLK SPI-B clock enable	*/
N#define	PCLKCR0_SPIAENCLK	(1<<8)	/* SPI-A clock enable			*/
N#define	PCLKCR0_I2CAENCLK	(1<<4)	/* I2C clock enable				*/
N#define	PCLKCR0_ADCENCLK	(1<<3)	/* ADC clock enable				*/
N#define	PCLKCR0_TBCLKSYNC	(1<<2)	/* ePWM Module Time Base Clock (TBCLK) Sync
N									 * : Allows the user to globally synchronize all enabled
N									 * ePWM modules to the time base clock (TBCLK):
N									 */
N#define	PCLKCR0_LINAENCLK	(1<<1)	/* LIN-A clock enable.			*/
N#define	PCLKCR0_HRPWMENCLK	(1<<0)	/*  HRPWM clock enable			*/
N
N/** PCLKCR1 ***/
N#define	PCLKCR1		(volatile unsigned short *)0x701D	/* Peripheral Clock Control 1				*/
N#define	PCLKCR1_EQEP1ENCLK	(1<<14)	/* eQEP1 clock enable	*/
N#define	PCLKCR1_ECAP1ENCLK	(1<<8)	/* eCAP1 clock enable	*/
N#define	PCLKCR1_EPWM7ENCLK	(1<<6)	/* ePWM7 clock enable	*/
N#define	PCLKCR1_EPWM6ENCLK	(1<<5)	/* ePWM6 clock enable	*/
N#define	PCLKCR1_EPWM5ENCLK	(1<<4)	/* ePWM5 clock enable	*/
N#define	PCLKCR1_EPWM4ENCLK	(1<<3)	/* ePWM4 clock enable	*/
N#define	PCLKCR1_EPWM3ENCLK	(1<<2)	/* ePWM3 clock enable	*/
N#define	PCLKCR1_EPWM2ENCLK	(1<<1)	/* ePWM2 clock enable	*/
N#define	PCLKCR1_EPWM1ENCLK	(1<<0)	/* ePWM1 clock enable	*/
N
N/** LPMCR0 ***/
N#define	LPMCR0		(volatile unsigned short *)0x701E	/* Low Power Mode Control 0					*/
N#define	LPMCR0_WDINTE	(1<<15)	/* Watchdog interrupt enable */
N#define	LPMCR0_QUALSTDBY(_v)	( ((_v)&0x3f) << 2 )
N#define	LPMCR0_LPM_IDLE		0	/* Set the low power mode to IDLE (default) */
N#define	LPMCR0_LPM_STANDBY	1	/* Set the low power mode to STANDBY */
N#define	LPMCR0_LPM_HALT2	2	/* Set the low power mode to HALT */
N#define	LPMCR0_LPM_HALT3	3	/* Set the low power mode to HALT */
N
N/** PCLKCR3 ***/
N#define	PCLKCR3		(volatile unsigned short *)0x7020	/* Peripheral Clock Control 3				*/
N#define	PCLKCR3_CLA1ENCLK		(1<<14)	/* CLA module clock enable	*/
N#define	PCLKCR3_GPIOINENCLK		(1<<13)	/* GPIO Input Clock Enable	*/
N#define	PCLKCR3_CPUTIMER2ENCLK	(1<<10)	/* CPU Timer 2 Clock Enable	*/
N#define	PCLKCR3_CPUTIMER1ENCLK	(1<<9)	/* CPU Timer 1 Clock Enable	*/
N#define	PCLKCR3_CPUTIMER0ENCLK	(1<<8)	/* CPU Timer 0 Clock Enable	*/
N#define	PCLKCR3_COMP3ENCLK		(1<<2)	/* Comparator3 clock enable	*/
N#define	PCLKCR3_COMP2ENCLK		(1<<1)	/* Comparator2 clock enable	*/
N#define	PCLKCR3_COMP1ENCLK		(1<<0)	/* Comparator1 clock enable	*/
N
N/** PLLCR ***/
N#define	PLLCR		(volatile unsigned short *)0x7021	/* PLL Control								*/
N#define	PLLCR_BYPASS	0
N#define	PLLCR_1			1
N#define	PLLCR_2			2
N#define	PLLCR_3			3
N#define	PLLCR_4			4
N#define	PLLCR_5			5
N#define	PLLCR_6			6
N#define	PLLCR_7			7
N#define	PLLCR_8			8
N#define	PLLCR_9			9
N#define	PLLCR_10		10
N#define	PLLCR_11		11
N#define	PLLCR_12		12
N
N/*** SCSR ***/
N#define	SCSR		(volatile unsigned short *)0x7022	/* System Control & Status					*/
N
N#define	SCSR_WDINTS		(1<<2)	/* Watchdog interrupt status */
N#define	SCSR_WDENINT	(1<<1)	/* Watchdog interrupt enable */
N#define	SCSR_WDOVERRIDE	(1<<0)	/* Watchdog override */
N
N/*** WDCNTR ***/
N#define	WDCNTR		(volatile unsigned short *)0x7023	/* Watchdog Counter							*/
N
N/*** WDKEY ***/
N#define	WDKEY		(volatile unsigned short *)0x7025	/* Watchdog Reset Key						*/
N
N/*** WDCR ***/
N#define	WDCR		(volatile unsigned short *)0x7029	/* Watchdog Control							*/
N#define	WDCR_WDFLAG		(1<<7)	/* Watchdog reset status flag */
N#define	WDCR_WDDIS		(1<<6)	/* Watchdog disable */
N#define	WDCR_WDCHK_101	(5<<3)
N#define	WDCR_WDCHK_MSK	(7<<3)
N			/* Watchdog pre-sclase */
N#define	WDCR_WDPS_1		1	/* WDCLK = OSCCLK/512/1 */
N#define	WDCR_WDPS_2		2	/* WDCLK = OSCCLK/512/2 */
N#define	WDCR_WDPS_4		3	/* WDCLK = OSCCLK/512/4 */
N#define	WDCR_WDPS_8		4	/* WDCLK = OSCCLK/512/8 */
N#define	WDCR_WDPS_16	5	/* WDCLK = OSCCLK/512/16 */
N#define	WDCR_WDPS_32	6	/* WDCLK = OSCCLK/512/32 */
N#define	WDCR_WDPS_64	7	/* WDCLK = OSCCLK/512/64 */
N
N/*** NMICFG ***/
N#define	NMICFG		(volatile unsigned short *)0x7060	/* NMI Configuration Register				*/
N#define	NMICFG_CLOCKFAIL	(1<<1)	/* CLOCKFAIL-interrupt Enable */
N
N/*** NMIFLG/NMIFLGCLR/NMIFLGFRC ***/
N#define	NMIFLG		(volatile unsigned short *)0x7061	/* NMI Flag Register						*/
N#define	NMIFLGCLR	(volatile unsigned short *)0x7062	/* NMI Flag Clear Register					*/
N#define	NMIFLGFRC	(volatile unsigned short *)0x7063	/* NMI Flag Force Register					*/
N
N#define	NMIFLG_CLOCKFAIL	(1<<1)	/* CLOCKFAIL Interrupt Flag */
N#define	NMIFLG_NMIINT		(1<<0)	/* NMI Interrupt Flag */
N
N#define	NMIWDCNT	(volatile unsigned short *)0x7064	/* NMI Watchdog Counter Register			*/
N/*** NMIWDPRD ***/
N#define	NMIWDPRD	(volatile unsigned short *)0x7065	/* NMI Watchdog Period Register				*/
N#define	BORCFG		(volatile unsigned short *)0x0985	/* BOR Configuration						*/
N
N/*** FUNCTION ***/
N//static void ServiceDog(void);
Nstatic void DisableDog(void);
Nstatic void InitPll(unsigned short val, unsigned short divsel);
Nstatic void InitPeripheralClocks(void);
N//static unsigned short CsmUnlock();
Nstatic void IntOsc1Sel (void);
N//static void IntOsc2Sel (void);
N//static void XtalOscSel (void);
N//static void ExtOscSel (void);
N
N
N/*** WORK ***/
N
N
N/*---------------------------------------------------------------------------
N * InitSysCtrl:
N *---------------------------------------------------------------------------
N * This function initializes the System Control registers to a known state.
N * - Disables the watchdog
N * - Set the PLLCR for proper SYSCLKOUT frequency
N * - Set the pre-scaler for the high and low frequency peripheral clocks
N * - Enable the clocks to the peripherals
N */
N
Nvoid InitSysCtrl(void)
N{
N
N	/* Disable the watchdog */
N	DisableDog();
N
N	/* *IMPORTANT*
N	 * The Device_cal function, which copies the ADC & oscillator calibration values
N	 * from TI reserved OTP into the appropriate trim registers, occurs automatically
N	 * in the Boot ROM. If the boot ROM code is bypassed during the debug process, the
N	 * following function MUST be called for the ADC and oscillators to function according
N	 * to specification. The clocks to the ADC MUST be enabled before calling this
N	 * function.
N	 * See the device data manual and/or the ADC Reference
N	 * Manual for more information.
N	 */
N
N	EALLOW;
X	asm(" EALLOW");
N//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1; // Enable ADC peripheral clock
N		*PCLKCR0 |= PCLKCR0_ADCENCLK;	/* Enable ADC peripheral clock */
X		*(volatile unsigned short *)0x701C |= (1<<3);	 
N		(*Device_cal)();
X		(*(void(*)(void))0x3d7c80)();
N//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 0; // Return ADC clock to original state
N		*PCLKCR0 &= ~PCLKCR0_ADCENCLK;	/* Return ADC clock to original state */
X		*(volatile unsigned short *)0x701C &= ~(1<<3);	 
N	EDIS;
X	asm(" EDIS");
N
N	/* Select Internal Oscillator 1 as Clock Source (default), and turn off all unused clocks to
N	 * conserve power.
N	 */
N	IntOsc1Sel();
N
N	/* Initialize the PLL control: PLLCR and CLKINDIV
N	 * DSP28_PLLCR and DSP28_CLKINDIV are defined in DSP2803x_Examples.h
N	 */
N	InitPll(DSP28_PLLCR,DSP28_DIVSEL);
X	InitPll(12,2);
N
N	/* Initialize the peripheral clocks */
N	InitPeripheralClocks();
N
N	/* Initialize TI's Flash API */
N#ifdef	SUPPORT_FLASH
S	flashInit();
N#endif
N}
N
N
N/*---------------------------------------------------------------------------
N * Example: ServiceDog:
N *---------------------------------------------------------------------------
N * This function resets the watchdog timer.
N * Enable this function for using ServiceDog in the application
N */
N#if 0
Sstatic void ServiceDog(void)
S{
S	EALLOW;
S//		SysCtrlRegs.WDKEY = 0x0055;
S//		SysCtrlRegs.WDKEY = 0x00AA;
S		*WDKEY = 0x0055;
S		*WDKEY = 0x00aa;
S	EDIS;
S}
N#endif
N
N/*---------------------------------------------------------------------------
N * Example: DisableDog:
N *---------------------------------------------------------------------------
N * This function disables the watchdog timer.
N */
Nstatic void DisableDog(void)
N{
N	EALLOW;
X	asm(" EALLOW");
N//		SysCtrlRegs.WDCR= 0x0068;
N		*WDCR = WDCR_WDDIS | WDCR_WDCHK_101;
X		*(volatile unsigned short *)0x7029 = (1<<6) | (5<<3);
N	EDIS;
X	asm(" EDIS");
N}
N
N/*---------------------------------------------------------------------------
N * Example: InitPll:
N *---------------------------------------------------------------------------
N * This function initializes the PLLCR register.
N */
Nstatic void InitPll(unsigned short val, unsigned short divsel)
N{
N//	volatile unsigned short iVol;
N
N	/*
N	 * Make sure the PLL is not running in limp mode
N	 */
N//	if (SysCtrlRegs.PLLSTS.bit.MCLKSTS != 0)
N	if( *PLLSTS & PLLSTS_MCLKSTS )
X	if( *(volatile unsigned short *)0x7011 & (1<<3) )
N	{
N	EALLOW;
X	asm(" EALLOW");
N		/*
N		 * OSCCLKSRC1 failure detected. PLL running in limp mode.
N		 * Re-enable missing clock logic.
N		 */
N//		SysCtrlRegs.PLLSTS.bit.MCLKCLR = 1;
N		*PLLSTS |= PLLSTS_MCLKCLR;
X		*(volatile unsigned short *)0x7011 |= (1<<4);
W "../src/bsp/tms320f28035/dspCtrl.c" 348 3 function declared implicitly
N		sysHalt( "PLL limp mode detect!!" );
N	EDIS;
X	asm(" EDIS");
N
N	/*
N	 * Replace this line with a call to an appropriate
N	 * SystemShutdown(); function.
N	 */
N	asm("        ESTOP0");     /* Uncomment for debugging purposes */
N	}
N
N	/*
N	 * DIVSEL MUST be 0 before PLLCR can be changed from
N	 * 0x0000. It is set to 0 by an external reset XRSn
N	 * This puts us in 1/4
N	 */
N//	if (SysCtrlRegs.PLLSTS.bit.DIVSEL != 0)
N	if( *PLLSTS & PLLSTS_DIVSEL_MASK )
X	if( *(volatile unsigned short *)0x7011 & (3<<7) )
N	{
N		EALLOW;
X		asm(" EALLOW");
N//			SysCtrlRegs.PLLSTS.bit.DIVSEL = 0;
N			*PLLSTS &= ~PLLSTS_DIVSEL_MASK;
X			*(volatile unsigned short *)0x7011 &= ~(3<<7);
N		EDIS;
X		asm(" EDIS");
N	}
N
N	/*
N	 * Change the PLLCR
N	 */
N//	if (SysCtrlRegs.PLLCR.bit.DIV != val)
N	if( *PLLCR != val )
X	if( *(volatile unsigned short *)0x7021 != val )
N	{
N		EALLOW;
X		asm(" EALLOW");
N			/*
N			 * Before setting PLLCR turn off missing clock detect logic
N			 */
N//			SysCtrlRegs.PLLSTS.bit.MCLKOFF = 1;
N			*PLLSTS |= PLLSTS_MCLKOFF;
X			*(volatile unsigned short *)0x7011 |= (1<<6);
N//			SysCtrlRegs.PLLCR.bit.DIV = val;
N			*PLLCR = val;
X			*(volatile unsigned short *)0x7021 = val;
N		EDIS;
X		asm(" EDIS");
N
N		/*
N		 * Optional: Wait for PLL to lock.
N		 * During this time the CPU will switch to OSCCLK/2 until
N		 * the PLL is stable.  Once the PLL is stable the CPU will
N		 * switch to the new PLL value.
N		 *
N		 * This time-to-lock is monitored by a PLL lock counter.
N		 *
N		 * Code is not required to sit and wait for the PLL to lock.
N		 * However, if the code does anything that is timing critical,
N		 * and requires the correct clock be locked, then it is best to
N		 * wait until this switching has completed.
N		 *
N		 * Wait for the PLL lock bit to be set.
N		 *
N		 * The watchdog should be disabled before this loop, or fed within
N		 * the loop via ServiceDog().
N		 *
N		 * Uncomment to disable the watchdog
N		 */
N		DisableDog();
N
N//		while(SysCtrlRegs.PLLSTS.bit.PLLLOCKS != 1)
N		while( (*PLLSTS & PLLSTS_PLLLOCKS) == 0 )
X		while( (*(volatile unsigned short *)0x7011 & (1<<0)) == 0 )
N		{
N			/*
N			 * Uncomment to service the watchdog
N			 */
N			// ServiceDog();
N		}
N
N		EALLOW;
X		asm(" EALLOW");
N//			SysCtrlRegs.PLLSTS.bit.MCLKOFF = 0;
N			*PLLSTS &= PLLSTS_MCLKOFF;
X			*(volatile unsigned short *)0x7011 &= (1<<6);
N		EDIS;
X		asm(" EDIS");
N	}
N
N#if 1
N	EALLOW;
X	asm(" EALLOW");
N	{
N		unsigned short pllstsVal;
N
N		pllstsVal = *PLLSTS;
X		pllstsVal = *(volatile unsigned short *)0x7011;
N		pllstsVal &= ~PLLSTS_DIVSEL_MASK;
X		pllstsVal &= ~(3<<7);
N
N		switch( divsel ) {
N		case 1:
N		case 2:
N			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( divsel );
X			*(volatile unsigned short *)0x7011 = pllstsVal | (((divsel)&0x03) <<7);
N			break;
N		case 3:
N			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( 2 );
X			*(volatile unsigned short *)0x7011 = pllstsVal | (((2)&0x03) <<7);
N			DELAY_US(50L);
X			DSP28x_usDelay(((((long double) 50L * 1000.0L) / (long double)16.667L) - 9.0L) / 5.0L);
N			*PLLSTS = pllstsVal | PLLSTS_DIVSEL( 3 );
X			*(volatile unsigned short *)0x7011 = pllstsVal | (((3)&0x03) <<7);
N			break;
N		}
N	}
N	EDIS;
X	asm(" EDIS");
N#else
S	/*
S	 * If switching to 1/2
S	 */
S	if((divsel == 1)||(divsel == 2))
S	{
S		EALLOW;
S			SysCtrlRegs.PLLSTS.bit.DIVSEL = divsel;
S		EDIS;
S	}
S
S	/*
S	 * If switching to 1/1
S	 * * First go to 1/2 and let the power settle
S	 *   The time required will depend on the system, this is only an example
S	 * * Then switch to 1/1
S	 */
S	if(divsel == 3)
S	{
S		EALLOW;
S			SysCtrlRegs.PLLSTS.bit.DIVSEL = 2;
S			DELAY_US(50L);
S			SysCtrlRegs.PLLSTS.bit.DIVSEL = 3;
S		EDIS;
S	}
N#endif
N}
N
N/*--------------------------------------------------------------------------
N * Example: InitPeripheralClocks:
N *---------------------------------------------------------------------------
N * This function initializes the clocks to the peripheral modules.
N * First the high and low clock prescalers are set
N * Second the clocks are enabled to each peripheral.
N * To reduce power, leave clocks to unused peripherals disabled
N *
N * Note: If a peripherals clock is not enabled then you cannot
N * read or write to the registers for that peripheral
N */
Nstatic void InitPeripheralClocks(void)
N{
N	EALLOW;
X	asm(" EALLOW");
N
N		/*
N		 * LOSPCP prescale register settings, normally it will be set to default values
N		 */
N
N#if 0 // delete by mimu
S		GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 3;  /* GPIO18 = XCLKOUT */
N#endif
N//		SysCtrlRegs.LOSPCP.all = 0x0002;
N		*LOSPCP = LOSPCP_LSPCLK_4;
X		*(volatile unsigned short *)0x701B = 2;
N
N		/*
N		 * XCLKOUT to SYSCLKOUT ratio.  By default XCLKOUT = 1/4 SYSCLKOUT
N		 */
N//		SysCtrlRegs.XCLK.bit.XCLKOUTDIV=2;
N		*XCLK = XCLK_XCLKOUTDIV_1;
X		*(volatile unsigned short *)0x7010 = (2);
N
N		/*
N		 * Peripheral clock enables set for the selected peripherals.
N		 * If you are not using a peripheral leave the clock off
N		 * to save on power.
N		 *
N		 * Note: not all peripherals are available on all 2803x derivates.
N		 * Refer to the datasheet for your particular device.
N		 *
N		 * This function is not written to be an example of efficient code.
N		 */
N
N//		SysCtrlRegs.PCLKCR0.bit.ADCENCLK   = 1;	/* ADC								*/
N//		SysCtrlRegs.PCLKCR0.bit.ECANAENCLK = 1;	/* eCAN-A							*/
N//		SysCtrlRegs.PCLKCR0.bit.HRPWMENCLK = 1;	/* HRPWM							*/
N//		SysCtrlRegs.PCLKCR0.bit.I2CAENCLK  = 1;	/* I2C								*/
N//		SysCtrlRegs.PCLKCR0.bit.LINAENCLK  = 1;	/* LIN-A							*/
N//		SysCtrlRegs.PCLKCR0.bit.SCIAENCLK  = 1;	/* SCI-A							*/
N//		SysCtrlRegs.PCLKCR0.bit.SPIAENCLK  = 1;	/* SPI-A							*/
N//		SysCtrlRegs.PCLKCR0.bit.SPIBENCLK  = 1;	/* SPI-B							*/
N//		SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC  = 1;	/* Enable TBCLK within the ePWM		*/
N		*PCLKCR0 = 0
X		*(volatile unsigned short *)0x701C = 0
N		         | PCLKCR0_ECANAENCLK		/* ECAN-A clock enable			*/
X		         | (1<<14)		 
N		         | PCLKCR0_SCIAENCLK		/* SCI-A clock enable			*/
X		         | (1<<10)		 
N		         | PCLKCR0_SPIBENCLK		/* SPIBENCLK SPI-B clock enable	*/
X		         | (1<<9)		 
N		         | PCLKCR0_SPIAENCLK		/* SPI-A clock enable			*/
X		         | (1<<8)		 
N		         | PCLKCR0_I2CAENCLK		/* I2C clock enable				*/
X		         | (1<<4)		 
N		         | PCLKCR0_ADCENCLK			/* ADC clock enable				*/
X		         | (1<<3)			 
N		         | PCLKCR0_TBCLKSYNC		/* ePWM Module Time Base Clock (TBCLK) Sync */
X		         | (1<<2)		 
N		         | PCLKCR0_LINAENCLK		/* LIN-A clock enable.			*/
X		         | (1<<1)		 
N		         | PCLKCR0_HRPWMENCLK		/*  HRPWM clock enable			*/
X		         | (1<<0)		 
N		         ;
N
N//		SysCtrlRegs.PCLKCR1.bit.ECAP1ENCLK = 1;	/* eCAP1							*/
N//		SysCtrlRegs.PCLKCR1.bit.EQEP1ENCLK = 1;	/* eQEP1							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM1ENCLK = 1;	/* ePWM1							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM2ENCLK = 1;	/* ePWM2							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM3ENCLK = 1;	/* ePWM3							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK = 1;	/* ePWM4							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK = 1;	/* ePWM5							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK = 1;	/* ePWM6							*/
N//		SysCtrlRegs.PCLKCR1.bit.EPWM7ENCLK = 1;	/* ePWM7							*/
N		*PCLKCR1 = 0
X		*(volatile unsigned short *)0x701D = 0
N		         | PCLKCR1_EQEP1ENCLK		/* eQEP1 clock enable	*/
X		         | (1<<14)		 
N		         | PCLKCR1_ECAP1ENCLK		/* eCAP1 clock enable	*/
X		         | (1<<8)		 
N		         | PCLKCR1_EPWM7ENCLK		/* ePWM7 clock enable	*/
X		         | (1<<6)		 
N		         | PCLKCR1_EPWM6ENCLK		/* ePWM6 clock enable	*/
X		         | (1<<5)		 
N		         | PCLKCR1_EPWM5ENCLK		/* ePWM5 clock enable	*/
X		         | (1<<4)		 
N		         | PCLKCR1_EPWM4ENCLK		/* ePWM4 clock enable	*/
X		         | (1<<3)		 
N		         | PCLKCR1_EPWM3ENCLK		/* ePWM3 clock enable	*/
X		         | (1<<2)		 
N		         | PCLKCR1_EPWM2ENCLK		/* ePWM2 clock enable	*/
X		         | (1<<1)		 
N		         | PCLKCR1_EPWM1ENCLK		/* ePWM1 clock enable	*/
X		         | (1<<0)		 
N		         ;
N
N//		SysCtrlRegs.PCLKCR3.bit.COMP1ENCLK = 1;	/* COMP1							*/
N//		SysCtrlRegs.PCLKCR3.bit.COMP2ENCLK = 1;	/* COMP2							*/
N//		SysCtrlRegs.PCLKCR3.bit.COMP3ENCLK = 1;	/* COMP3							*/
N//		SysCtrlRegs.PCLKCR3.bit.CLA1ENCLK  = 1;	/* CLA1								*/
N		*PCLKCR3 = 0
X		*(volatile unsigned short *)0x7020 = 0
N		         | PCLKCR3_CLA1ENCLK			/* CLA module clock enable	*/
X		         | (1<<14)			 
N		         | PCLKCR3_GPIOINENCLK			/* GPIO Input Clock Enable	*/
X		         | (1<<13)			 
N		         | PCLKCR3_CPUTIMER2ENCLK		/* CPU Timer 2 Clock Enable	*/
X		         | (1<<10)		 
N		         | PCLKCR3_CPUTIMER1ENCLK		/* CPU Timer 1 Clock Enable	*/
X		         | (1<<9)		 
N		         | PCLKCR3_CPUTIMER0ENCLK		/* CPU Timer 0 Clock Enable	*/
X		         | (1<<8)		 
N		         | PCLKCR3_COMP3ENCLK			/* Comparator3 clock enable	*/
X		         | (1<<2)			 
N		         | PCLKCR3_COMP2ENCLK			/* Comparator2 clock enable	*/
X		         | (1<<1)			 
N		         | PCLKCR3_COMP1ENCLK			/* Comparator1 clock enable	*/
X		         | (1<<0)			 
N		         ;
N
N	EDIS;
X	asm(" EDIS");
N}
N
N/*---------------------------------------------------------------------------
N * Example: CsmUnlock:
N *---------------------------------------------------------------------------
N * This function unlocks the CSM. User must replace 0xFFFF's with current
N * password for the DSP. Returns 1 if unlock is successful.
N */
N#if 0
S#define STATUS_FAIL          0
S#define STATUS_SUCCESS       1
S
Sstatic unsigned short CsmUnlock()
S{
S	volatile unsigned short temp;
S
S	/*
S	 * Load the key registers with the current password. The 0xFFFF's are dummy
S	 * passwords.  User should replace them with the correct password for the DSP.
S	 */
S
S	EALLOW;
S		CsmRegs.KEY0 = 0xFFFF;
S		CsmRegs.KEY1 = 0xFFFF;
S		CsmRegs.KEY2 = 0xFFFF;
S		CsmRegs.KEY3 = 0xFFFF;
S		CsmRegs.KEY4 = 0xFFFF;
S		CsmRegs.KEY5 = 0xFFFF;
S		CsmRegs.KEY6 = 0xFFFF;
S		CsmRegs.KEY7 = 0xFFFF;
S	EDIS;
S
S	/*
S	 * Perform a dummy read of the password locations
S	 * if they match the key values, the CSM will unlock
S	 */
S
S	temp = CsmPwl.PSWD0;
S	temp = CsmPwl.PSWD1;
S	temp = CsmPwl.PSWD2;
S	temp = CsmPwl.PSWD3;
S	temp = CsmPwl.PSWD4;
S	temp = CsmPwl.PSWD5;
S	temp = CsmPwl.PSWD6;
S	temp = CsmPwl.PSWD7;
S
S	/*
S	 * If the CSM unlocked, return succes, otherwise return
S	 * failure.
S	 */
S	if (CsmRegs.CSMSCR.bit.SECURE == 0) {
S		return STATUS_SUCCESS;
S	}
S
S	return STATUS_FAIL;
S
S}
N#endif
N
N/*---------------------------------------------------------------------------
N * Example: IntOsc1Sel:
N *---------------------------------------------------------------------------
N * This function switches to Internal Oscillator 1 and turns off all other clock
N * sources to minimize power consumption
N */
Nstatic void IntOsc1Sel (void)
N{
N	EALLOW;
X	asm(" EALLOW");
N//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF   = 0;
N		*CLKCTL &= ~CLKCTL_INTOSC1OFF;
X		*(volatile unsigned short *)0x7012 &= ~(1<<8);
N
N//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL = 0;	/* Clk Src = INTOSC1 */
N		*CLKCTL &= ~CLKCTL_OSCCLKSRCSEL;
X		*(volatile unsigned short *)0x7012 &= ~(1<<0);
N
N//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF    = 1;	/* Turn off XCLKIN   */
N		*CLKCTL |= CLKCTL_XCLKINOFF;
X		*(volatile unsigned short *)0x7012 |= (1<<13);
N
N//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF   = 1;	/* Turn off XTALOSC  */
N		*CLKCTL |= CLKCTL_XTALOSCOFF;
X		*(volatile unsigned short *)0x7012 |= (1<<14);
N
N//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF   = 1;	/* Turn off INTOSC2  */
N		*CLKCTL |= CLKCTL_INTOSC2OFF;
X		*(volatile unsigned short *)0x7012 |= (1<<10);
N
N	EDIS;
X	asm(" EDIS");
N}
N
N/*---------------------------------------------------------------------------
N * Example: IntOsc2Sel:
N *---------------------------------------------------------------------------
N * This function switches to Internal oscillator 2 from External Oscillator
N * and turns off all other clock sources to minimize power consumption
N * NOTE: If there is no external clock connection, when switching from
N *       INTOSC1 to INTOSC2, EXTOSC and XLCKIN must be turned OFF prior
N *       to switching to internal oscillator 1
N */
N#if 0
Sstatic void IntOsc2Sel (void)
S{
S	EALLOW;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 0;	/* Turn on INTOSC2                    */
S		*CLKCTL &= ~CLKCTL_INTOSC2OFF;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 1;	/* Switch to INTOSC2                  */
S		*CLKCTL |= CLKCTL_OSCCLKSRC2SEL;
S//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 1;	/* Turn off XCLKIN                    */
S		*CLKCTL |= CLKCTL_XCLKINOFF;
S//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 1;	/* Turn off XTALOSC                   */
S		*CLKCTL |= CLKCTL_XTALOSCOFF;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch to Internal Oscillator 2    */
S		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to INTOSC2 */
S		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                   */
S		*CLKCTL |= CLKCTL_INTOSC1OFF;
S	EDIS;
S}
N#endif
N/*---------------------------------------------------------------------------
N * Example: XtalOscSel:
N *---------------------------------------------------------------------------
N * This function switches to External CRYSTAL oscillator and turns off all other clock
N * sources to minimize power consumption. This option may not be available on all
N * device packages
N */
N#if 0
Sstatic void XtalOscSel (void)
S{
S	EALLOW;
S//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 0;	/* Turn on XTALOSC                           */
S		*CLKCTL &= ~CLKCTL_XTALOSCOFF;
S//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 1;	/* Turn off XCLKIN                           */
S		*CLKCTL |= CLKCTL_XCLKINOFF;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 0;	/* Switch to external clock                  */
S		*CLKCTL &= ~CLKCTL_OSCCLKSRC2SEL;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch from INTOSC1 to INTOSC2/ext clk    */
S		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to external clock */
S		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 1;	/* Turn off INTOSC2                          */
S		*CLKCTL |= CLKCTL_INTOSC2OFF;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                          */
S		*CLKCTL |= CLKCTL_INTOSC1OFF;
S	EDIS;
S}
N#endif
N
N/*---------------------------------------------------------------------------
N * Example: ExtOscSel:
N *
N * This function switches to External oscillator and turns off all other clock
N * sources to minimize power consumption.
N */
N#if 0
Sstatic void ExtOscSel (void)
S{
S	EALLOW;
S//		SysCtrlRegs.XCLK.bit.XCLKINSEL       = 1;	/* 1-GPIO19 = XCLKIN, 0-GPIO38 = XCLKIN      */
S		*XCLK |= XCLK_XCLKINSEL;
S//		SysCtrlRegs.CLKCTL.bit.XTALOSCOFF    = 1;	/* Turn on XTALOSC                           */
S		*CLKCTL |= CLKCTL_XTALOSCOFF;
S//		SysCtrlRegs.CLKCTL.bit.XCLKINOFF     = 0;	/* Turn on XCLKIN                            */
S		*CLKCTL &= ~CLKCTL_XCLKINOFF;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRC2SEL = 0;	/* Switch to external clock                  */
S		*CLKCTL &= ~CLKCTL_OSCCLKSRC2SEL;
S//		SysCtrlRegs.CLKCTL.bit.OSCCLKSRCSEL  = 1;	/* Switch from INTOSC1 to INTOSC2/ext clk    */
S		*CLKCTL |= CLKCTL_OSCCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.WDCLKSRCSEL   = 1;	/* Switch Watchdog Clk Src to external clock */
S		*CLKCTL |= CLKCTL_WDCLKSRCSEL;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC2OFF    = 1;	/* Turn off INTOSC2                          */
S		*CLKCTL |= CLKCTL_INTOSC2OFF;
S//		SysCtrlRegs.CLKCTL.bit.INTOSC1OFF    = 1;	/* Turn off INTOSC1                          */
S		*CLKCTL |= CLKCTL_INTOSC1OFF;
S	EDIS;
S}
N#endif
N
N#if 0
S/*******************************************************************************
S *
S */
Sint dsp_init( void )
S{
S
S	// Disable the watchdog
S//	DisableDog();
S	if( watchdog_init( 0 ) ) {
S		return -1;
S	}
S
S#if 1
S	/* *IMPORTANT*
S	 * The Device_cal function, which copies the ADC & oscillator calibration values
S	 * from TI reserved OTP into the appropriate trim registers, occurs automatically
S	 * in the Boot ROM. If the boot ROM code is bypassed during the debug process, the
S	 * following function MUST be called for the ADC and oscillators to function according
S	 * to specification. The clocks to the ADC MUST be enabled before calling this
S	 * function.
S	 * See the device data manual and/or the ADC Reference
S	 * Manual for more information.
S	 */
S	EALLOW;
S
S	//SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 1; // Enable ADC peripheral clock
S	* PCLKCR0 |= (1<<3);
S	(*Device_cal)();
S//	SysCtrlRegs.PCLKCR0.bit.ADCENCLK = 0; // Return ADC clock to original state
S	* PCLKCR0 &= ~(1<<3);
S
S	EDIS;
S#endif
S
S	clk_init();
S}
S
S/******************************************************************************/
S/* Clock control                                                              */
S/******************************************************************************/
S
S/*******************************************************************************
S *
S */
Sint clk_init( void )
S{
S
S#if 0
S	// Select Internal Oscillator 1 as Clock Source (default), and turn off all unused clocks to
S	// conserve power.
S	IntOsc1Sel();
S#endif
S
S	// Initialize the PLL control: PLLCR and CLKINDIV
S	// DSP28_PLLCR and DSP28_CLKINDIV are defined in DSP2803x_Examples.h
S	InitPll(DSP28_PLLCR,DSP28_DIVSEL);
S
S
S	/*
S	 * Initialize the peripheral clocks
S	 */
S	EALLOW;
S	*pCLKCR0 = PCLKCR0_INIT;
S	*pCLKCR1 = PCLKCR1_INIT;
S	*pCLKCR3 = PCLKCR3_INIT;
S
S	EDIS;
S
S	return 0;
S}
S
S/*******************************************************************************
S *
S */
Sint clk_ctrl( int clkID, int bEnable )
S{
S	int retVal;
S
S	volatile unsigned short * pReg;
S	int regVal;
S	int intLockVal;
S
S	switch( _CLK_ID_2_REG_NO( clkID ) ) {
S	case 0 : pReg = PCLKCR0; break;
S	case 1 : pReg = PCLKCR1; break;
S	case 3 : pReg = PCLKCR3; break;
S	default : pReg = NULL; break;
S	}
S
S	if( pReg ) {
S		regVal = 1 << _CLK_ID_2_BIT( clkID );
S
S		intLockVal = sysIntLock();
S
S		EALLOW;
S		if( bEnable ) {
S			*pReg |= regVal;
S		} else {
S			*pReg &= ~regVal;
S		}
S		EDIS;
S		sysIntUnlock( intLockVal );
S
S		retVal = 0;
S	}
S
S	return retVal;
S}
S
S/******************************************************************************/
S/* for Watchdog timer                                                         */
S/******************************************************************************/
S
S/*******************************************************************************
S *
S */
Sstatic int watchdog_init( int bEnable )
S{
S	int retVal = -1;
S
S	if( bEnable ) {
S		/*** Sorry : This version not support Watchdog-timer */
S	} else {
S
S		EALLOW;
S//		SysCtrlRegs.WDCR= 0x0068;
S		*WDCR = 0x0068;
S		EDIS;
S
S		retVal = 0;
S	}
S
S	return retVal;
S}
N#endif
N/*** end of file : clkCtrl.c ***/
