L 1 "../src/bsp/tms320f28035/gpioDrv.c"
N/*** gpioDrv.c - TMS320F28035(piccolo) GPIO Drvier ***/
N
N/*** Copyright(C) 2010 Sansei system Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/09 mimu
N              First Version
N */
N
N/*** NOTE ***
N * Table 4-17. GPIO Registers
N * +--------------+---------+-------+------------------------------------------------------+
N * |              |         | SIZE  |                                                      |
N * | NAME         | ADDRESS | (x16) | DESCRIPTION                                          |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPIO CONTROL REGISTERS (EALLOW PROTECTED)                                             |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPACTRL      | 0x6F80  |   2   | GPIO A Control Register (GPIO0 to 31)                |
N * | GPAQSEL1     | 0x6F82  |   2   | GPIO A Qualifier Select 1 Register (GPIO0 to 15)     |
N * | GPAQSEL2     | 0x6F84  |   2   | GPIO A Qualifier Select 2 Register (GPIO16 to 31)    |
N * | GPAMUX1      | 0x6F86  |   2   | GPIO A MUX 1 Register (GPIO0 to 15)                  |
N * | GPAMUX2      | 0x6F88  |   2   | GPIO A MUX 2 Register (GPIO16 to 31)                 |
N * | GPADIR       | 0x6F8A  |   2   | GPIO A Direction Register (GPIO0 to 31)              |
N * | GPAPUD       | 0x6F8C  |   2   | GPIO A Pull Up Disable Register (GPIO0 to 31)        |
N * | GPBCTRL      | 0x6F90  |   2   | GPIO B Control Register (GPIO32 to 44)               |
N * | GPBQSEL1     | 0x6F92  |   2   | GPIO B Qualifier Select 1 Register (GPIO32 to 44)    |
N * | GPBMUX1      | 0x6F96  |   2   | GPIO B MUX 1 Register (GPIO32 to 44)                 |
N * | GPBDIR       | 0x6F9A  |   2   | GPIO B Direction Register (GPIO32 to 44)             |
N * | GPBPUD       | 0x6F9C  |   2   | GPIO B Pull Up Disable Register (GPIO32 to 44)       |
N * | AIOMUX1      | 0x6FB6  |   2   | Analog, I/O mux 1 register (AIO0 to AIO15)           |
N * | AIODIR       | 0x6FBA  |   2   | Analog, I/O Direction Register (AIO0 to AIO15)       |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPIO DATA REGISTERS (NOT EALLOW PROTECTED)                                            |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPADAT       | 0x6FC0  |   2   | GPIO A Data Register (GPIO0 to 31)                   |
N * | GPASET       | 0x6FC2  |   2   | GPIO A Data Set Register (GPIO0 to 31)               |
N * | GPACLEAR     | 0x6FC4  |   2   | GPIO A Data Clear Register (GPIO0 to 31)             |
N * | GPATOGGLE    | 0x6FC6  |   2   | GPIO A Data Toggle Register (GPIO0 to 31)            |
N * | GPBDAT       | 0x6FC8  |   2   | GPIO B Data Register (GPIO32 to 44)                  |
N * | GPBSET       | 0x6FCA  |   2   | GPIO B Data Set Register (GPIO32 to 44)              |
N * | GPBCLEAR     | 0x6FCC  |   2   | GPIO B Data Clear Register (GPIO32 to 44)            |
N * | GPBTOGGLE    | 0x6FCE  |   2   | GPIO B Data Toggle Register (GPIO32 to 44)           |
N * | AIODAT       | 0x6FD8  |   2   | Analog I/O Data Register (AIO0 to AIO15)             |
N * | AIOSET       | 0x6FDA  |   2   | Analog I/O Data Set Register (AIO0 to AIO15)         |
N * | AIOCLEAR     | 0x6FDC  |   2   | Analog I/O Data Clear Register (AIO0 to AIO15)       |
N * | AIOTOGGLE    | 0x6FDE  |   2   | Analog I/O Data Toggle Register (AIO0 to AIO15)      |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED)                |
N * +--------------+---------+-------+------------------------------------------------------+
N * | GPIOXINT1SEL | 0x6FE0  |   1   | XINT1 GPIO Input Select Register (GPIO0 to 31)       |
N * | GPIOXINT2SEL | 0x6FE1  |   1   | XINT2 GPIO Input Select Register (GPIO0 to 31)       |
N * | GPIOXINT3SEL | 0x6FE2  |   1   | XINT3 GPIO Input Select Register (GPIO0 to 31)       |
N * | GPIOLPMSEL   | 0x6FE8  |   2   | LPM GPIO Select Register (GPIO0 to 31)               |
N * +--------------+---------+-------+------------------------------------------------------+
N */
N
N/*** INCLUDE ***/
N#include "config.h"
L 1 "D:/project/fpalette/ccs4/fpalette/ide_led/src/bsp/config.h" 1
N/*** config.h ***/
N
N
N#ifndef	INC_config_H
N#define	INC_config_H
N
N/******************************************************************************/
N/* NvRam Configuration                                                        */
N/******************************************************************************/
N#define	FLASH_PROGRAM_SEC_NO		0
N#define	FLASH_PROGRAM_SEC_NUM		6
N
N#define	FLASH_BOOTLOADER_SEC_NO		(FLASH_PROGRAM_SEC_NO + FLASH_PROGRAM_SEC_NUM)
N#define	FLASH_BOOTLOADER_SEC_NUM	2
N
N/******************************************************************************/
N/* Driver Configuration                                                       */
N/******************************************************************************/
N
N/*** CPU TIMER ***/
N#define	SUPPORT_CPU_TIMER_0		/* !! ATTATION !! system need timer0 */
N//#define	SUPPORT_CPU_TIMER_1
N//#define	SUPPORT_CPU_TIMER_2
N
N/*** SCI ***/
N#define	SUPPORT_SCI_A
N
N/*** Flash Mem. ***/
N//#define	SUPPORT_FLASH
N
N/******************************************************************************/
N/* GPIO map                                                                   */
N/******************************************************************************/
N#define	GPIO_NO_CPU_LED	39
N
N//#define	GPIO_NO_SCIRXDA	7
N#define	GPIO_NO_SCIRXDA	28
N
N//#define	GPIO_NO_SCITXDA	12
N#define	GPIO_NO_SCITXDA	29
N
N
N/*-----------------------------------------------------------------------------
N      Specify the PLL control register (PLLCR) and divide select (DIVSEL) value.
N-----------------------------------------------------------------------------*/
N
N//#define DSP28_DIVSEL   0 // Enable /4 for SYSCLKOUT
N//#define DSP28_DIVSEL   1 // Disable /4 for SYSCKOUT
N#define DSP28_DIVSEL   2 // Enable /2 for SYSCLKOUT
N//#define DSP28_DIVSEL   3 // Enable /1 for SYSCLKOUT
N
N
N#define DSP28_PLLCR   12    // Uncomment for 60 MHz devices [60 MHz = (10MHz * 12)/2]
N//#define DSP28_PLLCR   11
N//#define DSP28_PLLCR   10
N//#define DSP28_PLLCR    9
N//#define DSP28_PLLCR    8      // Uncomment for 40 MHz devices [40 MHz = (10MHz * 8)/2]
N//#define DSP28_PLLCR    7
N//#define DSP28_PLLCR    6
N//#define DSP28_PLLCR    5
N//#define DSP28_PLLCR    4
N//#define DSP28_PLLCR    3
N//#define DSP28_PLLCR    2
N//#define DSP28_PLLCR    1
N//#define DSP28_PLLCR    0  // PLL is bypassed in this mode
N//----------------------------------------------------------------------------
N
N/*-----------------------------------------------------------------------------
N      Specify the clock rate of the CPU (SYSCLKOUT) in nS.
N
N      Take into account the input clock frequency and the PLL multiplier
N      selected in step 1.
N
N      Use one of the values provided, or define your own.
N      The trailing L is required tells the compiler to treat
N      the number as a 64-bit value.
N
N      Only one statement should be uncommented.
N
N      Example:   60 MHz devices:
N                 CLKIN is a 10 MHz crystal or internal 10 MHz oscillator
N
N                 In step 1 the user specified PLLCR = 0xC for a
N                 60 MHz CPU clock (SYSCLKOUT = 60 MHz).
N
N                 In this case, the CPU_RATE will be 16.667L
N                 Uncomment the line: #define CPU_RATE 16.667L
N
N-----------------------------------------------------------------------------*/
N
N#define CPU_RATE   16.667L   // for a 60MHz CPU clock speed (SYSCLKOUT)
N//#define CPU_RATE   20.000L   // for a 50MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   25.000L   // for a 40MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   33.333L   // for a 30MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   41.667L   // for a 24MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   50.000L   // for a 20MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE   66.667L   // for a 15MHz CPU clock speed  (SYSCLKOUT)
N//#define CPU_RATE  100.000L   // for a 10MHz CPU clock speed  (SYSCLKOUT)
N
N#if 0
S//----------------------------------------------------------------------------
S
S// The following pointer to a function call calibrates the ADC and internal oscillators
S#define Device_cal (void   (*)(void))0x3D7C80
S
S//---------------------------------------------------------------------------
S// Include Example Header Files:
S//
S
S#include "DSP2803x_GlobalPrototypes.h"         // Prototypes for global functions within the
S                                               // .c files.
S#include "DSP2803x_EPwm_defines.h"             // Macros used for PWM examples.
S#include "DSP2803x_I2C_defines.h"              // Macros used for I2C examples.
S#include "DSP2803x_Cla_Defines.h"              // Macros used for CLA examples.
S
S
S#define   PARTNO_28030PAG   0xAA
S#define   PARTNO_28030PN    0xAB
S
S#define   PARTNO_28031PAG   0xAE
S#define   PARTNO_28031PN    0xAF
S
S#define   PARTNO_28032PAG   0xB2
S#define   PARTNO_28032PN    0xB3
S
S#define   PARTNO_28033PAG   0xB6
S#define   PARTNO_28033PN    0xB7
S
S#define   PARTNO_28034PAG   0xBA
S#define   PARTNO_28034PN    0xBB
S
S#define   PARTNO_28035PAG   0xBE
S#define   PARTNO_28035PN    0xBF
S
S// Include files not used with DSP/BIOS
S#ifndef DSP28_BIOS
S#include "DSP2803x_DefaultISR.h"
S#endif
N#endif
N
Nextern void DSP28x_usDelay(unsigned long Count);
N// DO NOT MODIFY THIS LINE.
N#define DELAY_US(A)  DSP28x_usDelay(((((long double) A * 1000.0L) / (long double)CPU_RATE) - 9.0L) / 5.0L)
N
N#endif	/* INC_config_H */
N
N/*** end of file : config.h ***/
L 59 "../src/bsp/tms320f28035/gpioDrv.c" 2
N#include "sysLib.h"
L 1 "D:/project/fpalette/ccs4/fpalette/ide_led/src/bsp/sysLib.h" 1
N/*** sysLib.h - system i/f header ***/
N
N/*** Copyright(C) 2008 mimu ***/
N
N/*** NOTE ***
N */
N
N/*** HISTORY ***
N * 2007/08/xx mimu@emb
N              First version for SH-4A
N */
N
N#ifndef	INC_sysLib_H
N#define	INC_sysLib_H
N
N#ifndef	_ASMLANGUAGE
N
N#include	"config.h"
N
N//#include	"kernel.h"
N
N#ifdef	__cplusplus
Sextern "C" {
N#endif
N
N
Nint sysInit( void );
N
Nvoid sysDummyFunc();
Nvoid sysHalt( const char * msg );
N
N/* interrupt mask */
Nint		sysIntLock( void );
Nvoid	sysIntUnlock( int );
N
Nvoid bspDelay100nSec( int count );
N
Nconst char * sysOptionStringSerch( const char * pOpt, int *pLen, char startcode );
N
N/* for interrupt */
Nint sysIntConnect( unsigned int vecNo, void (*pFunc)(unsigned int vecNo) );
Nint sysIntEnable( unsigned int vecNo );
Nint sysIntDisable( unsigned int vecNo );
N
N//void * sysAlloc( unsigned long size, unsigned long aligne, int bDma );
N
N//extern void cachePurge( void * p, unsigned long size);	/* this function defined in romInit.src */
N
N/*** for NV-RAM ***/
N
N#if 0
S//int sysNvRamInit( void );
S//
S//int	sysNvRamMacGet( int ch, unsigned char * pMac );
S//int sysNvRamMacSet( int ch, unsigned char * pMac );
S//
S//int sysNvRamNetGet( int ch, SysNetCfg * pNet );
S//int sysNvRamNetSet( int ch, SysNetCfg * pNet );
S//
S//int sysNvRamExceptionGet( SysException * pSysException );
S//
S//int sysNvRamConsoleBpsGet( void );
S//int sysNvRamConsoleBpsSet( int bps );
S//
S//int sysNvRamPasswordGet( unsigned char * password );
S//int sysNvRamPasswordSet( unsigned char * password, int len );
S//
S//int sysNvRamLcdContrastGet( void );
S//int sysNvRamLcdContrastSet( int contrast );
N#endif
N
N#if 0	/* move to sysNvRam.h */
Sint	sysNvRamMacGet( int ch, unsigned char * );
Sint sysNvRamMacSet( int ch, unsigned char * );
N#endif
N
N#ifdef	__cplusplus
S}
N#endif
N
N#endif	/* _ASMLANGUAGE */
N#endif	/* INC_sysLib_H */
N
N/*** end of file : sysLib.h ***/
L 60 "../src/bsp/tms320f28035/gpioDrv.c" 2
N//#include "DSP2803x_Device.h"
N#include "tms320f28035.h"
L 1 "..\src\bsp\tms320f28035\tms320f28035.h" 1
N/*** tms320f28035.h - TI TMS320F28035 ***/
N
N/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/22 mimu
N              First Version
N */
N
N#ifndef	INC_tms320f28035_H
N#define	INC_tms320f28035_H
N
N
N/*** Core control ***/
Nextern cregister volatile unsigned int IFR;
Nextern cregister volatile unsigned int IER;
N
N#define  EINT   asm(" clrc INTM")
N#define  DINT   asm(" setc INTM")
N#define  ERTM   asm(" clrc DBGM")
N#define  DRTM   asm(" setc DBGM")
N#define  EALLOW asm(" EALLOW")
N#define  EDIS   asm(" EDIS")
N#define  ESTOP0 asm(" ESTOP0")
N
N
N/*** Memory Map ***/
N
N#define	PIE_VECT_ADRS	0x000d00
N
N
N#endif	/* INC_tms320f28035_H */
N
N/*** end of file : tms320f28035.h ***/
L 62 "../src/bsp/tms320f28035/gpioDrv.c" 2
N#include "gpioDrv.h"
L 1 "..\src\bsp\tms320f28035\gpioDrv.h" 1
N/*** gpioDrv.h - TMS320F28035(piccolo) GPIO driver ***/
N
N/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/
N
N/*** HISTORY ***
N * 2011/02/09 mimu
N              First Version
N */
N
N#ifndef	INC_gpioDrv_H
N#define	INC_gpioDrv_H
N
N#ifdef	__cplusplus
Sextern "C" {
N#endif
N
N#define	GPIO_MAX	44
N#define	GPIO_NUM	45
N
Nint gpioInit( void );
Nint gpioConfig( int no, int mode );
N#define	GPIO_MODE_GPIO			0x001
N#define	GPIO_MODE_1				0x002
N#define	GPIO_MODE_2				0x004
N#define	GPIO_MODE_3				0x008
N
N#define	GPIO_MODE_IN			0x010
N#define	GPIO_MODE_OUT			0x020
N#define	GPIO_MODE_PULLUP_DIS	0x040
N#define	GPIO_MODE_PULLUP		0x080
N
N#define	GPIO_MODE_QUALIFIER_0	0x100
N#define	GPIO_MODE_QUALIFIER_1	0x200
N#define	GPIO_MODE_QUALIFIER_2	0x400
N#define	GPIO_MODE_QUALIFIER_3	0x800
N#define	GPIO_MODE_QUALIFILER(_x)	(0x100<<(_x))&0x0f00))
N
N#define	GPIO_CTRL_SET		0
N#define	GPIO_CTRL_CLR		1
N#define	GPIO_CTRL_TOGGLE	2
N#define	GPIO_CTRL_GET		3
Nint gpioCtrl( int no, int ctrl );
N
N//int gpioControl( int no, int bHigh );
N#define	gpioControl( _no, _bH )	(bH?gpioCtrl(_no, GPIO_CTRL_SET):gpioCtrl(_no, GPIO_CTRL_CLR))
N//int gpioSet( int no );
N#define	gpioSet( _no )		gpioCtrl( _no, GPIO_CTRL_SET )
N//int gpioClear( int no );
N#define	gpioClear( _no )	gpioCtrl( _no, GPIO_CTRL_CLR )
N//int gpioToggle( int no );
N#define	gpioToggle( _no )	gpioCtrl( _no, GPIO_CTRL_TOGGLE )
N//int gpioGet( int no );
N#define	gpioGet( _no )		gpioCtrl( _no, GPIO_CTRL_GET )
N
Nint gpioCtrlRegSet( int reg, int value );
N#define	GPIO_CTRLREG_QUALPRD_0_7	0		/* GPIO 0-7 */
N#define	GPIO_CTRLREG_QUALPRD_8_15	1		/* GPIO 8-15 */
N#define	GPIO_CTRLREG_QUALPRD_16_23	2		/* GPIO 16-23 */
N#define	GPIO_CTRLREG_QUALPRD_24_31	3		/* GPIO 24-31 */
N
N#define	GPIO_CTRLREG_QUBLPRD_32_39	4		/* GPIO 32-39 */
N#define	GPIO_CTRLREG_QUBLPRD_40_44	5		/* GPIO 32-44 */
N
N#ifdef	__cplusplus
S} /* extern "C" */
N#endif
N
N#endif	/* INC_gpioDrv_H */
N
N/*** end of file : gpioDrv.h ***/
L 63 "../src/bsp/tms320f28035/gpioDrv.c" 2
N
N/*** DEFINE ***/
N//#define	CHECK_EN
N
N/* GPIO CONTROL REGISTERS (EALLOW PROTECTED) */
N#define	GPACTRL			(volatile unsigned long *)0x6F80  /* GPIO A Control Register (GPIO0 to 31)				*/
N#define	GPAQSEL1		(volatile unsigned long *)0x6F82  /* GPIO A Qualifier Select 1 Register (GPIO0 to 15)	*/
N#define	GPAQSEL2		(volatile unsigned long *)0x6F84  /* GPIO A Qualifier Select 2 Register (GPIO16 to 31)	*/
N#define	GPAMUX1			(volatile unsigned long *)0x6F86  /* GPIO A MUX 1 Register (GPIO0 to 15)				*/
N#define	GPAMUX2			(volatile unsigned long *)0x6F88  /* GPIO A MUX 2 Register (GPIO16 to 31)				*/
N#define	GPADIR			(volatile unsigned long *)0x6F8A  /* GPIO A Direction Register (GPIO0 to 31)			*/
N#define	GPAPUD			(volatile unsigned long *)0x6F8C  /* GPIO A Pull Up Disable Register (GPIO0 to 31)		*/
N
N#define	GPBCTRL			(volatile unsigned long *)0x6F90  /* GPIO B Control Register (GPIO32 to 44)				*/
N#define	GPBQSEL1		(volatile unsigned long *)0x6F92  /* GPIO B Qualifier Select 1 Register (GPIO32 to 44)	*/
N#define	GPBMUX1			(volatile unsigned long *)0x6F96  /* GPIO B MUX 1 Register (GPIO32 to 44)				*/
N#define	GPBDIR			(volatile unsigned long *)0x6F9A  /* GPIO B Direction Register (GPIO32 to 44)			*/
N#define	GPBPUD			(volatile unsigned long *)0x6F9C  /* GPIO B Pull Up Disable Register (GPIO32 to 44)		*/
N
N#define	AIOMUX1			(volatile unsigned long *)0x6FB6  /* Analog, I/O mux 1 register (AIO0 to AIO15)			*/
N#define	AIODIR			(volatile unsigned long *)0x6FBA  /* Analog, I/O Direction Register (AIO0 to AIO15)		*/
N
N/* GPIO DATA REGISTERS (NOT EALLOW PROTECTED) */
N#define	GPADAT			(volatile unsigned long *)0x6FC0  /* GPIO A Data Register (GPIO0 to 31)					*/
N#define	GPASET			(volatile unsigned long *)0x6FC2  /* GPIO A Data Set Register (GPIO0 to 31)				*/
N#define	GPACLEAR		(volatile unsigned long *)0x6FC4  /* GPIO A Data Clear Register (GPIO0 to 31)			*/
N#define	GPATOGGLE		(volatile unsigned long *)0x6FC6  /* GPIO A Data Toggle Register (GPIO0 to 31)			*/
N
N#define	GPBDAT			(volatile unsigned long *)0x6FC8  /* GPIO B Data Register (GPIO32 to 44)				*/
N#define	GPBSET			(volatile unsigned long *)0x6FCA  /* GPIO B Data Set Register (GPIO32 to 44)			*/
N#define	GPBCLEAR		(volatile unsigned long *)0x6FCC  /* GPIO B Data Clear Register (GPIO32 to 44)			*/
N#define	GPBTOGGLE		(volatile unsigned long *)0x6FCE  /* GPIO B Data Toggle Register (GPIO32 to 44)			*/
N
N#define	AIODAT			(volatile unsigned long *)0x6FD8  /* Analog I/O Data Register (AIO0 to AIO15)			*/
N#define	AIOSET			(volatile unsigned long *)0x6FDA  /* Analog I/O Data Set Register (AIO0 to AIO15)		*/
N#define	AIOCLEAR		(volatile unsigned long *)0x6FDC  /* Analog I/O Data Clear Register (AIO0 to AIO15)		*/
N#define	AIOTOGGLE		(volatile unsigned long *)0x6FDE  /* Analog I/O Data Toggle Register (AIO0 to AIO15)	*/
N
N/* GPIO INTERRUPT AND LOW POWER MODES SELECT REGISTERS (EALLOW PROTECTED) */
N#define	GPIOXINT1SEL	(volatile unsigned short *)0x6FE0  /* XINT1 GPIO Input Select Register (GPIO0 to 31)	*/
N#define	GPIOXINT2SEL	(volatile unsigned short *)0x6FE1  /* XINT2 GPIO Input Select Register (GPIO0 to 31)	*/
N#define	GPIOXINT3SEL	(volatile unsigned short *)0x6FE2  /* XINT3 GPIO Input Select Register (GPIO0 to 31)	*/
N#define	GPIOLPMSEL		(volatile unsigned long *)0x6FE8   /* LPM GPIO Select Register (GPIO0 to 31)			*/
N
N/*** FUNCTION ***/
N
N/*** WORK ***/
N#ifdef	CHECK_EN
Sstatic struct gpio_tbl_t {
S	volatile unsigned long * pDataReg;
S	volatile unsigned long * pSetReg;
S	volatile unsigned long * pClrReg;
S	volatile unsigned long * pToggleReg;
S
S	volatile unsigned long * pDirectionReg;
S
S//	volatile unsigned long * pCtrlReg;
S	volatile unsigned long * pPullUpDisableReg;
S
S	volatile unsigned long * pQualifierSelectReg;
S	volatile unsigned long * pMuxReg;
S
S	int shift;
S	int shift2;
S
S}gpioTbl[GPIO_NUM];
N#endif
N
N#ifdef	CHECK_EN
Sstatic const int gpioMask[GPIO_NUM] =
S{
S#define	_GPIO_MODE_DEFAULT (GPIO_MODE_IN|GPIO_MODE_OUT|GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP|GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3)
S
S	/*  0 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/*  1 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/*  2 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/*  3 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/*  4 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/*  5 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/*  6 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/*  7 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),
S	/*  8 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/*  9 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),
S
S	/* 10 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 11 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2                | _GPIO_MODE_DEFAULT ),
S	/* 12 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 13 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 14 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 15 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 16 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 17 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 18 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 19 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S
S	/* 20 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 21 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 22 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 23 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 24 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 25 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 26 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 27 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 28 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 29 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S
S	/* 30 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/* 31 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/* 32 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 33 */ (GPIO_MODE_GPIO | GPIO_MODE_1 | GPIO_MODE_2 | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 34 */ (GPIO_MODE_GPIO | GPIO_MODE_1               | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 35 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S	/* 36 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S	/* 37 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S	/* 38 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S	/* 39 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S
S	/* 40 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/* 41 */ (GPIO_MODE_GPIO | GPIO_MODE_1                              | _GPIO_MODE_DEFAULT ),
S	/* 42 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 43 */ (GPIO_MODE_GPIO                             | GPIO_MODE_3  | _GPIO_MODE_DEFAULT ),
S	/* 44 */ (GPIO_MODE_GPIO                                            | _GPIO_MODE_DEFAULT ),
S
S};
N#endif
N
N/*******************************************************************************
N *
N */
Nint gpioInit( void )
N{
N#ifdef	CHECK_EN
S	int no;
S
S	for( no=0; no<GPIO_NUM; no++ ) {
S		if( no < 32 ) {
S			gpioTbl[no].pDataReg   = GPADAT;
S			gpioTbl[no].pSetReg    = GPASET;
S			gpioTbl[no].pClrReg    = GPACLEAR;
S			gpioTbl[no].pToggleReg = GPATOGGLE;
S
S			gpioTbl[no].pDirectionReg = GPADIR;
S
S//			gpioTbl[no].pCtrlReg   = GPACTRL;
S			gpioTbl[no].pPullUpDisableReg = GPAPUD;
S
S			if( no < 16 ) {
S				gpioTbl[no].pQualifierSelectReg = GPAQSEL1;
S				gpioTbl[no].pMuxReg = GPAMUX1;
S				gpioTbl[no].shift2 = no<<1;
S			} else {
S				gpioTbl[no].pQualifierSelectReg = GPAQSEL2;
S				gpioTbl[no].pMuxReg = GPAMUX2;
S				gpioTbl[no].shift2 = (no-16)<<1;
S			}
S			gpioTbl[no].shift = no;
S
S		} else {
S			gpioTbl[no].pDataReg   = GPBDAT;
S			gpioTbl[no].pSetReg    = GPBSET;
S			gpioTbl[no].pClrReg    = GPBCLEAR;
S			gpioTbl[no].pToggleReg = GPBTOGGLE;
S
S			gpioTbl[no].pDirectionReg = GPBDIR;
S
S//			gpioTbl[no].pCtrlReg   = GBACTRL;
S			gpioTbl[no].pPullUpDisableReg = GPBPUD;
S
S			gpioTbl[no].pQualifierSelectReg = GPBQSEL1;
S			gpioTbl[no].pMuxReg = GPBMUX1;
S			gpioTbl[no].shift2 = (no-32)<<1;
S
S			gpioTbl[no].shift = (no-32);
S		}
S	}
N#endif
N	return 0;
N}
N
N
N/*******************************************************************************
N *
N */
N#ifdef	CHECK_EN
Sint gpioConfig( int no, int mode )
S{
S	int intLockVal;
S	unsigned long mask, mask2;
S	unsigned long regVal;
S
S	unsigned long mux;
S	unsigned long dir;
S	unsigned long pullup;
S	unsigned long qualifier;
S
S	if( no >= GPIO_NUM ) {
S		return -1;
S	}
S
S	if( mode != (mode & gpioMask[no]) ) {
S		return -1;
S	}
S
S	mask = ~(1<<no);
S	mask2 = ~(3<<gpioTbl[no].shift2);
S
S	switch( mode & (GPIO_MODE_GPIO|GPIO_MODE_1|GPIO_MODE_2|GPIO_MODE_3) ) {
S	case GPIO_MODE_GPIO: mux = 0; break;
S	case GPIO_MODE_1:    mux = 1; break;
S	case GPIO_MODE_2:    mux = 2; break;
S	case GPIO_MODE_3:    mux = 3; break;
S	default: return -1;
S	}
S
S	switch( mode & (GPIO_MODE_IN|GPIO_MODE_OUT) ) {
S	case GPIO_MODE_IN:  dir = 0; break;
S	case GPIO_MODE_OUT: dir = 1; break;
S	default: return -1;
S	}
S
S	switch( mode & (GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP) ) {
S	case GPIO_MODE_PULLUP_DIS: pullup = 1; break;
S	case 0: /* break; */
S	case GPIO_MODE_PULLUP:     pullup = 0; break;
S	default: return -1;
S	}
S
S
S	switch( mode & (GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3) ) {
S	case 0:	/* break; */
S	case GPIO_MODE_QUALIFIER_0: qualifier = 0; break;
S	case GPIO_MODE_QUALIFIER_1: qualifier = 1; break;
S	case GPIO_MODE_QUALIFIER_2: qualifier = 2; break;
S	case GPIO_MODE_QUALIFIER_3: qualifier = 3; break;
S	default : return -1;
S	}
S
S	dir    <<= gpioTbl[no].shift;
S	pullup <<= gpioTbl[no].shift;
S
S	qualifier <<= gpioTbl[no].shift2;
S	mux       <<= gpioTbl[no].shift2;
S
S	intLockVal = sysIntLock();
S		EALLOW;
S
S			regVal = *gpioTbl[no].pDirectionReg;
S			regVal &= mask;
S			regVal |= dir;
S			*gpioTbl[no].pDirectionReg = regVal;
S
S			regVal = *gpioTbl[no].pPullUpDisableReg;
S			regVal &= mask;
S			regVal |= pullup;
S			*gpioTbl[no].pPullUpDisableReg = regVal;
S
S			regVal = *gpioTbl[no].pQualifierSelectReg;
S			regVal &= mask2;
S			regVal |= qualifier;
S			*gpioTbl[no].pQualifierSelectReg = regVal;
S
S			regVal = *gpioTbl[no].pMuxReg;
S			regVal &= mask2;
S			regVal |= mux;
S			*gpioTbl[no].pMuxReg = regVal;
S
S		EDIS;
S	sysIntUnlock( intLockVal );
S
S	return 0;
S}
N#else
Nint gpioConfig( int no, int mode )
N{
N	int intLockVal;
N	unsigned long mask, mask2;
N	unsigned long regVal;
N
N	unsigned long mux;
N	unsigned long dir;
N	unsigned long pullup;
N	unsigned long qualifier;
N
N	int	shift;
N	int	shift2;
N	volatile unsigned long * pDirectionReg;
N	volatile unsigned long * pPullUpDisableReg;
N	volatile unsigned long * pQualifierSelectReg;
N	volatile unsigned long * pMuxReg;
N
N	switch( no >> 4 ) {
N	case 0:	/* no = 0 to 15 */
N		shift  = no;
N		shift2 = no<<1;
N
N		pDirectionReg       = GPADIR;
X		pDirectionReg       = (volatile unsigned long *)0x6F8A;
N		pPullUpDisableReg   = GPAPUD;
X		pPullUpDisableReg   = (volatile unsigned long *)0x6F8C;
N		pQualifierSelectReg = GPAQSEL1;
X		pQualifierSelectReg = (volatile unsigned long *)0x6F82;
N		pMuxReg             = GPAMUX1;
X		pMuxReg             = (volatile unsigned long *)0x6F86;
N		break;
N
N	case 1:	/* no = 16 to 31 */
N		shift  = no;
N		shift2 = (no-16)<<1;
N
N		pDirectionReg       = GPADIR;
X		pDirectionReg       = (volatile unsigned long *)0x6F8A;
N		pPullUpDisableReg   = GPAPUD;
X		pPullUpDisableReg   = (volatile unsigned long *)0x6F8C;
N		pQualifierSelectReg = GPAQSEL2;
X		pQualifierSelectReg = (volatile unsigned long *)0x6F84;
N		pMuxReg             = GPAMUX2;
X		pMuxReg             = (volatile unsigned long *)0x6F88;
N		break;
N
N	case 2:	/* no = 32 to 47 */
N		shift  = no - 32;
N		shift2 = (no-32)<<1;
N
N		pDirectionReg       = GPBDIR;
X		pDirectionReg       = (volatile unsigned long *)0x6F9A;
N		pPullUpDisableReg   = GPBPUD;
X		pPullUpDisableReg   = (volatile unsigned long *)0x6F9C;
N		pQualifierSelectReg = GPBQSEL1;
X		pQualifierSelectReg = (volatile unsigned long *)0x6F92;
N		pMuxReg             = GPBMUX1;
X		pMuxReg             = (volatile unsigned long *)0x6F96;
N		break;
N
N	default :
N		return -1;
N		/* break; un-reach */
N	}
N
N	mask = ~(1<<no);
N	mask2 = ~(3<<shift2);
N
N	switch( mode & (GPIO_MODE_GPIO|GPIO_MODE_1|GPIO_MODE_2|GPIO_MODE_3) ) {
X	switch( mode & (0x001|0x002|0x004|0x008) ) {
N	case GPIO_MODE_GPIO: mux = 0; break;
X	case 0x001: mux = 0; break;
N	case GPIO_MODE_1:    mux = 1; break;
X	case 0x002:    mux = 1; break;
N	case GPIO_MODE_2:    mux = 2; break;
X	case 0x004:    mux = 2; break;
N	case GPIO_MODE_3:    mux = 3; break;
X	case 0x008:    mux = 3; break;
N	default: return -1;
N	}
N
N	switch( mode & (GPIO_MODE_IN|GPIO_MODE_OUT) ) {
X	switch( mode & (0x010|0x020) ) {
N	case GPIO_MODE_IN:  dir = 0; break;
X	case 0x010:  dir = 0; break;
N	case GPIO_MODE_OUT: dir = 1; break;
X	case 0x020: dir = 1; break;
N	default: return -1;
N	}
N
N	switch( mode & (GPIO_MODE_PULLUP_DIS|GPIO_MODE_PULLUP) ) {
X	switch( mode & (0x040|0x080) ) {
N	case GPIO_MODE_PULLUP_DIS: pullup = 1; break;
X	case 0x040: pullup = 1; break;
N	case 0: /* break; */
N	case GPIO_MODE_PULLUP:     pullup = 0; break;
X	case 0x080:     pullup = 0; break;
N	default: return -1;
N	}
N
N
N	switch( mode & (GPIO_MODE_QUALIFIER_0|GPIO_MODE_QUALIFIER_1|GPIO_MODE_QUALIFIER_2|GPIO_MODE_QUALIFIER_3) ) {
X	switch( mode & (0x100|0x200|0x400|0x800) ) {
N	case 0:	/* break; */
N	case GPIO_MODE_QUALIFIER_0: qualifier = 0; break;
X	case 0x100: qualifier = 0; break;
N	case GPIO_MODE_QUALIFIER_1: qualifier = 1; break;
X	case 0x200: qualifier = 1; break;
N	case GPIO_MODE_QUALIFIER_2: qualifier = 2; break;
X	case 0x400: qualifier = 2; break;
N	case GPIO_MODE_QUALIFIER_3: qualifier = 3; break;
X	case 0x800: qualifier = 3; break;
N	default : return -1;
N	}
N
N	dir    <<= shift;
N	pullup <<= shift;
N
N	qualifier <<= shift2;
N	mux       <<= shift2;
N
N	intLockVal = sysIntLock();
N		EALLOW;
X		asm(" EALLOW");
N
N			regVal = *pDirectionReg;
N			regVal &= mask;
N			regVal |= dir;
N			*pDirectionReg = regVal;
N
N			regVal = *pPullUpDisableReg;
N			regVal &= mask;
N			regVal |= pullup;
N			*pPullUpDisableReg = regVal;
N
N			regVal = *pQualifierSelectReg;
N			regVal &= mask2;
N			regVal |= qualifier;
N			*pQualifierSelectReg = regVal;
N
N			regVal = *pMuxReg;
N			regVal &= mask2;
N			regVal |= mux;
N			*pMuxReg = regVal;
N
N		EDIS;
X		asm(" EDIS");
N	sysIntUnlock( intLockVal );
N
N	return 0;
N}
N#endif
N
N/*******************************************************************************
N *
N */
Nstatic volatile unsigned long * regTbl[2][4] =
N{
N	{ GPASET, GPACLEAR, GPATOGGLE, GPADAT, },
X	{ (volatile unsigned long *)0x6FC2, (volatile unsigned long *)0x6FC4, (volatile unsigned long *)0x6FC6, (volatile unsigned long *)0x6FC0, },
N	{ GPBSET, GPBCLEAR, GPBTOGGLE, GPBDAT, },
X	{ (volatile unsigned long *)0x6FCA, (volatile unsigned long *)0x6FCC, (volatile unsigned long *)0x6FCE, (volatile unsigned long *)0x6FC8, },
N};
Nint gpioCtrl( int no, int ctrl )
N{
N	int retVal;
N	int intLockVal;
N	unsigned long regVal;
N
N	if( no < 32 ) {
N		regVal = 1<<no;
N		no = 0;
N	} else {
N		regVal = 1<<(no-32);
N		no = 1;
N	}
N
N	retVal = 0;
N
N	if( ctrl != GPIO_CTRL_GET ) {
X	if( ctrl != 3 ) {
N		intLockVal = sysIntLock();
N		*regTbl[no][ctrl] = regVal;
N		sysIntUnlock( intLockVal );
N	} else {
N		if( *regTbl[no][ctrl] & regVal ) {
N			retVal = 1;
N		}
N	}
N
N	return retVal;
N}
N
N
N#if 0
S/*******************************************************************************
S *
S */
Sint gpioControl( int no, int bHigh )
S{
S	if( bHigh ) {
S		return gpioSet( no );
S	}
S
S	return gpioClear( no );
S
S}
S
S/*******************************************************************************
S *
S */
Sint gpioSet( int no )
S{
S
S#ifdef	CHECK_EN
S	if( no >= GPIO_NUM ) {
S		return -1;
S	}
S#endif
S
S	*gpioTbl[no].pSetReg = (1<<gpioTbl[no].shift);
S
S	return 0;
S}
S
S/*******************************************************************************
S *
S */
Sint gpioClear( int no )
S{
S
S#ifdef	CHECK_EN
S	if( no >= GPIO_NUM ) {
S		return -1;
S	}
S#endif
S
S	*gpioTbl[no].pClrReg = (1<<gpioTbl[no].shift);
S
S	return 0;
S}
S
S/*******************************************************************************
S *
S */
Sint gpioToggle( int no )
S{
S
S#ifdef	CHECK_EN
S	if( no >= GPIO_NUM ) {
S		return -1;
S	}
S#endif
S
S	*gpioTbl[no].pToggleReg = (1<<gpioTbl[no].shift);
S
S	return 0;
S}
S
S/*******************************************************************************
S *
S */
Sint gpioGet( int no )
S{
S	unsigned long regVal;
S
S	regVal = *gpioTbl[no].pDataReg;
S	if( regVal & (1<<gpioTbl[no].shift) ) {
S		return 1;
S	}
S
S	return 0;
S}
N#endif
N
N/*******************************************************************************
N *
N */
Nint gpioCtrlRegSet( int reg, int _value )
N{
N	volatile unsigned long * pReg;
N	int shift;
N	unsigned long mask;
N	unsigned long value;
N	unsigned long regVal;
N
N	mask = 0x0ff;
N	value = (unsigned long)_value;
N	value &= mask;
N	switch( reg ) {
N	case GPIO_CTRLREG_QUALPRD_0_7:			/* GPIO 0-7 */
X	case 0:			 
N		pReg = GPACTRL;
X		pReg = (volatile unsigned long *)0x6F80;
N		shift = 0;
N		break;
N	case GPIO_CTRLREG_QUALPRD_8_15:			/* GPIO 8-15 */
X	case 1:			 
N		pReg = GPACTRL;
X		pReg = (volatile unsigned long *)0x6F80;
N		shift = 8;
N		break;
N	case GPIO_CTRLREG_QUALPRD_16_23:			/* GPIO 16-23 */
X	case 2:			 
N		pReg = GPACTRL;
X		pReg = (volatile unsigned long *)0x6F80;
N		shift = 16;
N		break;
N	case GPIO_CTRLREG_QUALPRD_24_31:			/* GPIO 24-31 */
X	case 3:			 
N		pReg = GPACTRL;
X		pReg = (volatile unsigned long *)0x6F80;
N		shift = 24;
N		break;
N	case GPIO_CTRLREG_QUBLPRD_32_39:			/* GPIO 32-39 */
X	case 4:			 
N		pReg = GPBCTRL;
X		pReg = (volatile unsigned long *)0x6F90;
N		shift = 0;
N		break;
N	case GPIO_CTRLREG_QUBLPRD_40_44:			/* GPIO 32-44 */
X	case 5:			 
N		pReg = GPBCTRL;
X		pReg = (volatile unsigned long *)0x6F90;
N		shift = 8;
N	default:
N		return -1;
N	}
N
N	mask  <<= shift;
N	value <<= shift;
N
N	mask =~mask;
N
N	EALLOW;
X	asm(" EALLOW");
N
N	regVal = *pReg;
N	regVal &= mask;
N	regVal |= value;
N	*pReg = regVal;
N
N	EDIS;
X	asm(" EDIS");
N
N	return 0;
N}
N
N/*** end of file : gpioDrv.c ***/
