/*
// TI File $Revision: /main/2 $
// Checkin $Date: February 20, 2009   15:34:01 $
//###########################################################################
//
// FILE:    28035_RAM_lnk.cmd
//
// TITLE:   Linker Command File For 28035 examples that run out of RAM
//
//          This ONLY includes all SARAM blocks on the 28035 device.
//          This does not include flash or OTP.
//
//          Keep in mind that L0,L1,L2, and L3 are protected by the code
//          security module.
//
//          What this means is in most cases you will want to move to
//          another memory map file which has more memory defined.
//
//###########################################################################
// $TI Release: 2803x Internal Release 2 $
// $Release Date: November 11, 2008 $
//###########################################################################
*/

/* ======================================================
// For Code Composer Studio V2.2 and later
// ---------------------------------------
// In addition to this memory linker command file,
// add the header linker command file directly to the project.
// The header linker command file is required to link the
// peripheral structures to the proper locations within
// the memory map.
//
// The header linker files are found in <base>\DSP2803x_headers\cmd
//
// For BIOS applications add:      DSP2803x_Headers_BIOS.cmd
// For nonBIOS applications add:   DSP2803x_Headers_nonBIOS.cmd
========================================================= */

/* ======================================================
// For Code Composer Studio prior to V2.2
// --------------------------------------
// 1) Use one of the following -l statements to include the
// header linker command file in the project. The header linker
// file is required to link the peripheral structures to the proper
// locations within the memory map                                    */

/* Uncomment this line to include file only for non-BIOS applications */
/* -l DSP2803x_Headers_nonBIOS.cmd */

/* Uncomment this line to include file only for BIOS applications */
/* -l DSP2803x_Headers_BIOS.cmd */

/* 2) In your project add the path to <base>\DSP2803x_headers\cmd to the
   library search path under project->build options, linker tab,
   library search path (-i).
/*========================================================= */

/* Define the memory block start/length for the DSP2803x
   PAGE 0 will be used to organize program sections
   PAGE 1 will be used to organize data sections

   Notes:
         Memory blocks on F28035 are uniform (ie same
         physical memory) in both PAGE 0 and PAGE 1.
         That is the same memory region should not be
         defined for both PAGE 0 and PAGE 1.
         Doing so will result in corruption of program
         and/or data.

         L0 block is mirrored - that is it
         can be accessed in high memory or low memory.
         For simplicity only one instance is used in this
         linker file.

         Contiguous SARAM memory blocks can be combined
         if required to create a larger memory block.
*/

MEMORY
{
	/*
	 * [PAGE n : ] name [(attr)] : origin = constant, length = constant[, fill = constant]
	 * PAGE 0 : text
	 * PAGE 1 : data
	 * PAGE 2 : used by boot-rom
	 * PAGE 3 : user application save area (flash)
	 */
	/*
	 */
/*
 * On chip memory
 *
 *	M0_SRAM		(RWXI)	: origin = 0x000000, length = 0x0400	?*  1K x 16 SARAM	00:0000 - 00:03ff *?
 *	M1_SRAM		(RWXI)	: origin = 0x000400, length = 0x0400	?*  1K x 16 SARAM	00:0400 - 00:07ff *?
 *
 *	L0_SRAM		(RWXI)	: origin = 0x008000, length = 0x0800	?*  2K x 16 SRAM	00:8000 - 00:87ff *?
 *	L1_SRAM		(RWXI)	: origin = 0x008800, length = 0x0400	?*  1K x 16 DP-SRAM	00:8800 - 00:8bff *?
 *	L2_SRAM		(RWXI)	: origin = 0x008c00, length = 0x0400	?*  1K x 16 DP-SRAM	00:8c00 - 00:8fff *?
 *	L3_SRAM		(RWXI)	: origin = 0x009000, length = 0x1000	?*  4K x 16 DP-SRAM	00:9000 - 00:9fff *?
 *
 *	FLASH_H		(RX)	: origin = 0x3e8000, length = 0x2000	?*  8K x 16 SectorH 3e:8000 - 3e:9fff *?
 *	FLASH_G		(RX)	: origin = 0x3ea000, length = 0x2000	?*  8K x 16 SectorG 3e:8000 - 3e:9fff *?
 *	FLASH_F		(RX)	: origin = 0x3ec000, length = 0x2000	?*  8K x 16 SectorF 3e:8000 - 3e:9fff *?
 *	FLASH_E		(RX)	: origin = 0x3ee000, length = 0x2000	?*  8K x 16 SectorE 3e:8000 - 3e:9fff *?
 *	FLASH_D		(RX)	: origin = 0x3f0000, length = 0x2000	?*  8K x 16 SectorE 3e:8000 - 3e:9fff *?
 *	FLASH_C		(RX)	: origin = 0x3f2000, length = 0x2000	?*  8K x 16 SectorC 3e:8000 - 3e:9fff *?
 *	FLASH_B		(RX)	: origin = 0x3f4000, length = 0x2000	?*  8K x 16 SectorB 3e:8000 - 3e:9fff *?
 *	FLASH_A		(RX)	: origin = 0x3f6000, length = 0x2000	?*  8K x 16 SectorA 3e:8000 - 3e:9fff *?
 *						  origin = 0x3f7f80, length = 0x0076	?* Program to 0x0000 When using the Code Security Module *?
 *						  origin = 0x3f7ff6, length = 0x0002	?* Boot-to-Flash Entry Point (program branch instruction here) *?
 *						  origin = 0x3f7ff8, length = 0x0008	?* Security Password (128bits) (Do not program to all zeros)
 */

/*	PAGE 0 : ENTRY		(RWXI)	: origin = 0x0000, length = 0x0002 */
	PAGE 0 : _bootrom_			: origin = 0x0002, length = 0x004e
	PAGE 0 : SRAM00		(RWXI)	: origin = 0x0050, length = 0x03b0

	PAGE 0 : SRAM01		(RWXI)	: origin = 0x0400, length = 0x0400

	PAGE 0 : SRAM1		(RWXI)	: origin = 0x8000, length = 0x0c00
	PAGE 0 : SRAM0		(RWXI)	: origin = 0x8c00, length = 0x1400


	PAGE 0 : FLASH		(RX)	: origin = 0x3e8000, length = 0xff80	/* 64K x 16 Flash Memory */
	PAGE 0 : CSM_RSVD	(R)		: origin = 0x3f7f80, length = 0x0076	/* Part of FLASH Sector A.  Reserved when CSM is in use. */
	PAGE 0 : ENTRY		(RI)	: origin = 0x3f7ff6, length = 0x0002	/* Part of FLASH Sector A.  Used for "Jump to flash" bootloader mode. */
	PAGE 0 : PASSWORDS	(R)		: origin = 0x3f7ff8, length = 0x0008	/* Part of FLASH Sector A.  CSM password locations. (128bits) */

/*	L0_SRAM		(RWXI)	: origin = 0x003f8000, length = 0x0800	?*  2K x 16 SRAM (mirror) */

/*	BOOT_ROM	(R)		: origin = 0x003fe000, length = 0x2000	?* BOOT ROM */

/*
 * TI sample
 * PAGE 0 :	?* for text *?
 */
/*
 *	PAGE 0:    ?* Program Memory *\?
 *		BEGIN_M0        : origin = 0x000000, length = 0x000002     ?* Part of M0SARAM.  Used for "Boot to M0" bootloader mode. *?
 *		L0SARAM         : origin = 0x008000, length = 0x000800     ?* L0 SARAM, CSM secure *?
 *		L3DPSARAM       : origin = 0x009000, length = 0x001000     ?* L3 DPSARAM, CSM secure, CLA Prog RAM *?
 *		OTP             : origin = 0x3D7800, length = 0x000400     ?* OTP *?
 *		DEVICE_CAL      : origin = 0x3D7C80, length = 0x000040     ?* Device_cal function in reserved memory *?
 *		FLASH_ABCDEFGH  : origin = 0x3E8000, length = 0x00FF80     ?* FLASH, All sectors combined *?
 *		CSM_RSVD        : origin = 0x3F7F80, length = 0x000076     ?* Part of FLASH Sector A.  Reserved when CSM is in use. *?
 *		BEGIN_FLASH     : origin = 0x3F7FF6, length = 0x000002     ?* Part of FLASH Sector A.  Used for "Jump to flash" bootloader mode. *?
 *		PASSWORDS       : origin = 0x3F7FF8, length = 0x000008     ?* Part of FLASH Sector A.  CSM password locations. *?
 *		IQTABLES        : origin = 0x3FE000, length = 0x000B50     ?* Part of Boot ROM *?
 *		IQTABLES2       : origin = 0x3FEB50, length = 0x00008C     ?* Part of Boot ROM *?
 *		IQTABLES3       : origin = 0x3FEBDC, length = 0x0000AA     ?* Part of Boot ROM *?
 *		BOOTROM         : origin = 0x3FF27C, length = 0x000D44     ?* Boot ROM *?
 *		RESET           : origin = 0x3FFFC0, length = 0x000002     ?* part of Boot ROM *?
 *
 *	PAGE 1 :   ?* Data Memory *?
 *		M0SARAM         : origin = 0x000002, length = 0x0003FE     ?* M0 SARAM *?
 *		M1SARAM         : origin = 0x000400, length = 0x000400     ?* M1 SARAM *?
 *		L1DPSARAM       : origin = 0x008800, length = 0x000400     ?* L1 DPSARAM, CSM secure, CLA Data RAM 0 *?
 *		L2DPSARAM       : origin = 0x008C00, length = 0x000400     ?* L2 DPSARAM, CSM secure, CLA Data RAM 1 *?
 */

}


SECTIONS
{
/*
 * <<< Initialized Sections Created by the Compiler >>>
 * +---------+---------------------------------------------------------------+-------------------------
 * | Name    | Contents                                                      | Restrictions
 * +---------+---------------------------------------------------------------+-------------------------
 * | .cinit  | Tables for explicitly initialized global and static variables | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .const  | Global and static const variables that are explicitly         | Low 64K
 * |         | initialized and contain string literals                       |
 * +---------+---------------------------------------------------------------+-------------------------
 * | .econst | Far constant variables                                        | Anywhere in data
 * +---------+---------------------------------------------------------------+-------------------------
 * | .pinit  | Table of constructors to be called at startup                 | Program
 * +---------+---------------------------------------------------------------+-------------------------
 * | .switch | Jump tables for large switch statements                       | Program with -mt option
 * |         |                                                               | Data without -mt option
 * +---------+---------------------------------------------------------------+-------------------------
 * | .text   |Executable code and constants                                  | Program
 * +---------+---------------------------------------------------------------+-------------------------
 *
 * <<< Uninitialized Sections Created by the Compiler >>>
 * +----------+------------------------------------+-----------------------
 * | Name     | Contents                           | Restrictions
 * +----------+------------------------------------+-----------------------
 * | .bss     | Global and static variables        | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .ebss    | Far global/static variables        | Anywhere in data
 * +----------+------------------------------------+-----------------------
 * | .stack   | Stack                              | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .sysmem  | Memory for malloc functions (heap) | Low 64K
 * +----------+------------------------------------+-----------------------
 * | .esysmem | Memory for far_malloc functions    | Anywhere in data
 * +----------+------------------------------------+-----------------------
 *
 * <<< Additional Sections by program >>>
 * +-----------+------------------------------------+-----------------------
 * | Name      | Contents                           | Restrictions
 * +-----------+------------------------------------+-----------------------
 * | codestart |
 * +-----------+------------------------------------+-----------------------
 * | ramfuncs  |
 * +-----------+------------------------------------+-----------------------
 * | .reset    |
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 * +-----------+------------------------------------+-----------------------
 *
 * <<< Section Linking in Non-DSP/BIOS Projects (Large memory model) >>>
 * +--------------+----------------+
 * | Section Name | Where to Link  |
 * +--------------+----------------+
 * | .cinit       | Flash          |
 * | .cio         | RAM            |
 * | .const       | Flash          |
 * | .econst      | Flash          |
 * | .pinit       | Flash          |
 * | .switch      | Flash          |
 * | .text        | Flash          |
 * | .bss         | RAM            |
 * | .ebss        | RAM            |
 * | .stack       | Lower 64Kw RAM |
 * | .sysmem      | RAM            |
 * | .esysmem     | RAM            |
 * | .reset       | RAM(1)         |
 * +--------------+----------------+
 * (1) The .reset section contains nothing more than a 32-bit interrupt vector
 * that points to the C-compiler boot function in the runtime support library
 * (the _c_int00 routine). It generally is not used. Instead, the user typically
 * creates their own branch instruction to point to the starting point of the
 * code (see Sections 6 and 7). When not in use, the .reset section should be
 * omitted from the code build by using a DSECT modifier in the linker command
 * file.
 */

	.reset		:                                    PAGE = 0, TYPE = DSECT


	.stack		: > SRAM01,                          PAGE = 0

	.bss		: > SRAM1,                           PAGE = 0
	.ebss		: > SRAM1,                           PAGE = 0
	.cio		: > SRAM1,                           PAGE = 0
	.sysmem		: > SRAM1,                           PAGE = 0
	.esysmem	: > SRAM1,                           PAGE = 0

	codestart	: > ENTRY,                           PAGE = 0

	.text		: > FLASH,                           PAGE = 0
	.cinit		: > FLASH,                           PAGE = 0
	.econst		: > FLASH,                           PAGE = 0
	.pinit		: > FLASH,                           PAGE = 0
	.switch		: > FLASH,                           PAGE = 0

	ramfuncs	: LOAD = FLASH,                      PAGE = 0
	              RUN = SRAM00,
	              LOAD_START(_ramfuncs_loadstart),
	              LOAD_SIZE( _ramfuncs_loadsize),
	              RUN_START( _ramfuncs_runstart)

	csm_rsvd	: > CSM_RSVD,                        PAGE = 0
	passwords	: > PASSWORDS,                       PAGE = 0










/*
 * TI sample
 */
/*
 *	?*** Compiler Required Sections ***?
 *
 *	?* Program memory (PAGE 0) sections *?
 *		.text             : > FLASH_ABCDEFGH,        PAGE = 0
 *		.cinit            : > FLASH_ABCDEFGH,        PAGE = 0
 *		.const            : > FLASH_ABCDEFGH,        PAGE = 0
 *		.econst           : > FLASH_ABCDEFGH,        PAGE = 0
 *		.pinit            : > FLASH_ABCDEFGH,        PAGE = 0
 *		.reset            : > RESET,                 PAGE = 0, TYPE = DSECT  ?* We are not using the .reset section *?
 *		.switch           : > FLASH_ABCDEFGH,        PAGE = 0
 *
 *	?* Data Memory (PAGE 1) sections *?
 *		.bss              : > M0SARAM,               PAGE = 1
 *		.ebss             : > M0SARAM,               PAGE = 1
 *		.cio              : > M0SARAM,               PAGE = 1
 *		.stack            : > M1SARAM,               PAGE = 1
 *		.sysmem           : > M1SARAM,               PAGE = 1
 *		.esysmem          : > M1SARAM,               PAGE = 1
 *
 *	?*** User Defined Sections ***?
 *		codestart         : > BEGIN_FLASH,           PAGE = 0                ?* Used by file CodeStartBranch.asm *?
 *		csm_rsvd          : > CSM_RSVD,              PAGE = 0                ?* Used by file Passwords.asm *?
 *		passwords         : > PASSWORDS,             PAGE = 0                ?* Used by file Passwords.asm *?
 *		secureRamFuncs    :   LOAD = FLASH_ABCDEFGH, PAGE = 0                ?* Used by file Flash.c *?
 *		                      RUN = L0SARAM,         PAGE = 0                ?* Load to flash, run from CSM secure RAM *?
 *		                      LOAD_START(_secureRamFuncs_loadstart),
 *		                      LOAD_SIZE(_secureRamFuncs_loadsize),
 *		                      RUN_START(_secureRamFuncs_runstart)
 */

}

/*
//===========================================================================
// End of file.
//===========================================================================
*/
