/*** ringLib.c - Ring Buffer Manager ***/

/*** Copyright(C) 1989-2011 Kazuya Mimura ***/
/*** Copyrigth(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/16 mimu
              Clean-up & remodify
 * 1989/07/xx mimu
              First Version
 */

/*** NOTE ***/

/*** INCLUDE ***/
#include "sysLib.h"
#include "ringLib.h"

/*** DEFINE ***/
/*** FUNCTION ***/
/*** WORK ***/

/*******************************************************************************
 *
 */
int ringInit( Ring * pRing )
{
	int retVal;

	retVal = -1;
	if( pRing->pDataTop ) {
		if( pRing->size >= 4 ) {
			pRing->pDataBotom = pRing->pDataTop + pRing->size;
			pRing->pGet = pRing->pDataTop;
			pRing->pSetNext = pRing->pDataTop + 1;

			retVal = 0;
		}
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int ringSet( Ring * pRing, unsigned char * pData, int size )
{
	int retVal;
	int intLockVal;
	int bNotFull;

	retVal = 0;

	for( bNotFull=1; (bNotFull) && (size > 0); ) {
		intLockVal = sysIntLock();
		if( pRing->pSetNext != pRing->pGet ) {
			*pRing->pSetNext = *pData;
			pRing->pSetNext ++;
			if( pRing->pSetNext == pRing->pDataBotom ) {
				pRing->pSetNext = pRing->pDataTop;
			}
			pData ++;
			size --;
			retVal ++;
		} else {
			bNotFull = 0;
		}
		sysIntUnlock( intLockVal );
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int ringGet( Ring * pRing, unsigned char * pData, int size )
{
	int retVal;
	unsigned char * pNext;
	int intLockVal;
	int bNotEmpty;

	retVal = 0;

	intLockVal = sysIntLock();

	pNext = pRing->pGet + 1;
	if( pNext == pRing->pDataBotom ) {
		pNext = pRing->pDataTop;
	}

	for( bNotEmpty=1; (bNotEmpty) && (size > 0); ) {
		if( pNext != pRing->pSetNext ) {
			*pData = *pNext;
			pData ++;
			size --;
			pRing->pGet = pNext;
			retVal ++;

			pNext ++;
			if( pNext == pRing->pDataBotom ) {
				pNext = pRing->pDataTop;
			}
		} else {
			bNotEmpty = 0;
		}
	}
	sysIntUnlock(intLockVal);

	return retVal;
}

/*******************************************************************************
 *
 */
unsigned char ringPeek( Ring * pRing )
{
	unsigned char retVal;
	unsigned char * pNext;
	int intLockVal;

	retVal = 0;

	intLockVal = sysIntLock();

	pNext = pRing->pGet + 1;
	if( pNext == pRing->pDataBotom ) {
		pNext = pRing->pDataTop;
	}

	if( pNext != pRing->pSetNext ) {
		retVal = *pNext;
	} else {
	}

	sysIntUnlock(intLockVal);

	return retVal;

}

/*******************************************************************************
 *
 */
void ringFlush( Ring * pRing )
{
	int intLockVal;

	intLockVal = sysIntLock();

	pRing->pGet = pRing->pDataTop;
	pRing->pSetNext = pRing->pDataTop+1;

	sysIntUnlock(intLockVal);
}

/*******************************************************************************
 *
 */
int ringSize( Ring * pRing )
{
	int intLockVal;
	int size;
	unsigned long get, setNext;

	intLockVal = sysIntLock();
	get = (unsigned long)pRing->pGet;
	setNext = (unsigned long)pRing->pSetNext;
	sysIntUnlock(intLockVal);

	if( get == setNext ) {
		size = pRing->size;
	} else if( get < setNext ) {
		size = setNext - get - 1;
	} else {
		size = pRing->size - (get - setNext);
	}

	return size;
}

/*** end of file : ringLib.c ***/
