/*** nvram.c - Non-Volatile memory manage ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/02/24 mimu
              First Version
 */


/*** INCLUDE ***/
#include "config.h"
#include "sysLib.h"

#include "tms320f28035/flashDrv.h"

/*** DEFINE ***/

#ifndef	FLASH_PROGRAM_SEC_NO
#error
#endif

#ifndef	FLASH_PROGRAM_SEC_NUM
#error
#endif

#ifndef	FLASH_BOOTLOADER_SEC_NO
#error
#endif

#ifndef	FLASH_BOOTLOADER_SEC_NUM
#error
#endif
#if ( (FLASH_PROGRAM_SEC_NO + FLASH_PROGRAM_SEC_NUM) > FLASH_BOOTLOADER_SEC_NO )
#error
#endif

#if ( (FLASH_BOOTLOADER_SEC_NO + FLASH_BOOTLOADER_SEC_NUM) > FLASH_SEC_NUM )
#error
#endif

#define	FLASH_PROGRAM_SEC_END	(FLASH_PROGRAM_SEC_NO + FLASH_PROGRAM_SEC_NUM)

/*** FUNCTION ***/

/*** WORK ***/
static struct prog_mng_t {
	unsigned long offset;
	unsigned long rest;
}progMng;

/*******************************************************************************
 *
 */
int programUpdate_start( void )
{
	int secNo;

	progMng.offset = 0;
	progMng.rest = 0;

	for( secNo = FLASH_PROGRAM_SEC_NO; secNo<FLASH_PROGRAM_SEC_END; secNo ++ ) {
		progMng.rest += flashSecSize( secNo );
	}

	return flashErase( FLASH_PROGRAM_SEC_NO, FLASH_PROGRAM_SEC_NUM );
}

/*******************************************************************************
 *
 */
int programUpdate( unsigned short * pData, unsigned short size )
{
	int retVal = -1;

	if( progMng.rest >= size ) {
		retVal = flashWrite( FLASH_PROGRAM_SEC_NO, progMng.offset, pData, size );
		if( retVal == 0 ) {
			progMng.offset += size;
			progMng.rest   -= size;
		}
	}

	return retVal;
}

/*******************************************************************************
 *
 */
int programUpdate_end( void )
{
	return 0;
}

/*******************************************************************************
 *
 */
void programExec( void )
{
	void (*pEntry)( void );
	unsigned short * p;

	p = flashSecTop( FLASH_PROGRAM_SEC_NO );
	if( p ) {
		if( *p != 0xffff ) {
			pEntry = ( void(*)(void) )p;
			pEntry();
		}
	}
}

