/*** adcDrv.cpp - TMS320F28035 adc driver ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/08/23 mimu
              First Version
 */

/*** NOTE ***/

/*** INCLUDE ***/
#include "config.h"
#include "tms320f28035.h"
#include "dspCtrl.h"

#include "adcDrv.h"

/*** DEFINE ***/
typedef struct adc_reg_t {
	volatile unsigned short ctl1;			/* Control 1 Register (EALLOW) */
#define	ADC_CTL1_RESET		(1<<15)				/* ADC module software reset. */
#define	ADC_CTL1_ADCENABLE	(1<<14)				/* ADC Enable */
#define	ADC_CTL1_ADCBSY		(1<<13)				/* ADC Busy */

#define	ADC_CTL1_BSYCHN(_v)	((_v)<<8)			/* 12-8 ADCBSYCHN Set when ADC SOC for current channel is generated */
#define	ADC_CTL1_BSYCHN_A0	ADC_CTL1_BSYCHN(0)
#define	ADC_CTL1_BSYCHN_A1	ADC_CTL1_BSYCHN(1)
#define	ADC_CTL1_BSYCHN_A2	ADC_CTL1_BSYCHN(2)
#define	ADC_CTL1_BSYCHN_A3	ADC_CTL1_BSYCHN(3)
#define	ADC_CTL1_BSYCHN_A4	ADC_CTL1_BSYCHN(4)
#define	ADC_CTL1_BSYCHN_A5	ADC_CTL1_BSYCHN(5)
#define	ADC_CTL1_BSYCHN_A6	ADC_CTL1_BSYCHN(6)
#define	ADC_CTL1_BSYCHN_A7	ADC_CTL1_BSYCHN(7)

#define	ADC_CTL1_BSYCHN_B0	ADC_CTL1_BSYCHN(8)
#define	ADC_CTL1_BSYCHN_B1	ADC_CTL1_BSYCHN(9)
#define	ADC_CTL1_BSYCHN_B2	ADC_CTL1_BSYCHN(10)
#define	ADC_CTL1_BSYCHN_B3	ADC_CTL1_BSYCHN(11)
#define	ADC_CTL1_BSYCHN_B4	ADC_CTL1_BSYCHN(12)
#define	ADC_CTL1_BSYCHN_B5	ADC_CTL1_BSYCHN(13)
#define	ADC_CTL1_BSYCHN_B6	ADC_CTL1_BSYCHN(14)
#define	ADC_CTL1_BSYCHN_B7	ADC_CTL1_BSYCHN(15)

#define	ADC_CTL1_ADCPWDN		(1<<7)			/* ADC power down (active low). */
#define	ADC_CTL1_ADCBGPWD		(1<<6)			/* Bandgap circuit power down (active low) */
#define	ADC_CTL1_ADCREFPWD		(1<<5)			/* Reference buffers circuit power down (active low) */
#define	ADC_CTL1_ADCREFSEL		(1<<3)			/* Internal/external reference select */
#define	ADC_CTL1_INTPULSEPOS	(1<<2)			/* INT Pulse Generation control */
#define	ADC_CTL1_VREFLOCONV		(1<<1)			/* VREFLO Convert. */
#define	ADC_CTL1_TEMPCONV		(1<<0)			/* Temperature sensor convert. */

	short __dummy01[3];
	volatile unsigned short intFlg;			/* Interrupt Flag Register */
	volatile unsigned short intFlgClr;		/* Interrupt Flag Clear Register */
	volatile unsigned short intOvf;			/* Interrupt Overflow Register */
	volatile unsigned short intOfvClr;		/* Interrupt Overflow Clear Register */
#define	ADC_INT_xxx_INT1		(1<<0)
#define	ADC_INT_xxx_INT2		(1<<1)
#define	ADC_INT_xxx_INT3		(1<<2)
#define	ADC_INT_xxx_INT4		(1<<3)
#define	ADC_INT_xxx_INT5		(1<<4)
#define	ADC_INT_xxx_INT6		(1<<5)
#define	ADC_INT_xxx_INT7		(1<<6)
#define	ADC_INT_xxx_INT8		(1<<7)
#define	ADC_INT_xxx_INT9		(1<<8)


	volatile unsigned short intSel1n2;			/* Interrupt 1 and 2 Selection Register (EALLOW) */
#define	ADC_INTSEL_INT2CONT			(1<<14)		/* ADCINT2 Continuous Mode Enable */
#define	ADC_INTSEL_INT2E			(1<<13)		/* Interrupt Enable */
#define	ADC_INTSEL_INT2SEL(_eoc)	((_eoc)<<8)	/* EOCx (x:0-15) is trigger for ADCINT2 */
#define	ADC_INTSEL_INT1CONT			(1<<6)		/* ADCINT1 Continuous Mode Enable */
#define	ADC_INTSEL_INT1E			(1<<5)		/* Interrupt Enable */
#define	ADC_INTSEL_INT1SEL(_eoc)	(_eoc)		/* EOCx (x:0-15) is trigger for ADCINT 1 */

	volatile unsigned short intSel3n4;			/* Interrupt 3 and 4 Selection Register (EALLOW) */
#define	ADC_INTSEL_INT4CONT			(1<<14)		/* ADCINT4 Continuous Mode Enable */
#define	ADC_INTSEL_INT4E			(1<<13)		/* Interrupt Enable */
#define	ADC_INTSEL_INT4SEL(_eoc)	((_eoc)<<8)	/* EOCx (x:0-15) is trigger for ADCINT2 */
#define	ADC_INTSEL_INT3CONT			(1<<6)		/* ADCINT3 Continuous Mode Enable */
#define	ADC_INTSEL_INT3E			(1<<5)		/* Interrupt Enable */
#define	ADC_INTSEL_INT3SEL(_eoc)	(_eoc)		/* EOCx (x:0-15) is trigger for ADCINT 3 */

	volatile unsigned short intSel5n6;			/* Interrupt 5 and 6 Selection Register (EALLOW) */
#define	ADC_INTSEL_INT6CONT			(1<<14)		/* ADCINT6 Continuous Mode Enable */
#define	ADC_INTSEL_INT6E			(1<<13)		/* Interrupt Enable */
#define	ADC_INTSEL_INT6SEL(_eoc)	((_eoc)<<8)	/* EOCx (x:0-15) is trigger for ADCINT2 */
#define	ADC_INTSEL_INT5CONT			(1<<6)		/* ADCINT5 Continuous Mode Enable */
#define	ADC_INTSEL_INT5E			(1<<5)		/* Interrupt Enable */
#define	ADC_INTSEL_INT5SEL(_eoc)	(_eoc)		/* EOCx (x:0-15) is trigger for ADCINT 3 */

	volatile unsigned short intSel7n8;			/* Interrupt 7 and 8 Selection Register (EALLOW) */
#define	ADC_INTSEL_INT8CONT			(1<<14)		/* ADCINT8 Continuous Mode Enable */
#define	ADC_INTSEL_INT8E			(1<<13)		/* Interrupt Enable */
#define	ADC_INTSEL_INT8SEL(_eoc)	((_eoc)<<8)	/* EOCx (x:0-15) is trigger for ADCINT2 */
#define	ADC_INTSEL_INT7CONT			(1<<6)		/* ADCINT7 Continuous Mode Enable */
#define	ADC_INTSEL_INT7E			(1<<5)		/* Interrupt Enable */
#define	ADC_INTSEL_INT7SEL(_eoc)	(_eoc)		/* EOCx (x:0-15) is trigger for ADCINT 3 */

	volatile unsigned short intSel9n10;			/* Interrupt 9 Selection Register (reserved Interrupt 10 Selection) (EALLOW) */
#define	ADC_INTSEL_INT9CONT			(1<<14)		/* ADCINT8 Continuous Mode Enable */
#define	ADC_INTSEL_INT9E			(1<<13)		/* Interrupt Enable */
#define	ADC_INTSEL_INT9SEL(_eoc)	((_eoc)<<8)	/* EOCx (x:0-15) is trigger for ADCINT2 */

	short __dummy0d[3];
	volatile unsigned short socPriCtl;			/* SOC Priority Control Register (EALLOW) */
												/* 10-5  Round Robin Pointer.
												 * Holds the value of the last converted round robin SOCx to be used by the
												 * round robin scheme to determine order of conversions.
												 *
												 *   00h SOC0 was last round robin SOC to convert. SOC1 is highest round robin priority.
												 *   01h SOC1 was last round robin SOC to convert. SOC2 is highest round robin priority.
												 *   02h SOC2 was last round robin SOC to convert. SOC3 is highest round robin priority.
												 *   03h SOC3 was last round robin SOC to convert. SOC4 is highest round robin priority.
												 *   04h SOC4 was last round robin SOC to convert. SOC5 is highest round robin priority.
												 *   05h SOC5 was last round robin SOC to convert. SOC6 is highest round robin priority.
												 *   06h SOC6 was last round robin SOC to convert. SOC7 is highest round robin priority.
												 *   07h SOC7 was last round robin SOC to convert. SOC8 is highest round robin priority.
												 *   08h SOC8 was last round robin SOC to convert. SOC9 is highest round robin priority.
												 *   09h SOC9 was last round robin SOC to convert. SOC10 is highest round robin priority.
												 *   0Ah SOC10 was last round robin SOC to convert. SOC11 is highest round robin priority.
												 *   0Bh SOC11 was last round robin SOC to convert. SOC12 is highest round robin priority.
												 *   0Ch SOC12 was last round robin SOC to convert. SOC13 is highest round robin priority.
												 *   0Dh SOC13 was last round robin SOC to convert. SOC14 is highest round robin priority.
												 *   0Eh SOC14 was last round robin SOC to convert. SOC15 is highest round robin priority.
												 *   0Fh SOC15 was last round robin SOC to convert. SOC0 is highest round robin priority.
												 *   1xh Invalid value
												 *   20h Reset value to indicate no SOC has been converted.
												 *       SOC0 is highest round robin priority. Set to this value when the device
												 *       is reset, when the ADCCTL1.RESET bit is set, or when the SOCPRICTL
												 *       register is written. In the latter case, if a conversion is currently in
												 *       progress, it will complete and then the new priority will take effect.
												 *   Others Invalid selection.
												 *
												 * 4-0 SOCPRIORITY SOC Priority.
												 * Determines the cutoff point for priority mode and round robin arbitration for SOCx
												 *
												 *   00h SOC priority is handled in round robin mode for all channels.
												 *   01h SOC0 is high priority, rest of channels are in round robin mode.
												 *   02h SOC0-SOC1 are high priority, SOC2-SOC15 are in round robin mode.
												 *   03h SOC0-SOC2 are high priority, SOC3-SOC15 are in round robin mode.
												 *   04h SOC0-SOC3 are high priority, SOC4-SOC15 are in round robin mode.
												 *   05h SOC0-SOC4 are high priority, SOC5-SOC15 are in round robin mode.
												 *   06h SOC0-SOC5 are high priority, SOC6-SOC15 are in round robin mode.
												 *   07h SOC0-SOC6 are high priority, SOC7-SOC15 are in round robin mode.
												 *   08h SOC0-SOC7 are high priority, SOC8-SOC15 are in round robin mode.
												 *   09h SOC0-SOC8 are high priority, SOC9-SOC15 are in round robin mode.
												 *   0Ah SOC0-SOC9 are high priority, SOC10-SOC15 are in round robin mode.
												 *   0Bh SOC0-SOC10 are high priority, SOC11-SOC15 are in round robin mode.
												 *   0Ch SOC0-SOC11 are high priority, SOC12-SOC15 are in round robin mode.
												 *   0Dh SOC0-SOC12 are high priority, SOC13-SOC15 are in round robin mode.
												 *   0Eh SOC0-SOC13 are high priority, SOC14-SOC15 are in round robin mode.
												 *   0Fh SOC0-SOC14 are high priority, SOC15 is in round robin mode.
												 *   10h All SOCs are in high priority mode, arbitrated by SOC number
												 *   Others Invalid selection.
												 */
	short __dummy11;
	volatile unsigned short sampleMode;		/* Sampling Mode Register(EALLOW) */
#define	ADC_SAMPLEMODE_SIMULEN(_x)		(1<<((_x)/2))	/* Simultaneous sampling enable for SOC(x)/SOC(x+1) */

	short __dummy13;
	volatile unsigned short intSocSel1;		/* Interrupt SOC Selection 1 Register (for 8 channels)(EALLOW) */
	volatile unsigned short intSocSel2;		/* Interrupt SOC Selection 2 Register (for 8 channels)(EALLOW) */
#define	ADC_INTSEL1_SOC(_x, _v)		((_v)<<((_x)<<1))			/* (_x) is 0-7 */
#define	ADC_INTSEL2_SOC(_x, _v)		((_v)<<(((_x)-8)<<1))		/* (_x) is 8-15 */
#define	ADC_INTSELx_SOC_NOINT		0
#define	ADC_INTSELx_SOC_ADCINT1		1
#define	ADC_INTSELx_SOC_ADCINT2		2

	short __dummy16[2];
	volatile unsigned short socFlg1;			/* SOC Flag 1 Register (for 16 channels) */
#define	ADC_SOCFLG1_SOC(_x)		(1<<(_x))		/* SOCx Start of Conversion Flag */

	short __dummy19;
	volatile unsigned short socFrc1;			/* SOC Force 1 Register (for 16 channels) */
#define	ADC_SOCFRC1_SOC(_x)	(1<<(_x))		/* SOCx Force Start of Conversion Flag */

	short __dummy1b;
	volatile unsigned short socOvf1;			/* SOC Overflow 1 Register (for 16 channels) */
#define	ADC_SOCOVF1_SOC(_x)	(1<<(_x))		/* SOCx Start of Conversion Overflow Flag */

	short __dummy1d;
	volatile unsigned short socOfvClr1;		/* SOC Overflow Clear 1 Register (for 16 channels) */
#define	ADC_SOCOVFCLR1_SOC(_x)	(1<<(_x))		/* SOCx Clear Start of Conversion Overflow Flag */

	short __dummy1f;
	volatile unsigned short socCtl[16];		/* SOC0 Control Register to SOC15 Control Register(EALLOW) */
#define	ADC_SOCCTL_TRIGSEL(_v)		((_v)<<11)
#define	ADC_SOCCTL_TRIGSEL_SOFT		ADC_SOCCTL_TRIGSEL(0)
#define	ADC_SOCCTL_TRIGSEL_TIMER0	ADC_SOCCTL_TRIGSEL(1)
#define	ADC_SOCCTL_TRIGSEL_TIMER1	ADC_SOCCTL_TRIGSEL(2)
#define	ADC_SOCCTL_TRIGSEL_TIMER2	ADC_SOCCTL_TRIGSEL(3)
#define	ADC_SOCCTL_TRIGSEL_XINT2	ADC_SOCCTL_TRIGSEL(4)
#define	ADC_SOCCTL_TRIGSEL_EPWM1A	ADC_SOCCTL_TRIGSEL(5)
#define	ADC_SOCCTL_TRIGSEL_EPWM1B	ADC_SOCCTL_TRIGSEL(6)
#define	ADC_SOCCTL_TRIGSEL_EPWM2A	ADC_SOCCTL_TRIGSEL(7)
#define	ADC_SOCCTL_TRIGSEL_EPWM2B	ADC_SOCCTL_TRIGSEL(8)
#define	ADC_SOCCTL_TRIGSEL_EPWM3A	ADC_SOCCTL_TRIGSEL(9)
#define	ADC_SOCCTL_TRIGSEL_EPWM3B	ADC_SOCCTL_TRIGSEL(10)
#define	ADC_SOCCTL_TRIGSEL_EPWM4A	ADC_SOCCTL_TRIGSEL(11)
#define	ADC_SOCCTL_TRIGSEL_EPWM4B	ADC_SOCCTL_TRIGSEL(12)
#define	ADC_SOCCTL_TRIGSEL_EPWM5A	ADC_SOCCTL_TRIGSEL(13)
#define	ADC_SOCCTL_TRIGSEL_EPWM5B	ADC_SOCCTL_TRIGSEL(14)
#define	ADC_SOCCTL_TRIGSEL_EPWM6A	ADC_SOCCTL_TRIGSEL(15)
#define	ADC_SOCCTL_TRIGSEL_EPWM6B	ADC_SOCCTL_TRIGSEL(16)
#define	ADC_SOCCTL_TRIGSEL_EPWM7A	ADC_SOCCTL_TRIGSEL(17)
#define	ADC_SOCCTL_TRIGSEL_EPWM7B	ADC_SOCCTL_TRIGSEL(18)

#define	ADC_SOCCTL_CHSEL(_v)		((_v)<<6)
#define	ADC_SOCCTL_CHSEL_ADCINA0	ADC_SOCCTL_CHSEL(0)
#define	ADC_SOCCTL_CHSEL_ADCINA1	ADC_SOCCTL_CHSEL(1)
#define	ADC_SOCCTL_CHSEL_ADCINA2	ADC_SOCCTL_CHSEL(2)
#define	ADC_SOCCTL_CHSEL_ADCINA3	ADC_SOCCTL_CHSEL(3)
#define	ADC_SOCCTL_CHSEL_ADCINA4	ADC_SOCCTL_CHSEL(4)
#define	ADC_SOCCTL_CHSEL_ADCINA5	ADC_SOCCTL_CHSEL(5)
#define	ADC_SOCCTL_CHSEL_ADCINA6	ADC_SOCCTL_CHSEL(6)
#define	ADC_SOCCTL_CHSEL_ADCINA7	ADC_SOCCTL_CHSEL(7)
#define	ADC_SOCCTL_CHSEL_ADCINB0	ADC_SOCCTL_CHSEL(8)
#define	ADC_SOCCTL_CHSEL_ADCINB1	ADC_SOCCTL_CHSEL(9)
#define	ADC_SOCCTL_CHSEL_ADCINB2	ADC_SOCCTL_CHSEL(10)
#define	ADC_SOCCTL_CHSEL_ADCINB3	ADC_SOCCTL_CHSEL(11)
#define	ADC_SOCCTL_CHSEL_ADCINB4	ADC_SOCCTL_CHSEL(12)
#define	ADC_SOCCTL_CHSEL_ADCINB5	ADC_SOCCTL_CHSEL(13)
#define	ADC_SOCCTL_CHSEL_ADCINB6	ADC_SOCCTL_CHSEL(14)
#define	ADC_SOCCTL_CHSEL_ADCINB7	ADC_SOCCTL_CHSEL(15)

#define	ADC_SOCCTL_ACQPS(_v)		((_v)-1)	/* (_v):7-64 SOCx Acquisition Prescale */

	short __dumy30[16];

	volatile unsigned short refTrim;			/* Reference Trim Register(EALLOW) */
	volatile unsigned short offTrim;			/* Offset Trim Register(EALLOW) */
	short __dummy42[13];
	volatile unsigned short rev;				/* Revision Register */
} AdcReg;


/*** FUNCTION ***/

/*** WORK ***/





























typedef volatile unsigned short * AdcResReg;	/* ADC Result 0 Register to ADC Result 15 Register */

#define	ADC_REG_TOP		0x7100
#define	ADC_RES_REG_TOP	0x0b00

static AdcReg * pReg;
static volatile unsigned short * pResReg;
static int sampleWindow;

/*******************************************************************************
 *
 */
int adc_init( int _sampleWindow )
{
	volatile int delay;

	pReg = (AdcReg *)ADC_REG_TOP;
	pResReg = (volatile unsigned short *)ADC_RES_REG_TOP;

	sampleWindow = _sampleWindow;
	if( sampleWindow < 7 ) {
		sampleWindow = 7;
	} else if( sampleWindow > 64 ) {
		sampleWindow = 64;
	}
	EALLOW;

	/*
	 * Reset ADC
	 */
	pReg->ctl1 = (unsigned short)ADC_CTL1_RESET;

	for( delay=0; delay<2; delay++ ) {
		;
	}

	/*
	 * Power Up Sequence
	 */

	/*
	 * 1. If an external reference is desired, enable this mode using bit 3 (ADCREFSEL) in the ADCCTL1 register.
	 */
	pReg->ctl1 = ADC_CTL1_ADCREFSEL;

	/*
	 * 2. Power up the reference, bandgap, and analog circuits together by setting bits 7-5 (ADCPWDN,
	 *    ADCBGPWD, ADCREFPWD) in the ADCCTL1 register. Intermediary states are not currently supported.
	 */
	pReg->ctl1 = ADC_CTL1_ADCREFSEL
	           | ADC_CTL1_ADCPWDN
	           | ADC_CTL1_ADCBGPWD
	           | ADC_CTL1_ADCREFPWD
	           ;

	/*
	 * 3. Enable the ADC by setting bit 14 (ADCENABLE) of the ADCCTL1 register.
	 */
	pReg->ctl1 = ADC_CTL1_ADCREFSEL
	           | ADC_CTL1_ADCPWDN
	           | ADC_CTL1_ADCBGPWD
	           | ADC_CTL1_ADCREFPWD
	           | ADC_CTL1_ADCENABLE
	           ;

	EDIS;
	/*
	 * 4. Before performing the first conversion, a delay of 1 millisecond after step 2 is required.
	 */
	for( delay=0; delay<1000; delay ++ ) {
		;
	}

	return 0;
}

#if 0
/*******************************************************************************
 *
 */
int adc_calibration( void )
{
}

#endif

int adc( int ch, unsigned short * pData )
{

	EALLOW;
	pReg->socPriCtl  = 0x0000;
	pReg->sampleMode = 0x0000;
	pReg->intSocSel1 = 0x0000;
	pReg->intSocSel2 = 0x0000;

	pReg->socCtl[0] = ADC_SOCCTL_TRIGSEL_SOFT
	                | ADC_SOCCTL_CHSEL( ch )
	                | ADC_SOCCTL_ACQPS( sampleWindow )
	                ;

	pReg->socFrc1 = ADC_SOCFRC1_SOC(0);	/* start adc */

	while( (pReg->ctl1 & ADC_CTL1_ADCBSY) != 0 ) {
		;
	}

	*pData = *pResReg;

	EDIS;

	return 0;
}



