/*** pwmDrv.c - TMS320F28035 PWM Driver ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/03/04 mimu
              First Version
 */

/*** NOTE ***/

/*** INCLUDE ***/
#include <stdlib.h>

#include "config.h"
#include "sysLib.h"

#include "gpioDrv.h"
#include "pwmDrv.h"


/*** DEFINE ***/
#define	SYSCLK	60000000UL
//#if 1
/* sysclk = 60Mhz -> 60/1 / 600 =>100K */
//#define	TBCTL_DIV_VAL	(TBCTL_HSPCLKDIV_1|TBCTL_CLKDIV_1)
//#define	TBPRD_VAL	600
//#endif

#define	PWM_CH_NUM	7
#define	PWM_CH2VECNO(_ch)		((_ch)+48)
#define	PWM_VECNO2CH(_vecno)	((_vecno)-48)

#define	PWM1_ADRS	0x6800	/* ePWM1 + HRPWM1 Registers 0x6800 - 0x683F 64 (3) */
#define	PWM2_ADRS	0x6840	/* ePWM2 + HRPWM2 Registers 0x6840 - 0x687F 64 (3) */
#define	PWM3_ADRS	0x6880	/* ePWM3 + HRPWM3 Registers 0x6880 - 0x68BF 64 (3) */
#define	PWM4_ADRS	0x68c0	/* ePWM4 + HRPWM4 Registers 0x68C0 - 0x68FF 64 (3) */
#define	PWM5_ADRS	0x6900	/* ePWM5 + HRPWM5 Registers 0x6900 - 0x693F 64 (3) */
#define	PWM6_ADRS	0x6940	/* ePWM6 + HRPWM6 Registers 0x6940 - 0x697F 64 (3) */
#define	PWM7_ADRS	0x6980	/* ePWM7 + HRPWM7 Registers 0x6980 - 0x69BF 64 (3) */


typedef struct pwm_reg_t {
	                                      /* name         addr  size [shadow:EALLOW] */
	/*
	 * Time-Base Submodule Registers
	 */
	volatile unsigned short tbctl;        /* TBCTL        0x0000 1 [No :   ] Time-Base Control Register */
#define	TBCTL_STOP_0		0			/* Stop after the next time-base counter increment or decrement */
#define	TBCTL_STOP_1		(1<<14)		/* Stop when counter completes a whole cycle */
#define	TBCTL_FREE			(2<<14)		/* Free run */
#define	TBCTL_PHSDIR		(1<<13)		/* Phase Direction Bit. */

#define	TBCTL_CLKDIV_1		(0<<10)		/* Time-base Clock Prescale (/1) */
#define	TBCTL_CLKDIV_2		(1<<10)		/* Time-base Clock Prescale (/2) */
#define	TBCTL_CLKDIV_4		(2<<10)		/* Time-base Clock Prescale (/4) */
#define	TBCTL_CLKDIV_8		(3<<10)		/* Time-base Clock Prescale (/8) */
#define	TBCTL_CLKDIV_16		(4<<10)		/* Time-base Clock Prescale (/16) */
#define	TBCTL_CLKDIV_32		(5<<10)		/* Time-base Clock Prescale (/32) */
#define	TBCTL_CLKDIV_64		(6<<10)		/* Time-base Clock Prescale (/64) */
#define	TBCTL_CLKDIV_128	(7<<10)		/* Time-base Clock Prescale (/128) */

#define	TBCTL_HSPCLKDIV_1	(0<<7)		/* High Speed Time-base Clock Prescale (/1) */
#define	TBCTL_HSPCLKDIV_2	(1<<7)		/* High Speed Time-base Clock Prescale (/2) */
#define	TBCTL_HSPCLKDIV_4	(2<<7)		/* High Speed Time-base Clock Prescale (/4) */
#define	TBCTL_HSPCLKDIV_6	(3<<7)		/* High Speed Time-base Clock Prescale (/6) */
#define	TBCTL_HSPCLKDIV_8	(4<<7)		/* High Speed Time-base Clock Prescale (/8) */
#define	TBCTL_HSPCLKDIV_10	(5<<7)		/* High Speed Time-base Clock Prescale (/10) */
#define	TBCTL_HSPCLKDIV_12	(6<<7)		/* High Speed Time-base Clock Prescale (/12) */
#define	TBCTL_HSPCLKDIV_14	(7<<7)		/* High Speed Time-base Clock Prescale (/14) */

#define	TBCTL_SYNCOSEL_EPWMxSYNC	(0<<4)	/* Synchronization Output Select : EPWMxSYNC */
#define	TBCTL_SYNCOSEL_CTR_ZERO		(1<<4)	/* Synchronization Output Select : CTR = zero */
#define	TBCTL_SYNCOSEL_CTR_CMPB		(2<<4)	/* Synchronization Output Select : CTR = CMPB */
#define	TBCTL_SYNCOSEL_DISABLE		(3<<4)	/* Synchronization Output Select : Disable */

#define	TBCTL_PRDLD			(1<<3)
#define	TBCTL_PHSEN			(1<<2)

#define	TBCTL_CTRMODE_UP		(0)
#define	TBCTL_CTRMODE_DOWN		(1)
#define	TBCTL_CTRMODE_UP_DOWN	(2)
#define	TBCTL_CTRMODE_STOP		(3)

	volatile unsigned short tbsts;        /* TBSTS        0x0001 1 [No :   ] Time-Base Status Register */
#define	TBSTS_CTRMAX	(1<<2)
#define	TBSTS_SYNCI		(1<<1)
#define	TBSTS_CTRDIR	(1<<0)

	volatile unsigned short tbphshr;      /* TBPHSHR      0x0002 1 [No :   ] Extension for HRPWM Phase Register (2) */
	volatile unsigned short tbphs;        /* TBPHS        0x0003 1 [No :   ] Time-Base Phase Register */
	volatile unsigned short tbctr;        /* TBCTR        0x0004 1 [No :   ] Time-Base Counter Register */
	volatile unsigned short tbprd;        /* TBPRD        0x0005 1 [Yes:   ] Time-Base Period Register */
	volatile unsigned short tbprdhr;      /* TBPRDHR      0x0006 1 [Yes:   ] Time Base Period High Resolution Register (3) */


	/*
	 * Counter-Compare Submodule Registers
	 */
	volatile unsigned short cmpctl;       /* CMPCTL       0x0007 1 [No :   ] Counter-Compare Control Register */
#define	CMPCTL_SHDWBFULL	(1<<9)	/* Counter-compare B (CMPB) Shadow Register Full Status Flag */
#define	CMPCTL_SHDWAFULL	(1<<8)	/* Counter-compare A (CMPA) Shadow Register Full Status Flag */
#define	CMPCTL_SHDWBMODE	(1<<6)	/* Counter-compare B (CMPB) Register Operating Mode */
#define	CMPCTL_SHDWAMODE	(1<<4)	/* Counter-compare A (CMPA) Register Operating Mode */

#define	CMPCTL_LOADBMODE_CTR_ZERO		(0<<2)		/* Load on CTR = Zero: Time-base counter equal to zero (TBCTR = 0x0000) */
#define	CMPCTL_LOADBMODE_CTR_PRD		(1<<2)		/* Load on CTR = PRD: Time-base counter equal to period (TBCTR = TBPRD) */
#define	CMPCTL_LOADBMODE_CTR_ZRO_PRD	(2<<2)		/* Load on either CTR = Zero or CTR = PRD */
#define	CMPCTL_LOADBMODE_FREEZE			(3<<2)		/* Freeze (no loads possible) */

#define	CMPCTL_LOADAMODE_CTR_ZERO		(0<<2)		/* Load on CTR = Zero: Time-base counter equal to zero (TBCTR = 0x0000) */
#define	CMPCTL_LOADAMODE_CTR_PRD		(1<<2)		/* Load on CTR = PRD: Time-base counter equal to period (TBCTR = TBPRD) */
#define	CMPCTL_LOADAMODE_CTR_ZRO_PRD	(2<<2)		/* Load on either CTR = Zero or CTR = PRD */
#define	CMPCTL_LOADAMODE_FREEZE			(3<<2)		/* Freeze (no loads possible) */


	volatile unsigned short cmpahr;       /* CMPAHR       0x0008 1 [Yes:   ] Extension for HRPWM Counter-Compare A Register (2) */
	volatile unsigned short cmpa;         /* CMPA         0x0009 1 [Yes:   ] Counter-Compare A Register */
	volatile unsigned short cmpb;         /* CMPB         0x000A 1 [Yes:   ] Counter-Compare B Register */


	/*
	 * Action-Qualifier Submodule Registers
	 */
	volatile unsigned short aqctla;       /* AQCTLA       0x000B 1 [No :   ] Action-Qualifier Control Register for Output A (EPWMxA) */
	volatile unsigned short aqctlb;       /* AQCTLB       0x000C 1 [No :   ] Action-Qualifier Control Register for Output B (EPWMxB) */

#define	AQCTLx_xxx_NOTHING	0		/* Do nothing (action disabled) */
#define	AQCTLx_xxx_CLEAR	1		/* Clear: force EPWMxA output low. */
#define	AQCTLx_xxx_SET		2		/* Set: force EPWMxA output high. */
#define	AQCTLx_xxx_TOGGLE	3		/* Toggle EPWMxA output */

#define	AQCTLx_CBD(_v)		((_v)<<10)
#define	AQCTLx_CBU(_v)		((_v)<<8)
#define	AQCTLx_CAD(_v)		((_v)<<6)
#define	AQCTLx_CAU(_v)		((_v)<<4)
#define	AQCTLx_PRD(_v)		((_v)<<2)
#define	AQCTLx_ZRO(_v)		( _v    )

	volatile unsigned short aqsfrc;       /* AQSFRC       0x000D 1 [No :   ] Action-Qualifier Software Force Register */

#define	AQSFRC_RLDCSF_ZERO			(0<<6)	/* Load on event counter equals zero */
#define	AQSFRC_RLDCSF_PERIOD		(1<<6)	/* Load on event counter equals period */
#define	AQSFRC_RLDCSF_ZERO_PERIOD	(2<<6)	/* Load on event counter equals zero or counter equals period */
#define	AQSFRC_RLDCSF_IMMEDIATELY	(3<<6)	/* Load immediately */

#define	AQSFRC_OTSFB		(1<<5)	/* One-Time Software Forced Event on Output B */
#define	AQSFRC_OTSFA		(1<<2)		/* One-Time Software Forced Event on Output A */

#define	AQSFRC_ACTSFx_NOTHING	0		/* Does nothing (action disabled) */
#define	AQSFRC_ACTSFx_CLEAR		1		/* Clear (low) */
#define	AQSFRC_ACTSFx_SET		2		/* Set (high) */
#define	AQSFRC_ACTSFx_TOGGLE	3		/* Toggle (Low -> High, High -> Low) */

#define	AQSFRC_ACTSFB(_v)	((_v)<<3)
#define	AQSFRC_ACTSFA(_v)	((_v)<<0)

	volatile unsigned short aqcsfrc;      /* AQCSFRC      0x000E 1 [Yes:   ] Action-Qualifier Continuous S/W Force Register Set */
#define	AQCSFCR_CSFx_DISABLED	0	/* Forcing disabled, i.e., has no effect */
#define	AQCSFCR_CSFx_LOW		1	/* Forces a continuous low on output x */
#define	AQCSFCR_CSFx_HIGH		2	/* Forces a continuous high on output x */
#define	AQCSFCR_CSFx_DISABLE	3	/* Software forcing is disabled and has no effect */

#define	AQCSFCR_CSFB(_v)		((_v)<<2)
#define	AQCSFCR_CSFA(_v)		((_v)<<0)


	/*
	 * Dead-Band Generator Submodule Registers
	 */
	volatile unsigned short dbctl;        /* DBCTL        0x000F 1 [No :   ] Dead-Band Generator Control Register */
#define	DBCTL_HALFCYCLE					(1<<15)		/* Half Cycle Clocking Enable Bit: */
#define	DBCTL_A_BOTH					(0<<4)		/* EPWMxA In (from the action-qualifier) is the source for both falling-edge
													 * and rising-edge delay.
													 */
#define	DBCTL_IN_MODE_A_FALL_B_RISE		(1<<4)		/* EPWMxB In (from the action-qualifier) is the source for rising-edge delayed signal.
													 * EPWMxA In (from the action-qualifier) is the source for falling-edge delayed signal.
													 */
#define	DBCTL_IN_MODE_A_RISE_B_FALL		(2<<4)		/* EPWMxA In (from the action-qualifier) is the source for rising-edge delayed signal.
													 * EPWMxB In (from the action-qualifier) is the source for falling-edge delayed signal.
													 */
#define	DBCTL_IN_MODE_B_BOTH			(3<<4)		/* EPWMxB In (from the action-qualifier) is the source for both rising-edge delay and falling-edge delayed signal. */
#define	DBCTL_POL_SEL_AH				(0<<2)		/* Active high (AH) mode. Neither EPWMxA nor EPWMxB is inverted (default). */
#define	DBCTL_POL_SEL_ALC				(1<<2)		/* 01 Active low complementary (ALC) mode. EPWMxA is inverted. */
#define	DBCTL_POL_SEL_AHC				(2<<2)		/* Active high complementary (AHC). EPWMxB is inverted. */
#define	DBCTL_POL_SEL_AL				(3<<2)		/* Active low (AL) mode. Both EPWMxA and EPWMxB are inverted. */
#define	DBCTL_OUT_MODE_BYPASS			(0<<0)		/* Dead-band generation is bypassed for both output signals. */
#define	DBCTL_OUT_MODE_FALL				(1<<0)		/* The falling-edge delayed signal is seen on output EPWMxB */
#define	DBCTL_OUT_MODE_RISE				(2<<0)		/* The rising-edge delayed signal is seen on output EPWMxA. */
#define	DBCTL_OUT_MODE_BOTH				(3<<0)		/* Dead-band is fully enabled for both rising-edge delay on output EPWMxA and falling-edge delay on output EPWMxB. */

	volatile unsigned short dbred;        /* DBRED        0x0010 1 [No :   ] Dead-Band Generator Rising Edge Delay Count Register */
	volatile unsigned short dbfed;        /* DBFED        0x0011 1 [No :   ] Dead-Band Generator Falling Edge Delay Count Register */


	/*
	 * Trip-Zone Submodule Registers
	 */
	volatile unsigned short tzsel;        /* TZSEL        0x0012 1 [   :Yes] Trip-Zone Select Register */
#define	TZSEL_DCBEVT1	(1<<15)			/* Digital Compare Output B Event 1 Select */
#define	TZSEL_DCAEVT1	(1<<14)			/* Digital Compare Output A Event 1 Select */
#define	TZSEL_OSHT6		(1<<13)			/* Trip-zone 6 (TZ6) Select */
#define	TZSEL_OSHT5		(1<<12)			/* Trip-zone 5 (TZ5) Select */
#define	TZSEL_OSHT4		(1<<11)			/* Trip-zone 4 (TZ4) Select */
#define	TZSEL_OSHT3		(1<<10)			/* Trip-zone 3 (TZ3) Select */
#define	TZSEL_OSHT2		(1<<9)			/* Trip-zone 2 (TZ2) Select */
#define	TZSEL_OSHT1		(1<<8)			/* Trip-zone 1 (TZ1) Select */
#define	TZSEL_DCBEVT2	(1<<7)			/* Digital Compare Output B Event 2 Select */
#define	TZSEL_DCAEVT2	(1<<6)			/* Digital Compare Output A Event 2 Select */
#define	TZSEL_CBC6		(1<<5)			/* Trip-zone 6 (TZ6) Select */
#define	TZSEL_CBC5		(1<<4)			/* Trip-zone 5 (TZ5) Select */
#define	TZSEL_CBC4		(1<<3)			/* Trip-zone 4 (TZ4) Select */
#define	TZSEL_CBC3		(1<<2)			/* Trip-zone 3 (TZ3) Select */
#define	TZSEL_CBC2		(1<<1)			/* Trip-zone 2 (TZ2) Select */
#define	TZSEL_CBC1		(1<<0)			/* Trip-zone 1 (TZ1) Select */

	volatile unsigned short tzdcsel;      /* TZDCSEL      0x0013 1 [   :Yes] Trip Zone Digital Compare Select Register */
#define	TZDCSEL_DCxEVTx_DISABLE		0	/* Event disabled */
#define	TZDCSEL_DCxEVTx_HL_Lx		1	/* DCAH = low, DCAL = don't care */
#define	TZDCSEL_DCxEVTx_HH_Lx		2	/* DCAH = high, DCAL = don't care */
#define	TXDCSEL_DCxEVTx_Hx_LL		3	/* DCAH = don't care, DCAL = low */
#define	TXDCSEL_DCxEVTx_Hx_LH		4	/* DCAH = don't care, DCAL = high */
#define	TXDCSEL_DCxEVTx_HL_LH		5	/* DCAH = low, DCAL = high */

#define	TXDCSEL_DCBEVT2(_v)		((_v)<<9)	/* Digital Compare Output B Event 2 Selection */
#define	TXDCSEL_DCBEVT1(_v)		((_v)<<6)	/* Digital Compare Output B Event 1 Selection */
#define	TXDCSEL_DCAEVT2(_v)		((_v)<<3)	/* Digital Compare Output A Event 2 Selection */
#define	TXDCSEL_DCAEVT1(_v)		((_v)<<0)	/* Digital Compare Output A Event 1 Selection */


	volatile unsigned short tzctl;        /* TZCTL        0x0014 1 [   :Yes] Trip-Zone Control Register (3) */
#define	TZCTL_xxxx_IMPEDANCE	0	/* High-impedance */
#define	TZCTL_xxxx_HIGH			1	/* Force to a high state. */
#define	TZCTL_xxxx_LOW			2	/* Force to a low state. */
#define	TZCTL_xxxx_DISABLE		3	/* Do Nothing, trip action is disabled */

#define	TZCTL_DCBEVT2(_v)	((_v)<<10)		/* Digital Compare Output B Event 2 Action On EPWMxB: */
#define	TZCTL_DCBEVT1(_v)	((_v)<<8)		/* Digital Compare Output B Event 1 Action On EPWMxB: */
#define	TZCTL_DCAEVT2(_v)	((_v)<<6)		/* Digital Compare Output A Event 2 Action On EPWMxA: */
#define	TZCTL_DCAEVT1(_v)	((_v)<<4)		/* Digital Compare Output A Event 1 Action On EPWMxA: */
#define	TZCTL_TZB(_v)		((_v)<<2)		/* When a trip event occurs the following action is taken on output EPWMxB. */
#define	TZCTL_TZA(_v)		((_v)<<0)		/* When a trip event occurs the following action is taken on output EPWMxA. */


	volatile unsigned short tzeint;       /* TZEINT       0x0015 1 [   :Yes] Trip-Zone Enable Interrupt Register (3) */
	volatile unsigned short tzflg;        /* TZFLG        0x0016 1 [   :   ]Trip-Zone Flag Register (3) */
	volatile unsigned short tzclr;        /* TZCLR        0x0017 1 [   :Yes] Trip-Zone Clear Register (3) */
	volatile unsigned short tzfrc;        /* TZFRC        0x0018 1 [   :Yes] Trip-Zone Force Register (3) */
#define	TZ_INT_DCBEVT2		(1<<6)		/* Digital Comparator Output B Event 2 Interrupt */
#define	TZ_INT_DCBEVT1		(1<<5)		/* Digital Comparator Output B Event 1 Interrupt */
#define	TZ_INT_DCAEVT2		(1<<4)		/* Digital Comparator Output A Event 2 Interrupt */
#define	TZ_INT_DCAEVT1		(1<<3)		/* Digital Comparator Output A Event 1 Interrupt */
#define	TZ_INT_OST			(1<<2)		/* Trip-zone One-Shot Interrupt */
#define	TZ_INT_CBC			(1<<1)		/* Trip-zone Cycle-by-Cycle Interrupt */
#define	TZ_INT				(1<<0)		/* Latched Trip Interrupt */

	/*
	 * Event-Trigger Submodule Registers
	 */
	volatile unsigned short etsel;        /* ETSEL        0x0019 1 [   :   ] Event-Trigger Selection Register */
#define	ETSEL_SOCBEN		(1<<15)		/* Enable the ADC Start of Conversion B (EPWMxSOCB) Pulse */
#define	ETSEL_SOCAEN		(1<<11)		/* Enable the ADC Start of Conversion A (EPWMxSOCA) Pulse */
#define	ETSEL_INTEN		(1<<3)		/* Enable ePWM Interrupt (EPWMx_INT) Generation */

#define	ETSEL_xxxSEL_DCBEVT1				0		/* Enable DCBEVT1.soc event */
#define	ETSEL_xxxSEL_COUNTER_ZERO			1		/* Enable event time-base counter equal to zero. (TBCTR = 0x0000) */
#define	ETSEL_xxxSEL_COUNTER_PERIOD		2		/* Enable event time-base counter equal to period (TBCTR = TBPRD) */
#define	ETSEL_xxxSEL_COUNTER_ZERO_PERIOD	3		/* Enable event time-base counter equal to zero or period (TBCTR = 0x0000 or TBCTR = TBPRD). */
#define	ETSEL_xxxSEL_CMPA_INC				4		/* Enable event time-base counter equal to CMPA when the timer is incrementing. */
#define	ETSEL_xxxSEL_CMPA_DEC				5		/* Enable event time-base counter equal to CMPA when the timer is decrementing. */
#define	ETSEL_xxxSEL_CMPB_INC				6		/* Enable event: time-base counter equal to CMPB when the timer is incrementing. */
#define	ETSEL_xxxSEL_CMPB_DEC				7		/* Enable event: time-base counter equal to CMPB when the timer is decrementing. */

#define	ETSEL_SOCBSEL(_v)				((_v)<<12)	/* EPWMxSOCB Selection Options */
#define	ETSEL_SOCASEL(_v)				((_v)<<8)	/* EPWMxSOCA Selection Options */
#define	ETSEL_INTSEL(_v)				((_v)<<0)	/* ePWM Interrupt (EPWMx_INT) Selection Options */

	volatile unsigned short etps;         /* ETPS         0x001A 1 [   :   ] Event-Trigger Pre-Scale Register */
#define	ETPS_SOCBCNT(_reg)				(((_reg)>>14)&0x03)	/* ePWM ADC Start-of-Conversion B Event (EPWMxSOCB) Counter Register */
#define	ETPS_SOCBPRD(_v)				((_v)<<12)			/* ePWM ADC Start-of-Conversion B Event (EPWMxSOCB) Period Select */
#define	ETPS_SOCACNT(_reg)				(((_reg)>>10)&0x03)	/* ePWM ADC Start-of-Conversion A Event (EPWMxSOCA) Counter Register */
#define	ETPS_SOCAPRD(_v)				((_v)<<8)			/* ePWM ADC Start-of-Conversion A Event (EPWMxSOCA) Period Select */
#define	ETPS_INTCNT(_reg)				(((_reg)>>2)&0x03)	/* ePWM Interrupt Event (EPWMx_INT) Counter Register */
#define	ETPS_INTPRD(_v)					((_v)<<0)			/* ePWM Interrupt (EPWMx_INT) Period Select */

	volatile unsigned short etflg;        /* ETFLG        0x001B 1 [   :   ] Event-Trigger Flag Register */
	volatile unsigned short etclr;        /* ETCLR        0x001C 1 [   :   ] Event-Trigger Clear Register */
	volatile unsigned short etfrc;        /* ETFRC        0x001D 1 [   :   ] Event-Trigger Force Register */
#define	ETxxx_SOCB		(1<<3)		/* Latched ePWM ADC Start-of-Conversion B (EPWMxSOCB) Status Flag */
#define	ETxxx_SOCA		(1<<2)		/* Latched ePWM ADC Start-of-Conversion A (EPWMxSOCA) Status Flag */
#define	ETxxx_INT		(1<<0)		/* Latched ePWM Interrupt (EPWMx_INT) Status Flag */


	/*
	 * PWM-Chopper Submodule Registers
	 */
	volatile unsigned short pcctl;        /* PCCTL        0x001E 1 [   :   ] PWM-Chopper Control Register */
#define	PCCTL_CHPDUTY(_v)		((_v)<<8)		/* CHPDUTY Chopping Clock Duty Cycle */
#define	PCCTL_CHPFREQ(_v)		((_v)<<5)		/* Chopping Clock Frequency */
#define	PCCTL_OSHTWTH(_v)		((_v)<<1)		/* One-Shot Pulse Width */
#define	PCCTLCHPEN				(1<<0)			/* PWM-chopping Enable */

	         unsigned short __dummy_1f__;

	/*
	 * High-Resolution Pulse Width Modulator (HRPWM) Extension Registers
	 */
	volatile unsigned short hrcnfg;       /* HRCNFG       0x0020 1 [   :Yes] HRPWM Configuration Register (2) (3) */
	volatile unsigned short hrpwr;        /* HRPWR        0x0021 1 [   :Yes] HRPWM Power Register (3) (4) */
	         unsigned short __dumy_22__;
	         unsigned short __dumy_23__;
	         unsigned short __dumy_24__;
	         unsigned short __dumy_25__;
	volatile unsigned short hrmstep;      /* HRMSTEP      0x0026 1 [   :Yes] HRPWM MEP Step Register(3) (4) */
	         unsigned short __dumy_27__;
	volatile unsigned short hrpctl;       /* HRPCTL       0x0028 1 [   :Yes] High Resolution Period Control Register(3) */
#define	HRPCTL_TBPHSHRLOADE		(1<<2)	/* TBPHSHR Load Enable */
#define	HRPCTL_HRPE				(1<<0)	/* TBPHSHR Load Enable */

	         unsigned short __dumy_29__;
	volatile unsigned short tbprdhrm;     /* TBPRDHRM     0x002A 1 [Writes:] Time Base Period High Resolution Register Mirror(3) */
	volatile unsigned short tbprdm;       /* TBPRDM       0x002B 1 [Writes:] Time Base Period Register Mirror */
	volatile unsigned short cmpahrm;      /* CMPAHRM      0x002C 1 [Writes:] Compare A High Resolution Register Mirror(3) */
	volatile unsigned short cmpam;        /* CMPAM        0x002D 1 [Writes:] Compare A Register Mirror */
	         unsigned short __dumy_2e__;
	         unsigned short __dumy_2f__;

	/*
	 * Digital Compare Event Registers
	 */
	volatile unsigned short dctripsel;    /* DCTRIPSEL    0x0030 1 [   :Yes] Digital Compare Trip Select Register */
#define	DCTRIPSEL_DCxxCOMPSEL_TZ1		0	/* TZ1 input */
#define	DCTRIPSEL_DCxxCOMPSEL_TZ2		1	/* TZ2 input */
#define	DCTRIPSEL_DCxxCOMPSEL_TZ3		2	/* TZ3 input */
#define	DCTRIPSEL_DCxxCOMPSEL_COM1OUT	8	/* COMP1OUT input */
#define	DCTRIPSEL_DCxxCOMPSEL_COM2OUT	9	/* COMP2OUT input */
#define	DCTRIPSEL_DCxxCOMPSEL_COM3OUT	10	/* COMP3OUT input */

#define	DCTRIPSEL_DCBLCOMPSEL(_v)		((_v)<<12)		/* Digital Compare B Low Input Select */
#define	DCTRIPSEL_DCBHCOMPSEL(_v)		((_v)<<8)		/* Digital Compare B High Input Select */
#define	DCTRIPSEL_DCALCOMPSEL(_v)		((_v)<<4)		/* Digital Compare A Low Input Select */
#define	DCTRIPSEL_DCAHCOMPSEL(_v)		((_v)<<0)		/* Digital Compare A High Input Select */

	volatile unsigned short dcactl;       /* DCACTL       0x0031 1 [   :Yes] Digital Compare A Control Register */
	volatile unsigned short dcbctl;       /* DCBCTL       0x0032 1 [   :Yes] Digital Compare B Control Register */
#define	DCxCTL_EVT2FRCSYNCSEL	(1<<9)		/* DCxEVT2 Force Synchronization Signal Select */
#define	DCxCTL_EVT2SRCSEL		(1<<8)		/* DCxEVT2 Source Signal Select */
#define	DCxCTL_EVT1SYNCE		(1<<3)		/* DCxEVT1 SYNC, Enable/Disable */
#define	DCxCTL_EVT1SOCE			(1<<2)		/* DCxEVT1 SOC, Enable/Disable */
#define	DCxCTL_EVT1FRCSYNCSEL	(1<<1)		/* DCxEVT1 Force Synchronization Signal Select */
#define	DCxCTL_EVT1SRCSEL		(1<<0)		/* DCxEVT1 Source Signal Select */


	volatile unsigned short dcfctl;       /* DCFCTL       0x0033 1 [   :Yes] Digital Compare Filter Control Register */
#define	DCFCTL_PULSESEL_PERIOD	(0<<4)		/* Time-base counter equal to period (TBCTR = TBPRD) */
#define	DCFCTL_PULSESEL_ZERO	(1<<4)		/* Time-base counter equal to zero (TBCTR = 0x0000) */
#define	DCFCTL_BLANKINV			(1<<3)		/* Blanking Window Inversion */
#define	DCFCTL_BLANKE			(1<<2)		/* Blanking Window Enable/Disable */
#define	DCFCTL_DCAEVT1			(0<<0)		/* Source Is DCAEVT1 Signal */
#define	DCFCTL_DCAEVT2			(1<<0)		/* Source Is DCAEVT2 Signal */
#define	DCFCTL_DCBEVT1			(2<<0)		/* Source Is DCBEVT1 Signal */
#define	DCFCTL_DCBEVT2			(3<<0)		/* Source Is DCBEVT2 Signal */

	volatile unsigned short dccapctl;     /* DCCAPCTL     0x0034 1 [   :Yes] Digital Compare Capture Control Register */
#define	DCCAPCTL_SHDWMODE		(1<<1)		/* TBCTR Counter Capture Shadow Select Mode */
#define	DCCAPCTL_CAPE		(1<<0)		/* TBCTR Counter Capture Enable/Disable */

	volatile unsigned short dcfoffset;    /* DCFOFFSET    0x0035 1 [Writes:] Digital Compare Filter Offset Register */
	volatile unsigned short dcfoffsetcnt; /* DCFOFFSETCNT 0x0036 1 [   :   ] Digital Compare Filter Offset Counter Register */
	volatile unsigned short dcfwindow;    /* DCFWINDOW    0x0037 1 [   :   ] Digital Compare Filter Window Register */
	volatile unsigned short dcfwindowcnt; /* DCFWINDOWCNT 0x0038 1 [   :   ] Digital Compare Filter Window Counter Register */
	volatile unsigned short dccap;        /* DCCAP        0x0039 1 [Yes:   ] Digital Compare Counter Capture Register */
} PwmReg;

typedef struct freq_t {
	long	freqMin;
	long	freqMax;
	float	freq;
	short	bActive;
	unsigned short tbctl_div_val;
} Freq;

typedef struct pwm_chan_t {
	int	st;
#define	ST_STOP		0x00
#define	ST_RUN_A	0x01
#define	ST_RUN_B	0x02

	int	ch;
	int	gpioNoBase;
	int	gpioNo;
//	int vecNo;

	long	freq;
	const Freq * pFreq;
	unsigned short tbprdVal;
	float			fTbprdVal;

	float	duty;

	PwmReg * pReg;

	int timerID;

} PwmChan;

/*** FUNCTION ***/
//static void pwm_stop( PwmChan * pPwm );
static void pwm_stop( void * arg );

/*** WORK ***/
static const unsigned short ch_addr[PWM_CH_NUM] =
{
	PWM1_ADRS,
	PWM2_ADRS,
	PWM3_ADRS,
	PWM4_ADRS,
	PWM5_ADRS,
	PWM6_ADRS,
	PWM7_ADRS,
};

#define	ST_INIT		0x0000
#define	ST_CH_INIT	0x0001



static const Freq freqTbl[] = {
	/* freqMin, freqMax,    freq,                        bActive, tbctl_div_val, */
	{ 200001,   0x7fffffff, 0.0,                         0,       },		/* limit freq */
	{   1000,   200000,     ((float)SYSCLK/( 1*  1)),    1,       ( TBCTL_HSPCLKDIV_1  | TBCTL_CLKDIV_1   ), },		/* 1KHz - 200KHz  */
	{    300,      999,     ((float)SYSCLK/( 4*  1)),    1,       ( TBCTL_HSPCLKDIV_4  | TBCTL_CLKDIV_1   ), },		/*  300Hz -   999Hz */
	{     60,      299,     ((float)SYSCLK/( 8*  2)),    1,       ( TBCTL_HSPCLKDIV_8  | TBCTL_CLKDIV_2   ), },		/*   60Hz -   299Hz */
	{     15,       59,     ((float)SYSCLK/(10*  8)),    1,       ( TBCTL_HSPCLKDIV_10 | TBCTL_CLKDIV_8   ), },		/*   15Hz -    59Hz */
	{      3,       14,     ((float)SYSCLK/(10* 32)),    1,       ( TBCTL_HSPCLKDIV_10 | TBCTL_CLKDIV_32  ), },		/*    3Hz -    14Hz */
	{      1,        2,     ((float)SYSCLK/(14*128)),    1,       ( TBCTL_HSPCLKDIV_14 | TBCTL_CLKDIV_128 ), },		/*    1Hz -     2Hz */
	{      0,        0,     0.0,                         0,       },		/* limit freq */
};


static PwmChan pwmChan[PWM_CH_NUM];

/*******************************************************************************
 *
 */
int pwm_init( void )
{
	int ch;
	PwmChan * pPwm;

	pwmChan[0].timerID = SYSTIMER_PWM_0;
	pwmChan[1].timerID = SYSTIMER_PWM_1;
	pwmChan[2].timerID = SYSTIMER_PWM_2;
	pwmChan[3].timerID = SYSTIMER_PWM_3;
	pwmChan[4].timerID = SYSTIMER_PWM_4;
	pwmChan[5].timerID = SYSTIMER_PWM_5;
	pwmChan[6].timerID = SYSTIMER_PWM_6;

	for( ch=0; ch<PWM_CH_NUM; ch++ ) {
		pPwm = &pwmChan[ch];

		pPwm->st        = ST_STOP;
		pPwm->ch        = ch;
		if( ch < 6 ) {
			pPwm->gpioNoBase = ch * 2;
		} else {
			pPwm->gpioNoBase = 40;
		}

		pPwm->pFreq     = NULL;
		pPwm->tbprdVal  = 0;
		pPwm->fTbprdVal = 0.0;
		pPwm->freq      = 0;
		pPwm->duty      = 0.0;
		pPwm->pReg = (PwmReg *)ch_addr[ch];

		sysTimerConnect( pPwm->timerID, pwm_stop, (void *)pPwm );
	}


	return 0;
}


/*******************************************************************************
 *
 */
int pwm( int ch, int bSideB, long freq, float duty, unsigned int mSec )
{
	PwmChan * pPwm;
	const Freq    * pFreq;
	unsigned short cmpxVal;
	unsigned short aqctlxVal;

	int retVal = -1;
	int result;
	int i;

	if( (ch<0) || (PWM_CH_NUM<=ch) ) {
		goto abort;
	}

	pPwm = &pwmChan[ch];
	pFreq = pPwm->pFreq;

	result = 0;
	if( bSideB ) {
		bSideB = 1;		/* Normalize */
		if( pPwm->st & ST_RUN_A ) {
			result = -1;
		}
	} else {
		if( pPwm->st & ST_RUN_B ) {
			result = -1;
		}
	}

	if( result ) {
		goto abort;
	}

	pPwm->gpioNo = pPwm->gpioNoBase + bSideB;

	/*
	 * start configuration
	 */
	pwm_stop( (void *)pPwm );

	if( freq <= 0 ) {
		/* stop */

		pPwm->st = ST_STOP;
		pPwm->pFreq = NULL;
		pPwm->tbprdVal = 0;
		pPwm->fTbprdVal = 0.0;
		pPwm->freq = 0;
		pPwm->duty = 0.0;

		retVal = 0;
		goto exit;
	}

	/*
	 * frequency
	 */
	if( freq != pPwm->freq ) {
		if( pFreq ) {
			if( (freq < pFreq->freqMin) || (pFreq->freqMax < freq) ) {
				pFreq = NULL;
			}
		}

		for( i=0; pFreq == NULL; i++ ) {
			if( (freqTbl[i].freqMin <= freq) && (freq <= freqTbl[i].freqMax) ) {
				pFreq = &freqTbl[i];
				if( pFreq->bActive == 0 ) {
					goto abort;
				}
				pPwm->pFreq = pFreq;
			}
		}

		pPwm->fTbprdVal = pFreq->freq / (float)(freq);
		pPwm->tbprdVal  = (unsigned short)pPwm->fTbprdVal;
	}

	/*
	 * duty
	 */
	if( duty >= 100.0 ) {
		cmpxVal = pPwm->tbprdVal;
		aqctlxVal = AQCTLx_CBD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CBU(AQCTLx_xxx_SET)
		          | AQCTLx_CAD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CAU(AQCTLx_xxx_NOTHING)
		          | AQCTLx_PRD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_ZRO(AQCTLx_xxx_SET)
		          ;
	} else if( duty > 0.0 ) {
		cmpxVal = (unsigned short)( (pPwm->fTbprdVal * duty / 100.0) + 0.5 );
		if( cmpxVal >= pPwm->tbprdVal ) {
			cmpxVal = pPwm->tbprdVal;
		}
		aqctlxVal = AQCTLx_CBD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CBU(AQCTLx_xxx_CLEAR)
		          | AQCTLx_CAD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CAU(AQCTLx_xxx_NOTHING)
		          | AQCTLx_PRD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_ZRO(AQCTLx_xxx_SET)
		          ;
	} else {
		cmpxVal = 0;
		aqctlxVal = AQCTLx_CBD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CBU(AQCTLx_xxx_CLEAR)
		          | AQCTLx_CAD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_CAU(AQCTLx_xxx_NOTHING)
		          | AQCTLx_PRD(AQCTLx_xxx_NOTHING)
		          | AQCTLx_ZRO(AQCTLx_xxx_CLEAR)
		          ;
	}

	/*
	 * (re)configuration and (re)start PWM
	 */
	pPwm->pReg->tbprd = pPwm->tbprdVal;
	pPwm->pReg->tbphs = 0;
	pPwm->pReg->tbctr = 0;
	pPwm->pReg->cmpa  = cmpxVal;
	pPwm->pReg->cmpb  = cmpxVal;
	pPwm->pReg->aqctla = aqctlxVal;
	pPwm->pReg->aqctlb = aqctlxVal;
	pPwm->pReg->cmpctl = 0
	                   |CMPCTL_LOADAMODE_CTR_ZERO
	                   |CMPCTL_LOADBMODE_CTR_ZERO
	                   ;
	pPwm->pReg->tbctl = pFreq->tbctl_div_val
	                  | TBCTL_FREE
	                  | TBCTL_CTRMODE_UP
//	                  | TBCTL_PHSEN
//	                  | TBCTL_PRDLD
	                  | TBCTL_SYNCOSEL_DISABLE
	                  ;

	if( gpioConfig( pPwm->gpioNo, GPIO_MODE_1 | GPIO_MODE_OUT ) ) {
		goto abort;
	}

	if( mSec ) {
		sysTimerSet( pPwm->timerID, mSec );
	}

	retVal = 0;
	pPwm->freq = freq;
	pPwm->duty = duty;
	if( bSideB ) {
		pPwm->st = ST_RUN_B;
	} else {
		pPwm->st = ST_RUN_A;
	}

abort:
exit:
	return retVal;
}

/*******************************************************************************
 *
 */
static void pwm_stop( void * arg )
{
	PwmChan * pPwm = (PwmChan *)arg;
	sysTimerClr( pPwm->timerID );

	/*
	 * output low
	 */
	gpioClear( pPwm->gpioNo );
	gpioConfig( pPwm->gpioNo, GPIO_MODE_GPIO | GPIO_MODE_OUT );

	/*
	 * stop pwm
	 */
	pPwm->pReg->tbctl = TBCTL_STOP_0					/* set reset value */
//	                | TBCTL_PHSDIR
	                | TBCTL_CLKDIV_1
	                | TBCTL_CLKDIV_2
	                | TBCTL_SYNCOSEL_EPWMxSYNC
	                | TBCTL_CTRMODE_STOP
	                ;

}

/*** end of file : pwmDrv.c ***/
