/*
  HardwareSerial.cpp - Hardware serial library for Wiring
  Copyright (c) 2006 Nicholas Zambetti.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  Modified 23 November 2006 by David A. Mellis
  Modified 28 September 2010 by Mark Sproul
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <inttypes.h>

//#include "wiring.h"
//#include "wiring_private.h"

#include "tms320f28035/sciDrv.h"

#include "HardwareSerial.h"


/*******************************************************************************
 *
 */
HardwareSerial::HardwareSerial( )
{
	/* nothing todo */
//	sciDrvInit(  60000000/4 );
}

/*******************************************************************************
 *
 */
int HardwareSerial::begin( long bps )
{
	int retVal;

	retVal = -1;

	if( sciDrvInit( 60000000/4 ) ) {
		goto abort;
	}
	if( sciCtrl( bps, 8, 'N', 1 ) ) {
		goto abort;
	}

	if( sciOpen() == 0 ) {
		retVal = 0;
	}
abort:
	return retVal;
}

/*******************************************************************************
 *
 */
void HardwareSerial::end()
{
	sciClose();
}

/*******************************************************************************
 *
 */
int HardwareSerial::available(void)
{
	return sciRxSize();
}

/*******************************************************************************
 *
 */
int HardwareSerial::peek(void)
{
	return sciRxPeek();
}

/*******************************************************************************
 *
 */
int HardwareSerial::read(void)
{
	unsigned char rxData;

	if( sciRx( &rxData, 1, 0 ) != 1 ) {
		return -1;
	}

	return (int)rxData;

}

/*******************************************************************************
 *
 */
void HardwareSerial::flush()
{
	sciRxFlush();
}

/*******************************************************************************
 *
 */
void HardwareSerial::write(uint8_t c)
{
	sciTx( &c, 1, 1 );
}

// Preinstantiate Objects //////////////////////////////////////////////////////
/*******************************************************************************
 *
 */
HardwareSerial Serial;


