/*
 WCharacter.h - Character utility functions for Wiring & Arduino
 Copyright (c) 2010 Hernando Barragan.  All right reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef Character_h
#define Character_h

#include <ctype.h>

#if 1
/*******************************************************************************
 * Checks for an alphanumeric character.
 * It is equivalent to (isalpha(c) || isdigit(c)).
 */
#ifdef	_INLINE
static inline boolean isAlphaNumeric(int c) { return ( isalnum(c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isAlphaNumeric(int c);
#else
boolean isAlphaNumeric(int c) { return ( isalnum(c) == 0 ? false : true); }
#endif
#endif


/*******************************************************************************
 * Checks for an alphabetic character.
 * It is equivalent to (isupper(c) || islower(c)).
 */
#ifdef	_INLINE
static inline boolean isAlpha(int c)
{
	return ( isalpha(c) == 0 ? false : true);
}
#else
#ifndef	WCharacter_C
inline boolean isAlpha(int c);
#else
inline boolean isAlpha(int c)
{
	return ( isalpha(c) == 0 ? false : true);
}
#endif
#endif

/*******************************************************************************
 * Checks whether c is a 7-bit unsigned char value
 * that fits into the ASCII character set.
 */
#ifdef	_INLINE
static inline boolean isAscii(int c) { return ( isascii (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isAscii(int c);
#else
boolean isAscii(int c) { return ( isascii (c) == 0 ? false : true); }
#endif
#endif


/*******************************************************************************
 * Checks for a blank character, that is, a space or a tab.
 */
#ifdef	_INLINE
static inline boolean isWhitespace(int c) { return ( (c==' ') || (c=='\t') ); }
#else
#ifndef	WCharacter_C
boolean isWhitespace(int c);
#else
boolean isWhitespace(int c) { return ( (c==' ') || (c=='\t') ); };
#endif
#endif

/*******************************************************************************
 * Checks for a control character.
 */
#ifdef	_INLINE
static inline boolean isControl(int c) { return ( iscntrl (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isControl(int c);
#else
boolean isControl(int c) { return ( iscntrl (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for a digit (0 through 9).
 */
#ifdef	_INLINE
static inline boolean isDigit(int c) { return ( isdigit (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isDigit(int c);
#else
boolean isDigit(int c) { return ( isdigit (c) == 0 ? false : true); }
#endif
#endif


/*******************************************************************************
 * Checks for any printable character except space.
 */
#ifdef	_INLINE
static inline boolean isGraph(int c) { return ( isgraph (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isGraph(int c);
#else
boolean isGraph(int c) { return ( isgraph (c) == 0 ? false : true); }
#endif
#endif


/*******************************************************************************
 * Checks for a lower-case character.
 */
#ifdef	_INLINE
static inline boolean isLowerCase(int c) { return (islower (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isLowerCase(int c);
#else
boolean isLowerCase(int c) { return (islower (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for any printable character including space.
 */
#ifdef	_INLINE
static inline boolean isPrintable(int c) { return ( isprint (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isPrintable(int c);
#else
boolean isPrintable(int c) { return ( isprint (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for any printable character which is not a space
 * or an alphanumeric character.
 */
#ifdef	_INLINE
static inline boolean isPunct(int c) { return ( ispunct (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isPunct(int c);
#else
boolean isPunct(int c) { return ( ispunct (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for white-space characters. For the avr-libc library,
 * these are: space, formfeed ('\f'), newline ('\n'), carriage
 * return ('\r'), horizontal tab ('\t'), and vertical tab ('\v').
 */
#ifdef	_INLINE
static inline boolean isSpace(int c) { return ( isspace (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isSpace(int c);
#else
boolean isSpace(int c) { return ( isspace (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for an uppercase letter.
 */
#ifdef	_INLINE
static inline boolean isUpperCase(int c) { return ( isupper (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isUpperCase(int c);
#else
boolean isUpperCase(int c) { return ( isupper (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Checks for a hexadecimal digits, i.e. one of 0 1 2 3 4 5 6 7
 * 8 9 a b c d e f A B C D E F.
 */
#ifdef	_INLINE
static inline boolean isHexadecimalDigit(int c) { return ( isxdigit (c) == 0 ? false : true); }
#else
#ifndef	WCharacter_C
boolean isHexadecimalDigit(int c);
#else
boolean isHexadecimalDigit(int c) { return ( isxdigit (c) == 0 ? false : true); }
#endif
#endif

/*******************************************************************************
 * Converts c to a 7-bit unsigned char value that fits into the
 * ASCII character set, by clearing the high-order bits.
 */
#ifdef	_INLINE
static inline int scii(int c) { return toascii (c); }
#else
#ifndef	WCharacter_C
int scii(int c);
#else
int scii(int c) { return toascii (c); }
#endif
#endif

/*******************************************************************************
 * Converts the letter c to lower case, if possible.
 * Warning:
 * Many people will be unhappy if you use this function.
 * This function will convert accented letters into random
 * characters.
 *
 */
#ifdef	_INLINE
static inline int owerCase(int c) { return tolower (c); }
#else
#ifndef	WCharacter_C
int owerCase(int c);
#else
int owerCase(int c) { return tolower (c); }
#endif
#endif

/*******************************************************************************
 * Converts the letter c to upper case, if possible.
 */
#ifdef	_INLINE
static inline int toUpperCase(int c) { return toupper (c); }
#else
#ifndef	WCharacter_C
int toUpperCase(int c);
#else
int toUpperCase(int c) { return toupper (c); }
#endif
#endif


#else
// WCharacter.h prototypes
inline boolean isAlphaNumeric(int c) __attribute__((always_inline));
inline boolean isAlpha(int c) __attribute__((always_inline));
inline boolean isAscii(int c) __attribute__((always_inline));
inline boolean isWhitespace(int c) __attribute__((always_inline));
inline boolean isControl(int c) __attribute__((always_inline));
inline boolean isDigit(int c) __attribute__((always_inline));
inline boolean isGraph(int c) __attribute__((always_inline));
inline boolean isLowerCase(int c) __attribute__((always_inline));
inline boolean isPrintable(int c) __attribute__((always_inline));
inline boolean isPunct(int c) __attribute__((always_inline));
inline boolean isSpace(int c) __attribute__((always_inline));
inline boolean isUpperCase(int c) __attribute__((always_inline));
inline boolean isHexadecimalDigit(int c) __attribute__((always_inline));
inline int toAscii(int c) __attribute__((always_inline));
inline int toLowerCase(int c) __attribute__((always_inline));
inline int toUpperCase(int c)__attribute__((always_inline));


// Checks for an alphanumeric character.
// It is equivalent to (isalpha(c) || isdigit(c)).
inline boolean isAlphaNumeric(int c)
{
  return ( isalnum(c) == 0 ? false : true);
}


// Checks for an alphabetic character.
// It is equivalent to (isupper(c) || islower(c)).
inline boolean isAlpha(int c)
{
  return ( isalpha(c) == 0 ? false : true);
}


// Checks whether c is a 7-bit unsigned char value
// that fits into the ASCII character set.
inline boolean isAscii(int c)
{
  return ( isascii (c) == 0 ? false : true);
}


// Checks for a blank character, that is, a space or a tab.
inline boolean isWhitespace(int c)
{
  return ( isblank (c) == 0 ? false : true);
}


// Checks for a control character.
inline boolean isControl(int c)
{
  return ( iscntrl (c) == 0 ? false : true);
}


// Checks for a digit (0 through 9).
inline boolean isDigit(int c)
{
  return ( isdigit (c) == 0 ? false : true);
}


// Checks for any printable character except space.
inline boolean isGraph(int c)
{
  return ( isgraph (c) == 0 ? false : true);
}


// Checks for a lower-case character.
inline boolean isLowerCase(int c)
{
  return (islower (c) == 0 ? false : true);
}


// Checks for any printable character including space.
inline boolean isPrintable(int c)
{
  return ( isprint (c) == 0 ? false : true);
}


// Checks for any printable character which is not a space
// or an alphanumeric character.
inline boolean isPunct(int c)
{
  return ( ispunct (c) == 0 ? false : true);
}


// Checks for white-space characters. For the avr-libc library,
// these are: space, formfeed ('\f'), newline ('\n'), carriage
// return ('\r'), horizontal tab ('\t'), and vertical tab ('\v').
inline boolean isSpace(int c)
{
  return ( isspace (c) == 0 ? false : true);
}


// Checks for an uppercase letter.
inline boolean isUpperCase(int c)
{
  return ( isupper (c) == 0 ? false : true);
}


// Checks for a hexadecimal digits, i.e. one of 0 1 2 3 4 5 6 7
// 8 9 a b c d e f A B C D E F.
inline boolean isHexadecimalDigit(int c)
{
  return ( isxdigit (c) == 0 ? false : true);
}


// Converts c to a 7-bit unsigned char value that fits into the
// ASCII character set, by clearing the high-order bits.
inline int toAscii(int c)
{
  return toascii (c);
}


// Warning:
// Many people will be unhappy if you use this function.
// This function will convert accented letters into random
// characters.

// Converts the letter c to lower case, if possible.
inline int toLowerCase(int c)
{
  return tolower (c);
}


// Converts the letter c to upper case, if possible.
inline int toUpperCase(int c)
{
  return toupper (c);
}
#endif

#endif
