/*** f_int.cpp - fpalette interrupt control (sysLib rapper) ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/04/xx mimu
              First Version
 */

#include "sysLib.h"
#include "tms320f28035/gpioDrv.h"
#include "f_int.h"
#include "f_const.h"

//extern "C" {
//void xint1_isr( unsigned int vecNo );
//void xint2_isr( unsigned int vecNo );
//void xint3_isr( unsigned int vecNo );
//}

static struct xint_t {
	int bActive;
	int gpio_mode;
	int gpioNo;
	int vecNo;
} xintTbl[3] = {
	{ 0, GPIO_MODE_XINT(1), 0, 35,  },
	{ 0, GPIO_MODE_XINT(2), 0, 36,  },
	{ 0, GPIO_MODE_XINT(3), 0, 120, },
};

#if 0
/*******************************************************************************
 *
 */
void xint1_isr( unsigned int vecNo )
{
	int xint = 0;

	if( xintTbl[xint].pFunc ) {
		xintTbl[xint].pFunc();
	}
}

/*******************************************************************************
 *
 */
void xint2_isr( unsigned int vecNo )
{
	int xint = 1;

	if( xintTbl[xint].pFunc ) {
		xintTbl[xint].pFunc();
	}
}

/*******************************************************************************
 *
 */
void xint3_isr( unsigned int vecNo )
{
	int xint = 2;

	if( xintTbl[xint].pFunc ) {
		xintTbl[xint].pFunc();
	}
}
#endif

/*******************************************************************************
 *
 */
void attachInterrupt(uint8_t no, void (*pFunc)(unsigned int), int mode)
{
	int xint;
	int result;

	if( no >= 32 ) {
		return ;
	}


	if( pFunc ) {
		/* attach */
		switch( mode ) {
		case RISING:  mode = GPIO_MODE_XINTn_FALL; break;
		case FALLING: mode = GPIO_MODE_XINTn_FALL; break;
		case BOTH:    mode = GPIO_MODE_XINTn_BOTH; break;
		default: return;
		}

		for( xint=0; xint<3; xint ++ ){
			if( xintTbl[xint].bActive == 0 ) {
				xintTbl[xint].bActive = 1;
				xintTbl[xint].gpioNo = no;
//				xintTbl[xint].pFunc = pFunc;
//				if( sysIntConnect( xintTbl[xint].vecNo, xintTbl[xint].pIsr ) ) {
				if( sysIntConnect( xintTbl[xint].vecNo, pFunc ) ) {
					return ;
				}
				result = gpioConfig( no, ( GPIO_MODE_GPIO
				                         | GPIO_MODE_IN
				                         | xintTbl[xint].gpio_mode
				                         | mode
				                         ) );
				if( result ) {
					return;
				}
				sysIntEnable( xintTbl[xint].vecNo );
				return;
			}
		}
	} else {
		/* detach */
		for( xint=0; xint<3; xint ++ ) {
			if( (xintTbl[xint].bActive) && (xintTbl[xint].gpioNo==no) ) {
				gpioConfig( no, ( GPIO_MODE_GPIO
				                | GPIO_MODE_IN
				                | xintTbl[xint].gpio_mode
				                ) );
				sysIntDisable( xintTbl[xint].vecNo );
				sysIntDisconnect( xintTbl[xint].vecNo );
				xintTbl[xint].bActive = 0;
			}
		}
	}
}

/*******************************************************************************
 *
 */
void attachInterrupt(int vecNo, void (*pFunc)(void) )
{
	sysIntConnect( vecNo, (void (*)(unsigned int))pFunc );
	sysIntEnable( vecNo );
}

/*******************************************************************************
 *
 */
void detachInterrupt(int vecNo)
{
	sysIntDisable( vecNo );
	sysIntDisconnect( vecNo );
}

/*** end of file : f_int.cpp ***/
