/*** f_stdlib.c - stdlib ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/05/21 mimu
              First Version
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "f_stdlib.h"

static const char strTbl[] = { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f', };
char * f_itoa( int val, char * pTop, int base )
{
	if( (base<=1) || (base>16) ) {
		*pTop = '\0';
	} else {
		switch( base ) {
		case 10:
			sprintf( pTop, "%d", val );
			break;
		case 16:
			sprintf( pTop, "%x", val );
			break;
		default:
			if( val ) {
				char buff[sizeof(int)*8]; /* when base=2, max string length */
				int bNeg;
				int len;
				char * p;

				if( val > 0 ) {
					bNeg = 0;
				} else {
					bNeg = 1;
					val = -val;
				}

				for( len=0; (len<32) && (val>0); len++) {
					buff[len] = strTbl[val % base];
					val /= base;
					len++;
				}

				p = pTop;
				if( bNeg ) {
					*p = '-';
					p++;
				}
				for( ; len>0; len--, p++) {
					*p = buff[len];
				}
				*p = '\0';
			} else {
				sprintf( pTop, "0" );
			}
		}
	}

	return pTop;
}

char * f_ultoa( unsigned long val, char * pTop, int base )
{
	if( (base<=1) || (base>16) ) {
		*pTop = '\0';
	} else {
		switch( base ) {
		case 10:
			sprintf( pTop, "%lu", val );
			break;
		case 16:
			sprintf( pTop, "%lx", val );
			break;
		default:
			if( val ) {
				char buff[sizeof(long)*8]; /* when base=2, max string length */
				int len;
				char * p;

				for( len=0; (len<32) && (val>0); len++) {
					buff[len] = strTbl[val % base];
					val /= base;
					len++;
				}

				p = pTop;
				for( ; len>0; len--, p++) {
					*p = buff[len];
				}
				*p = '\0';
			} else {
				sprintf( pTop, "0" );
			}
		}
	}

	return pTop;
}

char * f_ltoa( long val, char * pTop, int base )
{
	if( (base<=1) || (base>16) ) {
		*pTop = '\0';
	} else {
		switch( base ) {
		case 10:
			sprintf( pTop, "%d", val );
			break;
		case 16:
			sprintf( pTop, "%x", val );
			break;
		default:
			if( val ) {
				char buff[sizeof(long)*8]; /* when base=2, max string length */
				int bNeg;
				int len;
				char * p;

				if( val > 0 ) {
					bNeg = 0;
				} else {
					bNeg = 1;
					val = -val;
				}

				for( len=0; (len<32) && (val>0); len++) {
					buff[len] = strTbl[val % base];
					val /= base;
					len++;
				}

				p = pTop;
				if( bNeg ) {
					*p = '-';
					p++;
				}
				for( ; len>0; len--, p++) {
					*p = buff[len];
				}
				*p = '\0';
			} else {
				sprintf( pTop, "0" );
			}
		}
	}

	return pTop;
}

/*** end of file : f_stdlib.c ***/
