/*** f_time.c - fpalette time/timer manage ***/

/*** Copyright(C) 2011 Sansei System Co.,Ltd. ***/

/*** HISTORY ***
 * 2011/03/03 mimu
              Direct control cpu-timer
 * 2011/03/01 mimu
              First Version
 */

/*** NOTE ***/

/*** INCLUDE ***/

#include "config.h"
#include "sysLib.h"

//#include "tms320f28035/timerDrv.h"

#include "f_time.h"

/*** DEFINE ***/
#define	TIMER_CH	2
//#define	TIMER_FREQ	1000	/* 1000Hz : 1mSec */

#if (TIMER_CH == 0)
#define	TIMER_TIM  (volatile unsigned long *)0x0C00 /* CPU-Timer 0, Counter */
#define	TIMER_PRD  (volatile unsigned long *)0x0C02 /* CPU-Timer 0, Period */
#define	TIMER_TCR  (volatile unsigned long *)0x0C04 /* CPU-Timer 0, Control */
#define	TIMER_TPR  (volatile unsigned long *)0x0C06 /* CPU-Timer 0, Prescale */
#elif (TIMER_CH == 1)
#define	TIMER_TIM  (volatile unsigned long *)0x0C08 /* CPU-Timer 1, Counter */
#define	TIMER_PRD  (volatile unsigned long *)0x0C0a /* CPU-Timer 1, Period */
#define	TIMER_TCR  (volatile unsigned long *)0x0C0c /* CPU-Timer 1, Control */
#define	TIMER_TPR  (volatile unsigned long *)0x0C0e /* CPU-Timer 1, Prescale */
#elif (TIMER_CH == 2)
#define	TIMER_TIM  (volatile unsigned long *)0x0C10 /* CPU-Timer 0, Counter */
#define	TIMER_PRD  (volatile unsigned long *)0x0C12 /* CPU-Timer 0, Period */
#define	TIMER_TCR  (volatile unsigned long *)0x0C14 /* CPU-Timer 0, Control */
#define	TIMER_TPR  (volatile unsigned long *)0x0C16 /* CPU-Timer 0, Prescale */
#else
#error
#endif

#define	TCR_TIF	(1<<15)		/* CPU-Timer Interrupt Flag. */
#define	TCR_TIE	(1<<14)		/* TIE CPU-Timer Interrupt Enable. */
#define	TCR_TRB	(1<<5)		/* TRB CPU-Timer Reload bit. The TRB bit is always read as zero. Writes of 0 are ignored. */
#define	TCR_TSS	(1<<4)		/* TSS CPU-Timer stop status bit. TSS is a 1-bit flag that stops or starts the CPU-timer. */

#define	TPR_PSC(_reg)		((_reg)>>8)		/* 15-8 PSC CPU-Timer Prescale Counter. */
#define	TPR_TDDR(_reg)		((_reg)&0xff)	/* 7-0 TDDR CPU-Timer Divide-Down */

/*** FUNCTION ***/
//static void isr( void * arg );

/*** WORK ***/
//static fTime * pAlarm;
//static fTime   measure;

/*******************************************************************************
 *
 */
int f_time_init( void )
{
//	pAlarm = 0;

//	timer_sec = 0;

	*TIMER_TCR = TCR_TSS;	/* timer stop, disable interrupt */

	*TIMER_TPR = 60;		/* 60MHz/60 = 1MHz (1uSec) */
	*TIMER_PRD = 0xffffffff;
	*TIMER_TCR = TCR_TSS	/* timer stop */
	            | TCR_TRB	/* reload timer */
	            ;

	*TIMER_TCR = 0;	/* start timer */

	return 0;
}

/*******************************************************************************
 *
 */
unsigned long millis(void)
{
	unsigned long count;

	count = 0xffffffffUL - *TIMER_TIM;
	count /= 1000;

	return count;
}

/*******************************************************************************
 *
 */
unsigned long micros(void)
{
	unsigned long count;

	count = 0xffffffffUL - *TIMER_TIM;

	return count;
}

/*******************************************************************************
 *
 */
void timeAlarmSet( fTime * pTime )
{

	pTime->start = *TIMER_TIM;

}

/*******************************************************************************
 *
 */
int  timeAlarmCheck( fTime * pTime )
{
	unsigned long now;

	now = *TIMER_TIM;

	if( (pTime->start - now) >= pTime->uSec ) {
		return 1;
	}

	return 0;

//	return retVal;
}

/*******************************************************************************
 *
 */
void timeMeasureStart( fTime * pTime )
{
	pTime->start = *TIMER_TIM;
}

/*******************************************************************************
 *
 */
void timeMeasureGet( fTime * pTime )
{
	unsigned long now;

	now = *TIMER_TIM;

	pTime->uSec = pTime->start - now;

}


/*******************************************************************************
 *
 */
void delay( unsigned long mSec )
{
	unsigned long start, now;

	start = *TIMER_TIM;
	for( ; mSec > 0; ) {
		now = *TIMER_TIM;
		if( (start-now) >= 1000 ) {
			mSec--;
			start -= 1000;
		}
	}
}

/*******************************************************************************
 *
 */
void delayMicroseconds( unsigned int uSec )
{
	unsigned long start, now, delta;

	start = *TIMER_TIM;
	for( ; uSec > 0; ) {
		now = *TIMER_TIM;
		delta = start - now;
		if( delta > uSec ) {
			delta = uSec;
		}
		uSec -= delta;
		start = now;
	}
//	DELAY_US( uSec );
}

/*** end of file : f_time.c ***/
