
/*
 * intout
 *   GPIO input/output test
 */

/*
 * 39 : On Board LED
 *
 * Cable Connection
 * MODE = 0;
 * +------+--------++------+--------+
 * | GPIO |   CN   || GPIO |   CN   |
 * +------+--------++------+--------+
 * |  18  | CH1- 5 || 15   | CH1- 6 |
 * |  19  | CH1- 7 || 14   | CH1- 8 |
 * |  17  | CH1- 9 || 31   | CH1-10 |
 * |  16  | CH1-11 || 30   | CH1-12 |
 * |  24  | CH1-13 || 25   | CH1-14 |
 * |  34  | CH1-16 ||  0   | CH1-18 |
 * |   2  | CH1-20 ||  4   | CH1-22 |
 * |   6  | CH1-24 ||  8   | CH1-26 |
 * |  10  | CH1-28 || 40   | CH1-30 |
 * +------+--------++------+--------+
 *
 * MODE = 1;
 * +------+--------++------+--------+
 * | GPIO |   CN   || GPIO |   CN   |
 * +------+--------++------+--------+
 * |  41  | CH2- 1 || 11   | CH2- 3 |
 * |   9  | CH2- 5 ||  7   | CH2- 7 |
 * |   5  | CH2- 9 ||  3   | CH2-11 |
 * |   1  | CH2-13 || 44   | CH2-15 |
 * |  43  | CH2-17 || 42   | CH2-18 |
 * |  12  | CH2-20 || 13   | CH2-22 |
 * |  32  | CH2-23 || 27   | CH2-24 |
 * |  33  | CH2-25 || 26   | CH2-26 |
 * |  22  | CH2-27 || 23   | CH2-28 |
 * |  20  | CH2-29 || 21   | CH2-30 |
 * +------+--------++------+--------+
 */
#include "WProgram.h"

#define	MODE	1

static const struct gpio_t {
	int	p0;
	int	p1;
} gpio[] =
{
#if (MODE == 0)
	{ 18, 15, },
	{ 19, 14, },
	{ 17, 31, },
	{ 16, 30, },
	{ 24, 25, },
	{ 34,  0, },
	{  2,  4, },
	{  6,  8, },
	{ 10, 40, },
#elif (MODE == 1)
	{ 41, 11, },
	{  9,  7, },
	{  5,  3, },
	{  1, 44, },
	{ 43, 42, },
	{ 12, 13, },
	{ 32, 27, },
	{ 33, 26, },
	{ 22, 23, },
	{ 20, 21, },
#endif
};

#define	LED	39

#define	recodeof(_v)	(sizeof(_v)/sizeof(_v[0]))

static int count;

void test_error( void )
{
	for( ; ; ) {
		delay( 100 );
		digitalWrite( LED, TOGGLE );
	}
}

void setup()
{
	int i;

	pinMode( LED, OUTPUT );
	digitalWrite( LED, LOW );

	for( i=0; i<recodeof(gpio); i++ ) {
		pinMode( gpio[i].p0, INPUT );
		pinMode( gpio[i].p1, INPUT );
	}

	count = 0;
}

void loop()
{
	int in, out;
	int tmp;
	int i, j;

	digitalWrite( LED, TOGGLE );	// toggle LED (on<->off)

	in  = gpio[count].p0;
	out = gpio[count].p1;

	for( i=0; i<2; i++ ) {

		pinMode( out, OUTPUT );

		for( j=0; j<2; j++ ) {
			digitalWrite( out, HIGH );
			if( digitalRead( in ) != HIGH ) {
				test_error();
			}

			digitalWrite( out, LOW );
			if( digitalRead( in ) != LOW ) {
				test_error();
			}
		}

		pinMode( out, INPUT );

		tmp = in;
		in = out;
		out = tmp;
	}

	count++;
	if( count >= recodeof(gpio) ) {
		count = 0;
	}

	delay(1000);              // wait for a second
}